'=============================================================================
' Events.vb
'
' The event methods registered by the source methods
'
' Copyright (C) 2013 - 2013 Quartzdyne, Inc.  All rights reserved.
'
' Express written permission is hereby granted from Quartzdyne, Inc., to
' copy, modify, reproduce, and transmit this source code in any form and by
' any means, electronic or mechanical, including photocopying, recording, and
' any information storage and retrieval system as needed.  Permission is also
' hereby granted to use this source code in any form and means as needed.
' While every reasonable precaution has been taken in the preparation of this
' source code, Quartzdyne, Inc., assumes no responsibility for errors,
' omissions, or damages from the use of the source code contained herein.
'
' NEITHER QUARTZDYNE, INC., ANY MEMBER OF QUARTZDYNE, INC., NOR ANY PERSON OR
' ORGANIZATION ACTING ON BEHALF OF THEM MAKES ANY WARRANTY OR REPRESENTATION
' WHATSOEVER, EXPRESS OR IMPLIED, INCLUDING ANY WARRANTY OF MERCHANTABILITY
' OR FITNESS FOR ANY PURPOSE WITH RESPECT TO THE SOURCE CODE OR ASSUMES ANY
' LIABILITY WHATSOEVER WITH RESPECT TO ANY USE OF THE SOURCE CODE OR ANY
' PORTION THEREOF OR WITH RESPECT TO ANY DAMAGES THAT MAY RESULT FROM SUCH
' USE.
'
' Noji Ratzlaff
'
' See QCOM.log for a complete history of all updates, changes, and revisions
' of this source and other files and policies regarding QCOM software
'
' Updated 04-23-2013
'=============================================================================
Imports System.ComponentModel
Imports System.IO
'-----------------------------------------------------------------------------
' QCOM_Form
'
' A continuation of the QCOM_Form class
'-----------------------------------------------------------------------------
Partial Class QCOM_Form
    Inherits Form
    '-------------------------------------------------------------------------
    ' QCOM_AnyObjectMouseExited
    '
    ' Handles mouse moving away from the object
    '-------------------------------------------------------------------------
    Public Function QCOM_AnyObjectMouseExited(ByVal sender As Object, ByVal evt As EventArgs)
        QCOM_UpdateStatusLine("QCOM Ready")
        Return Nothing
    End Function                        ' QCOM_AnyObjectMouseExited
    '-------------------------------------------------------------------------
    ' QCOM_ComboBoxAcceptEnterKey
    '
    ' Handles the acceptance of the Enter key in a combo box
    '
    ' Called by:    QCOM_SetUpUnitMiscControls
    '-------------------------------------------------------------------------
    Public Function QCOM_ComboBoxAcceptEnterKey(ByVal sender As Object, ByVal evt As PreviewKeyDownEventArgs)
        If evt.KeyCode = Keys.Enter Then
            evt.IsInputKey = True
        End If
        Return Nothing
    End Function                        ' QCOM_ComboBoxAcceptEnterKey
    '-------------------------------------------------------------------------
    ' QCOM_DefaultExceptionHandler
    '
    ' Handles any exception not already handled Elsewhere in the program
    '-------------------------------------------------------------------------
    Public Function QCOM_DefaultExceptionHandler(ByVal sender As Object, ByVal evt As UnhandledExceptionEventArgs)
        Dim defaultException As Exception = evt.ExceptionObject
        '---------------------------------------------------------------------
        MsgBox( _
            String.Concat(defaultException.InnerException), _
            MsgBoxStyle.OkOnly Or MsgBoxStyle.Critical, _
            "QCOM Exception")
        '---------------------------------------------------------------------
        ' Clean up, then bail
        '---------------------------------------------------------------------
        QCOM_ShutDownSoftware()
        Return Nothing
    End Function                        ' QCOM_DefaultExceptionHandler
    '-------------------------------------------------------------------------
    ' QCOM_DescriptionLabelMouseEntered
    '
    ' Handles the mouse entering the unit description string label
    '
    ' Called by:    QCOM_ConstructUnitReadingsDisplay
    '-------------------------------------------------------------------------
    Public Function QCOM_DescriptionLabelMouseEntered(ByVal sender As Object, ByVal evt As EventArgs)
        Dim unitNumber As UInt32 = sender.Tag
        '---------------------------------------------------------------------
        QCOM_UpdateStatusLine( _
            String.Format( _
                "General description of the module and transducer on unit {0:D}", _
                unitNumber))
        Return Nothing
    End Function                        ' QCOM_DescriptionLabelMouseEntered
    '-------------------------------------------------------------------------
    ' QCOM_ElapsedTimeMouseEntered
    '
    ' Handles the mouse entering the elapsed sampling time label
    '
    ' Called by:    QCOM_InstallHomeWindowObjects
    '-------------------------------------------------------------------------
    Public Function QCOM_ElapsedTimeMouseEntered(ByVal sender As Object, ByVal evt As EventArgs)
        QCOM_UpdateStatusLine("Displays the amount of time that has elapsed since sampling started")
        Return Nothing
    End Function                        ' QCOM_ElapsedTimeMouseEntered
    '-------------------------------------------------------------------------
    ' QCOM_ExitButtonClicked
    '
    ' Handles the attempt to close the home window, effectively exiting the
    ' program
    '
    ' Called by:    QCOM_ConstructAndDisplayUserInterface
    '               QCOM_InstallToolStrip
    '-------------------------------------------------------------------------
    Public Function QCOM_ExitButtonClicked(ByVal sender As Object, ByVal evt As EventArgs)
        '---------------------------------------------------------------------
        ' Exit the home window (this actually forces a call to
        ' QCOM_HomeClosingWindow, which presents the user with a modal choice
        ' whether to remain running or proceed to exit the program)
        '---------------------------------------------------------------------
        Me.Close()
        Return Nothing
    End Function                        ' QCOM_ExitButtonClicked
    '-------------------------------------------------------------------------
    ' QCOM_ExitButtonMouseEntered
    '
    ' Handles the mouse entering the home or tool strip Exit button
    '
    ' Called by:    QCOM_ConstructAndDisplayUserInterface
    '               QCOM_InstallToolStrip
    '-------------------------------------------------------------------------
    Public Function QCOM_ExitButtonMouseEntered(ByVal sender As Object, ByVal evt As EventArgs)
        QCOM_UpdateStatusLine("Exits the program")
        Return Nothing
    End Function                        ' QCOM_ExitButtonMouseEntered
    '-------------------------------------------------------------------------
    ' QCOM_HelpCloseWindow
    '
    ' Handles closing the Help > About window
    '
    ' Called by:    QCOM_HelpAbout
    '-------------------------------------------------------------------------
    Public Function QCOM_HelpCloseWindow(ByVal sender As Object, ByVal evt As EventArgs)
        aboutHelpWindow.Close()
        Return Nothing
    End Function                        ' QCOM_HelpCloseWindow
    '-------------------------------------------------------------------------
    ' QCOM_HomeClosingWindow
    '
    ' Handles the closing of the home window
    '
    ' Called by:    QCOM_ConstructHomeWindow
    '-------------------------------------------------------------------------
    Public Sub QCOM_HomeClosingWindow(ByVal sender As Object, ByVal evt As FormClosingEventArgs)
        Dim proceedWithExit As MsgBoxResult
        '---------------------------------------------------------------------
        proceedWithExit = MsgBox( _
            "Proceed to Exit ?", _
            MsgBoxStyle.YesNo Or MsgBoxStyle.DefaultButton1 Or MsgBoxStyle.Question, _
            "Exit QCOM")
        If proceedWithExit = MsgBoxResult.Yes Then
            QCOM_GeneralInfo.flags = (QCOM_GeneralInfo.flags Or QCOM_GENERAL_PROGRAM_EXITING)
            '-----------------------------------------------------------------
            ' Shut down the software by releasing resources
            '-----------------------------------------------------------------
            QCOM_ShutDownSoftware()
            evt.Cancel = False
        Else
            evt.Cancel = True
        End If
    End Sub                             ' QCOM_HomeClosingWindow
    '-------------------------------------------------------------------------
    ' QCOM_InternetHyperlinkClicked
    '
    ' Handles the click of an internet link
    '
    ' Called by:    QCOM_HelpAbout
    '-------------------------------------------------------------------------
    Public Function QCOM_InternetHyperlinkClicked(ByVal sender As Object, ByVal evt As LinkLabelLinkClickedEventArgs)
        Dim hyperLink As LinkLabel = sender
        Dim target As String = evt.Link.LinkData
        '---------------------------------------------------------------------
        System.Diagnostics.Process.Start(target)
        hyperLink.Links(hyperLink.Links.IndexOf(evt.Link)).Visited = True
        Return Nothing
    End Function                        ' QCOM_InternetHyperlinkClicked
    '-------------------------------------------------------------------------
    ' QCOM_MiscControlsCloseWindow
    '
    ' Handles closing the Advanced > Miscellaneous window
    '
    ' Called by:    QCOM_SetUpMiscControls
    '-------------------------------------------------------------------------
    Public Function QCOM_MiscControlsCloseWindow(ByVal sender As Object, ByVal evt As EventArgs)
        miscControlsWindow.Hide()
        Return Nothing
    End Function                        ' QCOM_MiscControlsCloseWindow
    '-------------------------------------------------------------------------
    ' QCOM_MiscControlsClosingWindow
    '
    ' Handles closing the Advanced > Miscellaneous window by the red X
    '
    ' Called by:    QCOM_SetUpMiscControls
    '-------------------------------------------------------------------------
    Public Function QCOM_MiscControlsClosingWindow(ByVal sender As Object, ByVal evt As FormClosingEventArgs)
        evt.Cancel = True
        miscControlsWindow.Hide()
        Return Nothing
    End Function                        ' QCOM_MiscControlsClosingWindow
    '-------------------------------------------------------------------------
    ' QCOM_MiscEnableSoundsChecked
    '
    ' Handles the check of the misc Enable program sounds box
    '
    ' Called by:    QCOM_SetUpMiscControls
    '-------------------------------------------------------------------------
    Public Function QCOM_MiscEnableSoundsChecked(ByVal sender As Object, ByVal evt As EventArgs)
        If (QCOM_GeneralInfo.flags And QCOM_GENERAL_SOUNDS_ENABLED) <> 0 Then
            QCOM_GeneralInfo.flags = (QCOM_GeneralInfo.flags And Not QCOM_GENERAL_SOUNDS_ENABLED)
            miscEnableSoundsCheck.Checked = False
        Else
            QCOM_GeneralInfo.flags = (QCOM_GeneralInfo.flags Or QCOM_GENERAL_SOUNDS_ENABLED)
            miscEnableSoundsCheck.Checked = True
        End If                          ' If (QCOM_GeneralInfo.flags And QCOM_GENERAL_SOUNDS_ENABLED) <> 0 Then
        Return Nothing
    End Function                        ' QCOM_MiscEnableSoundsChecked
    '-------------------------------------------------------------------------
    ' QCOM_MiscGetTimeoutsButtonClicked
    '
    ' Handles the click of the misc Get timeouts button
    '
    ' Called by:    QCOM_SetUpMiscControls
    '-------------------------------------------------------------------------
    Public Function QCOM_MiscGetTimeoutsButtonClicked(ByVal sender As Object, ByVal evt As EventArgs)
        Dim readTimeout As UInt32 = 0
        Dim writeTimeout As UInt32 = 0
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        status = QD_GetTimeouts(readTimeout, writeTimeout)
        If status = QCOM_SUCCESS Then
            miscReadTimeoutBox.Text = String.Concat(readTimeout)
            miscWriteTimeoutBox.Text = String.Concat(writeTimeout)
        Else
            miscReadTimeoutBox.Clear()
            miscWriteTimeoutBox.Clear()
            EMsg("Error 0x{0} retrieving the timeout values", status.ToString("X8"))
        End If                          ' If status = QCOM_SUCCESS Then
        Return Nothing
    End Function                        ' QCOM_MiscGetTimeoutsButtonClicked
    '-------------------------------------------------------------------------
    ' QCOM_MiscSetTimeoutsButtonClicked
    '
    ' Handles the click of the misc Set timeouts button
    '
    ' Called by:    QCOM_SetUpMiscControls
    '-------------------------------------------------------------------------
    Public Function QCOM_MiscSetTimeoutsButtonClicked(ByVal sender As Object, ByVal evt As EventArgs)
        Dim readTimeout As UInt32 = 0
        Dim writeTimeout As UInt32 = 0
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        If (miscReadTimeoutBox.Text.Length <> 0) AndAlso (miscWriteTimeoutBox.Text.Length <> 0) Then
            readTimeout = Convert.ToInt32(miscReadTimeoutBox.Text)
            writeTimeout = Convert.ToInt32(miscWriteTimeoutBox.Text)
            status = QD_SetTimeouts(readTimeout, writeTimeout)
            If status <> QCOM_SUCCESS Then
                readTimeout = 0
                writeTimeout = 0
                status = QD_GetTimeouts(readTimeout, writeTimeout)
                miscReadTimeoutBox.Text = String.Concat(readTimeout)
                miscWriteTimeoutBox.Text = String.Concat(writeTimeout)
            End If                      ' If status = QCOM_SUCCESS Then
        Else
            EMsg("Both Read and Write timeout values must be specified")
        End If                          ' If (miscReadTimeoutBox.Text.Length <> 0) ...
        Return Nothing
    End Function                        ' QCOM_MiscSetTimeoutsButtonClicked
    '-------------------------------------------------------------------------
    ' QCOM_MiscUnitBootLoaderModeButtonClicked
    '
    ' Handles the click of the misc unit Boot Loader Mode button
    '
    ' Called by:    QCOM_SetUpUnitMiscControls
    '-------------------------------------------------------------------------
    Public Function QCOM_MiscUnitBootLoaderModeButtonClicked(ByVal sender As Object, ByVal evt As EventArgs)
        Dim unitNumber As UInt32 = sender.Tag
        Dim unit As UnitInfo = QCOM_UnitInfoArray(unitNumber)
        '---------------------------------------------------------------------
        If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
            miscBootLoaderModeButtonArray(unitNumber).Enabled = False
            If (unit.flags And QCOM_UNIT_BOOT_LOADER_MODE) <> 0 Then
                QCOM_SwitchFromBootLoaderMode(unit)
            Else
                QCOM_SwitchToBootLoaderMode(unit)
            End If
            If (unit.flags And QCOM_UNIT_BOOT_LOADER_MODE) <> 0 Then
                miscBootLoaderModeButtonArray(unitNumber).Text = QCOM_BLM_RESET_STRING
            Else
                miscBootLoaderModeButtonArray(unitNumber).Text = QCOM_BLM_ENTER_STRING
                '-------------------------------------------------------------
                ' The transition to Boot Loader Mode resets the IC data rate
                ' to the minimum, so restore the data rate
                '-------------------------------------------------------------
                QD_SetI2CDataRate(unit.unitHandle, unit.dataRate)
            End If
            miscBootLoaderModeButtonArray(unitNumber).Enabled = True
            miscTabPageArray(unitNumber).Text = unit.moduleSerialNumber
            QCOM_UnitInfoArray(unitNumber) = unit
        End If                          ' If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
        Return Nothing
    End Function                        ' QCOM_MiscUnitBootLoaderModeButtonClicked
    '-------------------------------------------------------------------------
    ' QCOM_MiscUnitClearI2CCommandBox
    '
    ' Handles the click of the misc unit IC command combo
    '
    ' Called by:    QCOM_SetUpUnitMiscControls
    '-------------------------------------------------------------------------
    Public Function QCOM_MiscUnitClearI2CCommandBox(ByVal sender As Object, ByVal evt As EventArgs)
        Dim unitNumber As UInt32 = sender.Tag
        Dim unit As UnitInfo = QCOM_UnitInfoArray(unitNumber)
        '---------------------------------------------------------------------
        If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
            miscI2CResponseBoxArray(unitNumber).Clear()
        End If                          ' If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
        Return Nothing
    End Function                        ' QCOM_MiscUnitClearI2CCommandBox
    '-------------------------------------------------------------------------
    ' GUI_MiscUnitGetControlRegisterButtonClicked
    '
    ' Handles the click of the misc unit Get control register button
    '
    ' Called by:    QCOM_SetUpUnitMiscControls
    '-------------------------------------------------------------------------
    Public Function GUI_MiscUnitGetControlRegisterButtonClicked(ByVal sender As Object, ByVal evt As EventArgs)
        Dim controlRegister As Byte = 0
        Dim unitNumber As UInt32 = sender.Tag
        Dim unit As UnitInfo = QCOM_UnitInfoArray(unitNumber)
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
            status = QD_GetModuleControlRegister(unit.unitHandle, controlRegister)
            If status = QCOM_SUCCESS Then
                miscGetSetControlRegisterBoxArray(unitNumber).Text =
                    String.Format("0x{0:X2}", controlRegister)
            End If                      ' If status = QCOM_SUCCESS Then
        End If                          ' If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
        Return Nothing
    End Function                        ' GUI_MiscUnitGetControlRegisterButtonClicked
    '-------------------------------------------------------------------------
    ' QCOM_MiscUnitGetI2CDataRateButtonClicked
    '
    ' Handles the click of the misc unit Get IC data rate button
    '
    ' Called by:    QCOM_SetUpUnitMiscControls
    '-------------------------------------------------------------------------
    Public Function QCOM_MiscUnitGetI2CDataRateButtonClicked(ByVal sender As Object, ByVal evt As EventArgs)
        Dim currentDataRate As Double = 0.0
        Dim unitNumber As UInt32 = sender.Tag
        Dim unit As UnitInfo = QCOM_UnitInfoArray(unitNumber)
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
            status = QD_GetI2CDataRate(unit.unitHandle, currentDataRate)
            If status = QCOM_SUCCESS Then
                miscGetSetI2CDataRateBoxArray(unitNumber).Text = _
                    String.Format("{0:F1}", currentDataRate)
            End If                      ' If status = QCOM_SUCCESS Then
        End If                          ' If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
        Return Nothing
    End Function                        ' QCOM_MiscUnitGetI2CDataRateButtonClicked
    '-------------------------------------------------------------------------
    ' QCOM_MiscUnitGetMemoryTypeButtonClicked
    '
    ' Handles the click of the misc unit Get memory type button
    '
    ' Called by:    QCOM_SetUpUnitMiscControls
    '-------------------------------------------------------------------------
    Public Function QCOM_MiscUnitGetMemoryTypeButtonClicked(ByVal sender As Object, ByVal evt As EventArgs)
        Dim device As Byte
        Dim memoryType As Byte = QD_MEMORY_TYPE_ABSENT
        Dim unitNumber As UInt32 = sender.Tag
        Dim unit As UnitInfo = QCOM_UnitInfoArray(unitNumber)
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
            If (unit.transducerType <> QD_TRANSDUCER_TYPE_ABSENT) AndAlso _
                (unit.transducerType <> QD_TRANSDUCER_TYPE_FREQUENCY) Then
                device = QD_DEVICE_TRANSDUCER
            Else
                device = QD_DEVICE_MODULE
            End If
            status = QD_GetMemoryType(unit.unitHandle, device, memoryType)
            If status = QCOM_SUCCESS Then
                miscGetMemoryTypeBoxArray(unitNumber).Text = _
                    String.Format("0x{0:X2}", memoryType)
            End If                      ' If status = QCOM_SUCCESS Then
        End If                          ' If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
        Return Nothing
    End Function                        ' QCOM_MiscUnitGetMemoryTypeButtonClicked
    '-------------------------------------------------------------------------
    ' GUI_MiscUnitGetModuleFirmwareIDButtonClicked
    '
    ' Handles the click of the misc unit Get module firmware ID button
    '
    ' Called by:    QCOM_SetUpUnitMiscControls
    '-------------------------------------------------------------------------
    Public Function GUI_MiscUnitGetModuleFirmwareIDButtonClicked(ByVal sender As Object, ByVal evt As EventArgs)
        Dim firmwareIDString(QD_FIRMWARE_ID_LENGTH) As Byte
        Dim unitNumber As UInt32 = sender.Tag
        Dim unit As UnitInfo = QCOM_UnitInfoArray(unitNumber)
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
            status = QD_GetModuleFirmwareID(unit.unitHandle, firmwareIDString(0))
            If status = QCOM_SUCCESS Then
                miscGetModuleFirmwareIDBoxArray(unitNumber).Text =
                    String.Format("{0:X2} {1:X2} {2:X2} {3:X2}", _
                        firmwareIDString(0), firmwareIDString(1), _
                        firmwareIDString(2), firmwareIDString(3))
            End If                      ' If status = QCOM_SUCCESS Then
        End If                          ' If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
        Return Nothing
    End Function                        ' GUI_MiscUnitGetModuleFirmwareIDButtonClicked
    '-------------------------------------------------------------------------
    ' QCOM_MiscUnitGetStatusRegisterButtonClicked
    '
    ' Handles the click of the misc unit Get status register button
    '
    ' Called by:    QCOM_SetUpUnitMiscControls
    '-------------------------------------------------------------------------
    Public Function QCOM_MiscUnitGetStatusRegisterButtonClicked(ByVal sender As Object, ByVal evt As EventArgs)
        Dim statusRegister As Byte = 0
        Dim unitNumber As UInt32 = sender.Tag
        Dim unit As UnitInfo = QCOM_UnitInfoArray(unitNumber)
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
            status = QD_GetModuleStatusRegister(unit.unitHandle, statusRegister)
            If status = QCOM_SUCCESS Then
                miscGetStatusRegisterBoxArray(unitNumber).Text =
                    String.Format("0x{0:X2}", statusRegister)
            End If                      ' If status = QCOM_SUCCESS Then
        End If                          ' If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
        Return Nothing
    End Function                        ' QCOM_MiscUnitGetStatusRegisterButtonClicked
    '-------------------------------------------------------------------------
    ' GUI_MiscUnitGetTransducerChipIDButtonClicked
    '
    ' Handles the click of the misc unit Get transducer chip ID button
    '
    ' Called by:    QCOM_SetUpUnitMiscControls
    '-------------------------------------------------------------------------
    Public Function GUI_MiscUnitGetTransducerChipIDButtonClicked(ByVal sender As Object, ByVal evt As EventArgs)
        Dim commandString() As Byte
        Dim rawCommandString As String
        Dim replyString(QCOM_MAXIMUM_I2C_REPLY_STRING_SIZE) As Byte
        Dim transducerChipID(QD_TRANSDUCER_CHIP_ID_LENGTH) As Integer
        Dim unitNumber As UInt32 = sender.Tag
        Dim unit As UnitInfo = QCOM_UnitInfoArray(unitNumber)
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
            If (unit.transducerType <> QD_TRANSDUCER_TYPE_FREQUENCY) _
                AndAlso (unit.transducerType <> QD_TRANSDUCER_TYPE_DIGITAL_NOMEM) Then
                rawCommandString = "S9CR9DFFFFFFFFP"
            Else
                rawCommandString = "S98R99FFFFFFFFP"
            End If                      ' If (unit.transducerType <> QD_TRANSDUCER_TYPE_FREQUENCY) AndAlso ...
            commandString = QCOM_ConvertVBString(rawCommandString)
            status = QD_ExecuteI2CCommand( _
                unit.unitHandle, _
                commandString(0), _
                replyString(0))
            If status = QCOM_SUCCESS Then
                If (replyString(14) = 80) AndAlso (replyString(15) = 13) AndAlso (replyString(16) = 10) Then
                    For index As Integer = 0 To QD_TRANSDUCER_CHIP_ID_LENGTH - 1
                        Dim highByte As Integer = replyString(6 + (index * 2))
                        Dim lowByte As Integer = replyString(7 + (index * 2))
                        If (highByte >= 48) AndAlso (highByte <= 57) Then
                            highByte -= 48
                        Else
                            highByte -= 55
                        End If
                        If (lowByte >= 48) AndAlso (lowByte <= 57) Then
                            lowByte -= 48
                        Else
                            lowByte -= 55
                        End If
                        transducerChipID(index) = (highByte << 4) Or (lowByte)
                    Next                ' For index As Integer = 0 To QD_TRANSDUCER_CHIP_ID_LENGTH - 1
                End If                  ' If (replyString(14) = 80) AndAlso ...
                miscGetTransducerChipIDBoxArray(unitNumber).Text = String.Format( _
                    "{0:X2} {1:X2} {2:X2} {3:X2}", _
                    transducerChipID(0), transducerChipID(1), _
                    transducerChipID(2), transducerChipID(3))
            End If                      ' If status = QCOM_SUCCESS Then
        End If                          ' If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
        Return Nothing
    End Function                        ' GUI_MiscUnitGetTransducerChipIDButtonClicked
    '-------------------------------------------------------------------------
    ' GUI_MiscUnitGetTransducerTypeButtonClicked
    '
    ' Handles the click of the misc unit Get transducer type button
    '
    ' Called by:    QCOM_SetUpUnitMiscControls
    '-------------------------------------------------------------------------
    Public Function GUI_MiscUnitGetTransducerTypeButtonClicked(ByVal sender As Object, ByVal evt As EventArgs)
        Dim transducerType As Byte = 0
        Dim unitNumber As UInt32 = sender.Tag
        Dim unit As UnitInfo = QCOM_UnitInfoArray(unitNumber)
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
            status = QD_GetTransducerType(unit.unitHandle, transducerType)
            If status = QCOM_SUCCESS Then
                miscGetTransducerTypeBoxArray(unitNumber).Text =
                    String.Format("0x{0:X2}", transducerType)
            End If                      ' If status = QCOM_SUCCESS Then
        End If                          ' If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
        Return Nothing
    End Function                        ' GUI_MiscUnitGetTransducerTypeButtonClicked
    '-------------------------------------------------------------------------
    ' QCOM_MiscUnitImportCoefficientDataButtonClicked
    '
    ' Handles the click of the misc unit Import Coefficient Data button
    '
    ' Called by:    QCOM_SetUpUnitMiscControls
    '-------------------------------------------------------------------------
    Public Function QCOM_MiscUnitImportCoefficientDataButtonClicked(ByVal sender As Object, ByVal evt As EventArgs)
        Dim coefficientData(QD_COEFFICIENT_DATA_SIZE) As Byte
        Dim pathName() As Byte
        Dim unitNumber As UInt32 = sender.Tag
        Dim unit As UnitInfo = QCOM_UnitInfoArray(unitNumber)
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
            '-----------------------------------------------------------------
            ' Prompt to navigate for a filename
            '-----------------------------------------------------------------
            Dim readFile As New OpenFileDialog()
            readFile.Title = "Open Coefficient Data Hex File"
            If (unit.flags And QCOM_UNIT_COEFFICIENT_FILE_SPECIFIED) <> 0 Then
                readFile.InitialDirectory = Path.GetDirectoryName(unit.coefficientFilePath)
            Else
                readFile.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.Recent)
            End If
            readFile.DefaultExt = "hex"
            readFile.Filter = "Hex files (*.hex)|*.hex|All files (*.*)|*.*"
            readFile.FilterIndex = 1
            readFile.RestoreDirectory = True
            '-----------------------------------------------------------------
            ' The AutoUpgradeEnabled property must be disabled for Windows 7
            ' and later
            '-----------------------------------------------------------------
            readFile.AutoUpgradeEnabled = False
            If readFile.ShowDialog() = System.Windows.Forms.DialogResult.OK Then
                '-------------------------------------------------------------
                ' The tester has selected a filename
                '-------------------------------------------------------------
                If File.Exists(readFile.FileName) = True Then
                    '---------------------------------------------------------
                    ' The filename is that of an existing file, so read it in
                    '---------------------------------------------------------
                    unit.coefficientFilePath = readFile.FileName
                    unit.flags = (unit.flags Or QCOM_UNIT_COEFFICIENT_FILE_SPECIFIED)
                    pathName = QCOM_ConvertVBString(readFile.FileName)
                    status = QD_ReadCoefficientDataFromHexFile( _
                        pathName(0), _
                        coefficientData(0))
                    If status = QCOM_SUCCESS Then
                        '-----------------------------------------------------
                        ' Write the coefficient data to the first four pages
                        ' of module memory
                        '-----------------------------------------------------
                        For pageNumber As Byte = 0 To QD_NUMBER_OF_LOWER_QMEM_PAGES - 1
                            Dim interimStatus As UInt32
                            interimStatus = QD_WriteCoefficientDataToModulePage( _
                                unit.unitHandle, _
                                pageNumber, _
                                coefficientData(0))
                            If interimStatus <> QCOM_SUCCESS Then
                                EMsg("QD_WriteCoefficientDataToModulePage failed for page {0:D} with status 0x{1}", _
                                     pageNumber, interimStatus.ToString("X8"))
                                status = interimStatus
                            End If
                        Next            ' For pageNumber As Byte = 0 To QD_NUMBER_OF_LOWER_QMEM_PAGES - 1
                        If status = QCOM_SUCCESS Then
                            If (unit.flags And QCOM_UNIT_MODULE_SN_VALID) <> 0 Then
                                Msg("Coefficient data successfully written to module {0}", unit.moduleSerialNumber)
                            Else
                                Msg("Coefficient data successfully written to unit {0:D}", unitNumber)
                            End If
                            unit.flags = (unit.flags Or QCOM_UNIT_COEFFICIENTS_VERIFIED)
                            QCOM_ApplyCoefficientDataInfoToProgram(unit)
                            If (unit.transducerType <> QD_TRANSDUCER_TYPE_ABSENT) _
                                AndAlso (unit.transducerType <> QD_TRANSDUCER_TYPE_FREQUENCY) _
                                AndAlso (unit.transducerType <> QD_TRANSDUCER_TYPE_DIGITAL_NOMEM) Then
                                '---------------------------------------------
                                ' This is a digital transducer with memory, so
                                ' prompt to store the coefficient data there
                                ' as well
                                '---------------------------------------------
                                Dim chooseToStore As MsgBoxResult
                                chooseToStore = MsgBox( _
                                    "Store the coefficient data in transducer memory as well ?", _
                                    (MsgBoxStyle.YesNo Or MsgBoxStyle.DefaultButton1 Or MsgBoxStyle.Question), _
                                    "Commit Data to Digital Transducer Memory")
                                If chooseToStore = MsgBoxResult.Yes Then
                                    '-----------------------------------------
                                    ' Write the coefficient data to the first
                                    ' four pages of digital transducer memory
                                    '-----------------------------------------
                                    For pageNumber As Byte = 0 To QD_NUMBER_OF_LOWER_QMEM_PAGES - 1
                                        Dim interimStatus As UInt32
                                        interimStatus = QD_WriteCoefficientDataToTargetPage( _
                                            unit.unitHandle, _
                                            QD_DEVICE_TRANSDUCER, _
                                            pageNumber, _
                                            coefficientData(0))
                                        If interimStatus <> QCOM_SUCCESS Then
                                            EMsg("QD_WriteCoefficientDataToTargetPage failed on transducer {0} for page {1:D} with status 0x{2}", _
                                                 unit.transducerSerialNumber, _
                                                 pageNumber, _
                                                 interimStatus.ToString("X8"))
                                            status = interimStatus
                                        End If
                                    Next
                                    If status = QCOM_SUCCESS Then
                                        Msg("Coefficient data successfully committed to transducer {0}", _
                                            unit.transducerSerialNumber)
                                    End If
                                End If  ' If chooseToStore = MsgBoxResult.Yes Then
                            End If      ' If (unit.transducerType <> QD_TRANSDUCER_TYPE_ABSENT) ...
                            QCOM_UnitInfoArray(unitNumber) = unit
                        End If          ' If status = QCOM_SUCCESS Then
                    Else
                        Msg("QD_ReadCoefficientDataFromHexFile returned status 0x{0} for unit {1:D}", _
                            status.ToString("X8"), _
                            unitNumber)
                    End If              ' If status = QCOM_SUCCESS Then
                Else
                    Msg("File {0} is not found", readFile.FileName)
                End If                  ' If File.Exists(readFile.FileName) = True Then
            End If                      ' If readFile.ShowDialog() = System.Windows.Forms.DialogResult.OK Then
        End If                          ' If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
        Return Nothing
    End Function                        ' QCOM_MiscUnitImportCoefficientDataButtonClicked
    '-------------------------------------------------------------------------
    ' QCOM_MiscUnitProcessI2CCommandBoxEnterKey
    '
    ' Handles the press of the Enter key in the combo box
    '
    ' Called by:    QCOM_SetUpUnitMiscControls
    '-------------------------------------------------------------------------
    Public Function QCOM_MiscUnitProcessI2CCommandBoxEnterKey(ByVal sender As Object, ByVal evt As KeyEventArgs)
        Dim commandString() As Byte
        Dim rawCommandString As String
        Dim replyString(QCOM_MAXIMUM_I2C_REPLY_STRING_SIZE) As Byte
        Dim unitNumber As UInt32 = sender.Tag
        Dim unit As UnitInfo = QCOM_UnitInfoArray(unitNumber)
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
            If evt.KeyCode = Keys.Enter Then
                rawCommandString = miscSendI2CCommandComboArray(unitNumber).Text
                commandString = QCOM_ConvertVBString(rawCommandString)
                status = QD_ExecuteI2CCommand( _
                    unit.unitHandle, _
                    commandString(0), _
                    replyString(0))
                If status = QCOM_SUCCESS Then
                    miscI2CResponseBoxArray(unitNumber).Text = QCOM_ConvertCString(replyString)
                Else
                    miscI2CResponseBoxArray(unitNumber).Text = String.Format( _
                        "<Error 0x{0}>", status.ToString("X8"))
                End If                      ' If status = QCOM_SUCCESS Then
                '-------------------------------------------------------------
                ' The following line prevents the handler from reacting to
                ' this key press as though it was done in error (ding!)
                '-------------------------------------------------------------
                evt.SuppressKeyPress = True
            End If
        End If                          ' If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
        Return Nothing
    End Function                        ' QCOM_MiscUnitProcessI2CCommandBoxEnterKey
    '-------------------------------------------------------------------------
    ' GUI_MiscUnitReadCadenceTimerButtonClicked
    '
    ' Handles the click of the misc unit Read cadence timer button
    '
    ' Called by:    QCOM_SetUpUnitMiscControls
    '-------------------------------------------------------------------------
    Public Function GUI_MiscUnitReadCadenceTimerButtonClicked(ByVal sender As Object, ByVal evt As EventArgs)
        Dim cadenceTimer As UInt32 = 0
        Dim unitNumber As UInt32 = sender.Tag
        Dim unit As UnitInfo = QCOM_UnitInfoArray(unitNumber)
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
            status = QD_GetCadenceTimer(unit.unitHandle, cadenceTimer)
            If status = QCOM_SUCCESS Then
                miscGetSetCadenceTimerBoxArray(unitNumber).Text = _
                    String.Concat(cadenceTimer)
            End If                      ' If status = QCOM_SUCCESS Then
        End If                          ' If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
        Return Nothing
    End Function                        ' GUI_MiscUnitReadCadenceTimerButtonClicked
    '-------------------------------------------------------------------------
    ' QCOM_MiscUnitReadModuleSerialNumberButtonClicked
    '
    ' Handles the click of the misc unit Read module serial number button
    '
    ' Called by:    QCOM_SetUpUnitMiscControls
    '-------------------------------------------------------------------------
    Public Function QCOM_MiscUnitReadModuleSerialNumberButtonClicked(ByVal sender As Object, ByVal evt As EventArgs)
        Dim moduleSerialNumber(QD_MAXIMUM_SERIAL_NUMBER_SIZE) As Byte
        Dim unitNumber As UInt32 = sender.Tag
        Dim unit As UnitInfo = QCOM_UnitInfoArray(unitNumber)
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
            status = QD_GetModuleSerialNumber(unit.physicalUnitNumber, moduleSerialNumber(0))
            If status = QCOM_SUCCESS Then
                miscGetModuleSerialNumberBoxArray(unitNumber).Text = _
                    QCOM_ConvertCString(moduleSerialNumber)
            End If                      ' If status = QCOM_SUCCESS Then
        End If                          ' If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
        Return Nothing
    End Function                        ' QCOM_MiscUnitReadModuleSerialNumberButtonClicked
    '-------------------------------------------------------------------------
    ' QCOM_MiscUnitSendI2CCommandButtonClicked
    '
    ' Handles the click of the misc unit Send IC command button
    '
    ' Called by:    QCOM_SetUpUnitMiscControls
    '-------------------------------------------------------------------------
    Public Function QCOM_MiscUnitSendI2CCommandButtonClicked(ByVal sender As Object, ByVal evt As EventArgs)
        Dim commandString() As Byte
        Dim rawCommandString As String
        Dim replyString(QCOM_MAXIMUM_I2C_REPLY_STRING_SIZE) As Byte
        Dim unitNumber As UInt32 = sender.Tag
        Dim unit As UnitInfo = QCOM_UnitInfoArray(unitNumber)
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
            rawCommandString = miscSendI2CCommandComboArray(unitNumber).Text
            commandString = QCOM_ConvertVBString(rawCommandString)
            status = QD_ExecuteI2CCommand( _
                unit.unitHandle, _
                commandString(0), _
                replyString(0))
            If status = QCOM_SUCCESS Then
                If (unit.transducerChipID(2) >= 4) AndAlso (unit.transducerChipID(3) >= 2) Then
                    '---------------------------------------------------------
                    ' The ASIC is version 4.02 or later, so process checksums
                    ' for the appropriate commands
                    '---------------------------------------------------------
                    Dim command As String = QCOM_ConvertCString(commandString)
                    If command.Length >= 14 Then
                        Dim reply As String = QCOM_ConvertCString(replyString)
                        If command.StartsWith("S9DFFF") = True OrElse _
                            command.StartsWith("S9FFFF") = True OrElse _
                            command.StartsWith("S9CR9D") = True OrElse _
                            command.StartsWith("S99FFF") = True OrElse _
                            command.StartsWith("S9BFFF") = True OrElse _
                            command.StartsWith("S98R99") = True Then
                            Dim offset As Integer
                            If command.StartsWith("S9DFFF") = True OrElse _
                                command.StartsWith("S9FFFF") = True OrElse _
                                command.StartsWith("S99FFF") = True OrElse _
                                command.StartsWith("S9BFFF") = True Then
                                offset = 3
                            Else
                                offset = 6
                            End If      ' If command.Contains("S9D") = True OrElse ...
                            Dim payload(10) As Byte
                            For index As Integer = 0 To 9
                                payload(index) = replyString(index + offset)
                            Next
                            If QCOM_I2CChecksumVerified(payload) = False Then
                                EMsg("Bad IC checksum returned")
                            End If
                        End If          ' If command.Contains("S9D") = True OrElse ...
                    End If              ' If command.Length >= 14 Then
                End If                  ' If (unit.transducerChipID(2) >= 4) AndAlso ...
                miscI2CResponseBoxArray(unitNumber).Text = QCOM_ConvertCString(replyString)
            Else
                miscI2CResponseBoxArray(unitNumber).Text = String.Format( _
                    "<Error 0x{0}>", status.ToString("X8"))
            End If                      ' If status = QCOM_SUCCESS Then
        End If                          ' If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
        Return Nothing
    End Function                        ' QCOM_MiscUnitSendI2CCommandButtonClicked
    '-------------------------------------------------------------------------
    ' GUI_MiscUnitSetCadenceTimerButtonClicked
    '
    ' Handles the click of the misc unit Set cadence timer button
    '
    ' Called by:    QCOM_SetUpUnitMiscControls
    '-------------------------------------------------------------------------
    Public Function GUI_MiscUnitSetCadenceTimerButtonClicked(ByVal sender As Object, ByVal evt As EventArgs)
        Dim cadenceTimer As UInt32 = 0
        Dim unitNumber As UInt32 = sender.Tag
        Dim unit As UnitInfo = QCOM_UnitInfoArray(unitNumber)
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
            If miscGetSetCadenceTimerBoxArray(unitNumber).Text.Length <> 0 Then
                cadenceTimer = Convert.ToUInt32(miscGetSetCadenceTimerBoxArray(unitNumber).Text)
                status = QD_SetCadenceTimer(unit.unitHandle, cadenceTimer)
                If status = QCOM_SUCCESS Then
                    cadenceTimer = 0
                    status = QD_GetCadenceTimer(unit.unitHandle, cadenceTimer)
                    If status = QCOM_SUCCESS Then
                        miscGetSetCadenceTimerBoxArray(unitNumber).Text = _
                            String.Concat(cadenceTimer)
                    End If              ' If status = QCOM_SUCCESS Then
                End If                  ' If status = QCOM_SUCCESS Then
            End If                      ' If miscGetSetCadenceTimerBoxArray(unitNumber).Text.Length <> 0 Then
        End If                          ' If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
        Return Nothing
    End Function                        ' GUI_MiscUnitSetCadenceTimerButtonClicked
    '-------------------------------------------------------------------------
    ' GUI_MiscUnitSetControlRegisterButtonClicked
    '
    ' Handles the click of the misc unit Set control register button
    '
    ' Called by:    QCOM_SetUpUnitMiscControls
    '-------------------------------------------------------------------------
    Public Function GUI_MiscUnitSetControlRegisterButtonClicked(ByVal sender As Object, ByVal evt As EventArgs)
        Dim controlRegister As UInt32 = 0
        Dim unitNumber As UInt32 = sender.Tag
        Dim unit As UnitInfo = QCOM_UnitInfoArray(unitNumber)
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
            If miscGetSetControlRegisterBoxArray(unitNumber).Text.Length <> 0 Then
                controlRegister = Convert.ToUInt32(miscGetSetControlRegisterBoxArray(unitNumber).Text)
                status = QD_SetModuleControlRegister(unit.unitHandle, controlRegister)
                If status = QCOM_SUCCESS Then
                    controlRegister = 0
                    status = QD_GetModuleControlRegister(unit.unitHandle, controlRegister)
                    If status = QCOM_SUCCESS Then
                        miscGetSetControlRegisterBoxArray(unitNumber).Text = _
                            String.Concat(controlRegister)
                    End If              ' If status = QCOM_SUCCESS Then
                End If                  ' If status = QCOM_SUCCESS Then
            End If                      ' If miscGetSetCadenceTimerBoxArray(unitNumber).Text.Length <> 0 Then
        End If                          ' If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
        Return Nothing
    End Function                        ' GUI_MiscUnitSetControlRegisterButtonClicked
    '-------------------------------------------------------------------------
    ' QCOM_MiscUnitSetI2CDataRateButtonClicked
    '
    ' Handles the click of the misc unit Set IC data rate button
    '
    ' Called by:    QCOM_SetUpUnitMiscControls
    '-------------------------------------------------------------------------
    Public Function QCOM_MiscUnitSetI2CDataRateButtonClicked(ByVal sender As Object, ByVal evt As EventArgs)
        Dim currentDataRate As Double = 0.0
        Dim newDataRate As Double
        Dim unitNumber As UInt32 = sender.Tag
        Dim unit As UnitInfo = QCOM_UnitInfoArray(unitNumber)
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
            If miscGetSetI2CDataRateBoxArray(unitNumber).Text.Length <> 0 Then
                newDataRate = Convert.ToDouble(miscGetSetI2CDataRateBoxArray(unitNumber).Text)
                status = QD_SetI2CDataRate(unit.unitHandle, newDataRate)
                If status = QCOM_SUCCESS Then
                    currentDataRate = 0.0
                    status = QD_GetI2CDataRate(unit.unitHandle, currentDataRate)
                    If status = QCOM_SUCCESS Then
                        miscGetSetI2CDataRateBoxArray(unitNumber).Text = _
                            String.Format("{0:F1}", currentDataRate)
                    End If              ' If status = QCOM_SUCCESS Then
                End If                  ' If status = QCOM_SUCCESS Then
            End If                      ' If miscGetSetI2CDataRateBoxArray(unitNumber).Text.Length <> 0 Then
        End If                          ' If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
        Return Nothing
    End Function                        ' QCOM_MiscUnitSetI2CDataRateButtonClicked
    '-------------------------------------------------------------------------
    ' QCOM_MiscUnitTransducerPowerButtonClicked
    '
    ' Handles the click of the misc unit Disable Transducer Power button
    '
    ' Called by:    QCOM_SetUpUnitMiscControls
    '-------------------------------------------------------------------------
    Public Function QCOM_MiscUnitTransducerPowerButtonClicked(ByVal sender As Object, ByVal evt As EventArgs)
        Dim unitNumber As UInt32 = sender.Tag
        Dim unit As UnitInfo = QCOM_UnitInfoArray(unitNumber)
        '---------------------------------------------------------------------
        If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
            If (unit.flags And QCOM_UNIT_TRANSDUCER_POWER_ENABLED) <> 0 Then
                QD_SetTransducerPowerState( _
                    unit.unitHandle, _
                    QD_DISABLE_TRANSDUCER_POWER)
                unit.flags = (unit.flags And Not QCOM_UNIT_TRANSDUCER_POWER_ENABLED)
            Else
                QD_SetTransducerPowerState( _
                    unit.unitHandle, _
                    QD_ENABLE_TRANSDUCER_POWER)
                unit.flags = (unit.flags Or QCOM_UNIT_TRANSDUCER_POWER_ENABLED)
            End If
            If (unit.flags And QCOM_UNIT_TRANSDUCER_POWER_ENABLED) <> 0 Then
                miscTransducerPowerButtonArray(unitNumber).Text = QCOM_XD_POWER_DISABLE_STRING
            Else
                miscTransducerPowerButtonArray(unitNumber).Text = QCOM_XD_POWER_ENABLE_STRING
            End If
            QCOM_UnitInfoArray(unitNumber) = unit
        End If                          ' If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
        Return Nothing
    End Function                        ' QCOM_MiscUnitTransducerPowerButtonClicked
    '-------------------------------------------------------------------------
    ' QCOM_OneSecondTimerElapsed
    '
    ' Handles the lapsing of the one-second timer
    '
    ' Called by:    QCOM_InitializeGUIComponents
    '-------------------------------------------------------------------------
    Public Function QCOM_OneSecondTimerElapsed(ByVal sender As Object, ByVal evt As EventArgs)
        Dim dateTimes As DateTime = DateTime.Now
        '---------------------------------------------------------------------
        nowStatusLineTSSLabel.Text = String.Format( _
            "{0,9} {1:D2} {2} {3:D4} {4:D2}:{5:D2}:{6:D2}", _
            dateTimes.DayOfWeek, _
            dateTimes.Day, _
            QCOM_MonthStringArray(dateTimes.Month), _
            dateTimes.Year, _
            dateTimes.Hour, _
            dateTimes.Minute, _
            dateTimes.Second)
        If (QCOM_GeneralInfo.flags And QCOM_GENERAL_CURRENTLY_SAMPLING) <> 0 Then
            QCOM_UpdateElapsedTime()
        End If
        Return Nothing
    End Function                        ' QCOM_OneSecondTimerElapsed
    '-------------------------------------------------------------------------
    ' QCOM_OnlineHelpClicked
    '
    ' Handles the click of the Online Help link by invoking the default
    ' browser to display the QCOM online help
    '
    ' Called by:    QCOM_HelpAbout
    '               QCOM_InstallToolStrip
    '-------------------------------------------------------------------------
    Public Function QCOM_OnlineHelpClicked(ByVal sender As Object, ByVal evt As EventArgs)
        System.Diagnostics.Process.Start(QCOM_HELP_URL)
        Return Nothing
    End Function                        ' QCOM_OnlineHelpClicked
    '-------------------------------------------------------------------------
    ' QCOM_OpeningBannerMouseEntered
    '
    ' Handles the mouse entering the home opening banner
    '
    ' Called by:    QCOM_PrepareOpeningBanner
    '-------------------------------------------------------------------------
    Public Function QCOM_OpeningBannerMouseEntered(ByVal sender As Object, ByVal evt As EventArgs)
        QCOM_UpdateStatusLine("The number of physical QCOM modules and Quartzdyne transducers detected")
        Return Nothing
    End Function                        ' QCOM_OpeningBannerMouseEntered
    '-------------------------------------------------------------------------
    ' QCOM_PleaseWaitClosingWindow
    '
    ' Handles the closing of the Please wait window by the red X
    '
    ' Called by:    QCOM_InitializeProgramComponents
    '-------------------------------------------------------------------------
    Public Sub QCOM_PleaseWaitClosingWindow(ByVal sender As Object, ByVal evt As FormClosingEventArgs)
        evt.Cancel = True
        pleaseWaitWindow.Hide()
        pleaseWaitWindow.Update()
    End Sub                             ' QCOM_PleaseWaitClosingWindow
    '-------------------------------------------------------------------------
    ' QCOM_PressureCountAreaMouseEntered
    '
    ' Handles the mouse entering the unit pressure count area
    '
    ' Called by:    QCOM_ConstructUnitReadingsDisplay
    '-------------------------------------------------------------------------
    Public Function QCOM_PressureCountAreaMouseEntered(ByVal sender As Object, ByVal evt As EventArgs)
        Dim unitNumber As UInt32 = sender.Tag
        '---------------------------------------------------------------------
        QCOM_UpdateStatusLine( _
            String.Format( _
                "Most recent pressure reading in transition counts from the transducer on unit {0:D}", _
                unitNumber))
        Return Nothing
    End Function                        ' QCOM_PressureCountAreaMouseEntered
    '-------------------------------------------------------------------------
    ' QCOM_PressureFreqAreaMouseEntered
    '
    ' Handles the mouse entering the unit pressure frequency area
    '
    ' Called by:    QCOM_ConstructUnitReadingsDisplay
    '-------------------------------------------------------------------------
    Public Function QCOM_PressureFreqAreaMouseEntered(ByVal sender As Object, ByVal evt As EventArgs)
        Dim unitNumber As UInt32 = sender.Tag
        '---------------------------------------------------------------------
        QCOM_UpdateStatusLine( _
            String.Format( _
                "Most recent pressure reading in transition frequency from the transducer on unit {0:D}", _
                unitNumber))
        Return Nothing
    End Function                        ' QCOM_PressureFreqAreaMouseEntered
    '-------------------------------------------------------------------------
    ' QCOM_PressurePSIAreaMouseEntered
    '
    ' Handles the mouse entering the unit pressure PSI area
    '
    ' Called by:    QCOM_ConstructUnitReadingsDisplay
    '-------------------------------------------------------------------------
    Public Function QCOM_PressurePSIAreaMouseEntered(ByVal sender As Object, ByVal evt As EventArgs)
        Dim unitNumber As UInt32 = sender.Tag
        '---------------------------------------------------------------------
        QCOM_UpdateStatusLine( _
            String.Format( _
                "Most recent pressure reading in psi from the transducer on unit {0:D}", _
                unitNumber))
        Return Nothing
    End Function                        ' QCOM_PressurePSIAreaMouseEntered
    '-------------------------------------------------------------------------
    ' QCOM_ProgramInfoCloseWindow
    '
    ' Handles closing the Advanced > Program Information window
    '
    ' Called by:    QCOM_DisplayProgramInformation
    '-------------------------------------------------------------------------
    Public Function QCOM_ProgramInfoCloseWindow(ByVal sender As Object, ByVal evt As EventArgs)
        programInfoWindow.Close()
        Return Nothing
    End Function                        ' QCOM_ProgramInfoCloseWindow
    '-------------------------------------------------------------------------
    ' QCOM_SamplingIntervalAreaMouseEntered
    '
    ' Handles the mouse entering the sampling interval area objects
    '
    ' Called by:    QCOM_InstallSamplingIntervalEditArea
    '-------------------------------------------------------------------------
    Public Function QCOM_SamplingIntervalAreaMouseEntered(ByVal sender As Object, ByVal evt As EventArgs)
        QCOM_UpdateStatusLine("The amount of time between transducer readings")
        Return Nothing
    End Function                        ' QCOM_SamplingIntervalAreaMouseEntered
    '-------------------------------------------------------------------------
    ' QCOM_SamplingTimerElapsed
    '
    ' Handles the lapsing of the sampling timer by updating the transducer readings
    '
    ' Called by:    QCOM_InitializeGUIComponents
    '-------------------------------------------------------------------------
    Public Function QCOM_SamplingTimerElapsed(ByVal sender As Object, ByVal evt As EventArgs)
        Dim currentlySampling As Boolean = False
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        samplingTimer.Stop()
        If (QCOM_GeneralInfo.flags And QCOM_GENERAL_CURRENTLY_SAMPLING) <> 0 Then
            For Each unit As UnitInfo In QCOM_UnitInfoArray
                If (unit.flags And QCOM_UNIT_READY) = QCOM_UNIT_READY Then
                    status = QCOM_RetrieveTransducerReadings(unit)
                    If status = QCOM_SUCCESS Then
                        QCOM_UpdateReadout(unit)
                    Else
                        Exit For
                    End If              ' If status = QCOM_SUCCESS Then
                End If                  ' If (unit.flags And QCOM_UNIT_READY) = QCOM_UNIT_READY Then
            Next                        ' For Each unit As UnitInfo In QCOM_UnitInfoArray
            '-----------------------------------------------------------------
            ' Restart the timer only if there are no errors
            '-----------------------------------------------------------------
            If status = QCOM_SUCCESS Then
                samplingTimer.Start()
            End If
        End If                          ' If (QCOM_GeneralInfo.flags And QCOM_GENERAL_CURRENTLY_SAMPLING) <> 0 Then
        Return Nothing
    End Function                        ' QCOM_SamplingTimerElapsed
    '-------------------------------------------------------------------------
    ' QCOM_StartStopButtonClicked
    '
    ' Stops the transducer sampling if it is already running, or starts it if
    ' it is not, then sets the graphics appearances accordingly
    '
    ' Called by:    QCOM_InstallHomeWindowObjects
    '-------------------------------------------------------------------------
    Public Function QCOM_StartStopButtonClicked(ByVal sender As Object, ByVal evt As EventArgs)
        '---------------------------------------------------------------------
        If (QCOM_GeneralInfo.flags And QCOM_GENERAL_CURRENTLY_SAMPLING) <> 0 Then
            '-----------------------------------------------------------------
            ' Currently running, so stop sampling
            '-----------------------------------------------------------------
            QCOM_GeneralInfo.flags = (QCOM_GeneralInfo.flags And Not QCOM_GENERAL_CURRENTLY_SAMPLING)
            For unitNumber As UInt32 = 0 To QCOM_GeneralInfo.numberOfUnits - 1
                pressureBoxArray(unitNumber).BackColor = Color.Lavender
                temperatureBoxArray(unitNumber).BackColor = Color.Lavender
                pressureFreqBoxArray(unitNumber).BackColor = Color.Lavender
                temperatureFreqBoxArray(unitNumber).BackColor = Color.Lavender
                pressureCountBoxArray(unitNumber).BackColor = Color.Lavender
                temperatureCountBoxArray(unitNumber).BackColor = Color.Lavender
                voltageBoxArray(unitNumber).BackColor = Color.Lavender
                currentBoxArray(unitNumber).BackColor = Color.Lavender
            Next
            samplingTimer.Stop()
            startStopButton.Text = "Start"
            samplingIntervalBox.BackColor = Color.Cyan
            samplingIntervalBox.Enabled = True
            resetModuleTSButton.Enabled = True
        Else
            '-----------------------------------------------------------------
            ' Currently idle, so start sampling
            '-----------------------------------------------------------------
            QCOM_GeneralInfo.flags = (QCOM_GeneralInfo.flags Or QCOM_GENERAL_CURRENTLY_SAMPLING)
            For unitNumber As UInt32 = 0 To QCOM_GeneralInfo.numberOfUnits - 1
                pressureBoxArray(unitNumber).BackColor = Color.Cyan
                temperatureBoxArray(unitNumber).BackColor = Color.Cyan
                pressureFreqBoxArray(unitNumber).BackColor = Color.Cyan
                temperatureFreqBoxArray(unitNumber).BackColor = Color.Cyan
                pressureCountBoxArray(unitNumber).BackColor = Color.Cyan
                temperatureCountBoxArray(unitNumber).BackColor = Color.Cyan
                voltageBoxArray(unitNumber).BackColor = Color.Cyan
                currentBoxArray(unitNumber).BackColor = Color.Cyan
            Next
            samplingTimer.Start()
            QCOM_GeneralInfo.samplingStartTime = GetTickCount()
            startStopButton.Text = "Stop"
            samplingIntervalBox.BackColor = Color.Lavender
            samplingIntervalBox.Enabled = False
            resetModuleTSButton.Enabled = False
        End If
        Return Nothing
    End Function                        ' QCOM_StartStopButtonClicked
    '-------------------------------------------------------------------------
    ' QCOM_StartStopButtonMouseEntered
    '
    ' Handles the mouse entering the readout Start / Stop sampling button
    '
    ' Called by:    QCOM_InstallHomeWindowObjects
    '-------------------------------------------------------------------------
    Public Function QCOM_StartStopButtonMouseEntered(ByVal sender As Object, ByVal evt As EventArgs)
        If (QCOM_GeneralInfo.flags And QCOM_GENERAL_CURRENTLY_SAMPLING) <> 0 Then
            QCOM_UpdateStatusLine("Stops sampling of transducer data")
        Else
            QCOM_UpdateStatusLine("Starts sampling of transducer data")
        End If
        Return Nothing
    End Function                        ' QCOM_StartStopButtonMouseEntered
    '-------------------------------------------------------------------------
    ' QCOM_StatusStripDateTimeMouseEntered
    '
    ' Handles the mouse entering the status strip date-and-time section
    '
    ' Called by:    QCOM_InstallStatusStrip
    '-------------------------------------------------------------------------
    Public Function QCOM_StatusStripDateTimeMouseEntered(ByVal sender As Object, ByVal evt As EventArgs)
        QCOM_UpdateStatusLine("The current day, date, and time, updated every second")
        Return Nothing
    End Function                        ' QCOM_StatusStripDateTimeMouseEntered
    '-------------------------------------------------------------------------
    ' QCOM_StatusStripProgressBarMouseEntered
    '
    ' Handles the mouse entering the status strip progress bar section
    '
    ' Called by:    QCOM_InstallStatusStrip
    '-------------------------------------------------------------------------
    Public Function QCOM_StatusStripProgressBarMouseEntered(ByVal sender As Object, ByVal evt As EventArgs)
        QCOM_UpdateStatusLine("Progress bar for time-consuming operations")
        Return Nothing
    End Function                        ' QCOM_StatusStripProgressBarMouseEntered
    '-------------------------------------------------------------------------
    ' QCOM_StatusStripStatusLineMouseEntered
    '
    ' Handles the mouse entering the status strip status line section
    '
    ' Called by:    QCOM_InstallStatusStrip
    '-------------------------------------------------------------------------
    Public Function QCOM_StatusStripStatusLineMouseEntered(ByVal sender As Object, ByVal evt As EventArgs)
        QCOM_UpdateStatusLine("Function status or one-line help for objects moused over")
        Return Nothing
    End Function                        ' QCOM_StatusStripStatusLineMouseEntered
    '-------------------------------------------------------------------------
    ' QCOM_TemperatureCelsiusAreaMouseEntered
    '
    ' Handles the mouse entering the unit temperature Celsius area
    '
    ' Called by:    QCOM_ConstructUnitReadingsDisplay
    '-------------------------------------------------------------------------
    Public Function QCOM_TemperatureCelsiusAreaMouseEntered(ByVal sender As Object, ByVal evt As EventArgs)
        Dim unitNumber As UInt32 = sender.Tag
        '---------------------------------------------------------------------
        QCOM_UpdateStatusLine( _
            String.Format( _
                "Most recent temperature reading in Celsius from the transducer on unit {0:D}", _
                unitNumber))
        Return Nothing
    End Function                        ' QCOM_TemperatureCelsiusAreaMouseEntered
    '-------------------------------------------------------------------------
    ' QCOM_TemperatureCountAreaMouseEntered
    '
    ' Handles the mouse entering the unit temperature count area
    '
    ' Called by:    QCOM_ConstructUnitReadingsDisplay
    '-------------------------------------------------------------------------
    Public Function QCOM_TemperatureCountAreaMouseEntered(ByVal sender As Object, ByVal evt As EventArgs)
        Dim unitNumber As UInt32 = sender.Tag
        '---------------------------------------------------------------------
        QCOM_UpdateStatusLine( _
            String.Format( _
                "Most recent temperature reading in transition counts from the transducer on unit {0:D}", _
                unitNumber))
        Return Nothing
    End Function                        ' QCOM_TemperatureCountAreaMouseEntered
    '-------------------------------------------------------------------------
    ' QCOM_TemperatureFreqAreaMouseEntered
    '
    ' Handles the mouse entering the unit temperature frequency area
    '
    ' Called by:    QCOM_ConstructUnitReadingsDisplay
    '-------------------------------------------------------------------------
    Public Function QCOM_TemperatureFreqAreaMouseEntered(ByVal sender As Object, ByVal evt As EventArgs)
        Dim unitNumber As UInt32 = sender.Tag
        '---------------------------------------------------------------------
        QCOM_UpdateStatusLine( _
            String.Format( _
                "Most recent temperature reading in transition frequency from the transducer on unit {0:D}", _
                unitNumber))
        Return Nothing
    End Function                        ' QCOM_TemperatureFreqAreaMouseEntered
    '-------------------------------------------------------------------------
    ' QCOM_ToolStripAboutButtonClicked
    '
    ' Handles the click of the Help > About tool strip button
    '
    ' Called by:    QCOM_InstallToolStrip
    '-------------------------------------------------------------------------
    Public Function QCOM_ToolStripAboutButtonClicked(ByVal sender As Object, ByVal evt As EventArgs)
        QCOM_HelpAbout()
        Return Nothing
    End Function                        ' QCOM_ToolStripAboutButtonClicked
    '-------------------------------------------------------------------------
    ' QCOM_ToolStripAboutButtonMouseEntered
    '
    ' Handles the mouse entering the Help > About tool strip button
    '
    ' Called by:    QCOM_InstallToolStrip
    '-------------------------------------------------------------------------
    Public Function QCOM_ToolStripAboutButtonMouseEntered(ByVal sender As Object, ByVal evt As EventArgs)
        QCOM_UpdateStatusLine("Displays the program About help window")
        Return Nothing
    End Function                        ' QCOM_ToolStripAboutButtonMouseEntered
    '-------------------------------------------------------------------------
    ' QCOM_ToolStripAdvancedButtonMouseEntered
    '
    ' Handles the mouse entering the area over the Advanced tool strip menu
    '
    ' Called by:    QCOM_InstallToolStrip
    '-------------------------------------------------------------------------
    Public Function QCOM_ToolStripAdvancedButtonMouseEntered(ByVal sender As Object, ByVal evt As EventArgs)
        QCOM_UpdateStatusLine("Presents controls meant for the more experienced tester")
        advancedTSDDButton.ShowDropDown()
        Return Nothing
    End Function                        ' QCOM_ToolStripAdvancedButtonMouseEntered
    '-------------------------------------------------------------------------
    ' QCOM_ToolStripAdvancedButtonMouseMoved
    '
    ' Handles the mouse moving over the Advanced tool strip menu
    '
    ' Called by:    QCOM_InstallToolStrip
    '-------------------------------------------------------------------------
    Public Function QCOM_ToolStripAdvancedButtonMouseMoved(ByVal sender As Object, ByVal evt As MouseEventArgs)
        Dim previousVerticalPosition As Integer
        '---------------------------------------------------------------------
        previousVerticalPosition = (advancedTSDDButton.Tag And QCOM_DROP_DOWN_VERTICAL_POSITION_MASK)
        advancedTSDDButton.Tag = (evt.Y Or QCOM_DROP_DOWN_CLOSED)
        If (evt.Y > previousVerticalPosition) AndAlso (evt.X < (advancedTSDDButton.Width - 4)) Then
            advancedTSDDButton.Tag = (evt.Y Or QCOM_DROP_DOWN_OPENED)
        End If
        Return Nothing
    End Function                        ' QCOM_ToolStripAdvancedButtonMouseMoved
    '-------------------------------------------------------------------------
    ' QCOM_ToolStripButtonsMouseExited
    '
    ' Handles mousing away from any of the tool strip drop down buttons
    '
    ' Called by:    QCOM_InstallToolStrip
    '-------------------------------------------------------------------------
    Public Function QCOM_ToolStripButtonsMouseExited(ByVal sender As Object, ByVal evt As EventArgs)
        If (fileTSDDButton.Tag And QCOM_DROP_DOWN_DISPLAY_MASK) = QCOM_DROP_DOWN_CLOSED Then
            fileTSDDButton.HideDropDown()
        End If
        If (functionsTSDDButton.Tag And QCOM_DROP_DOWN_DISPLAY_MASK) = QCOM_DROP_DOWN_CLOSED Then
            functionsTSDDButton.HideDropDown()
        End If
        If (helpTSDDButton.Tag And QCOM_DROP_DOWN_DISPLAY_MASK) = QCOM_DROP_DOWN_CLOSED Then
            helpTSDDButton.HideDropDown()
        End If
        If (advancedTSDDButton.Tag And QCOM_DROP_DOWN_DISPLAY_MASK) = QCOM_DROP_DOWN_CLOSED Then
            advancedTSDDButton.HideDropDown()
        End If
        QCOM_UpdateStatusLine("QCOM Ready")
        Return Nothing
    End Function                        ' QCOM_ToolStripButtonsMouseExited
    '-------------------------------------------------------------------------
    ' QCOM_ToolStripConfigFileButtonClicked
    '
    ' Handles the click of the Don't Save Config File button
    '
    ' Called by:    QCOM_InstallToolStrip
    '-------------------------------------------------------------------------
    Public Function QCOM_ToolStripConfigFileButtonClicked(ByVal sender As Object, ByVal evt As EventArgs)
        If (QCOM_GeneralInfo.flags And QCOM_GENERAL_CONFIG_DONT_SAVE) <> 0 Then
            QCOM_GeneralInfo.flags = (QCOM_GeneralInfo.flags And Not QCOM_GENERAL_CONFIG_DONT_SAVE)
            configFileTSButton.Text = QCOM_DISABLE_CONFIG_SAVE_STRING
        Else
            QCOM_GeneralInfo.flags = (QCOM_GeneralInfo.flags Or QCOM_GENERAL_CONFIG_DONT_SAVE)
            configFileTSButton.Text = QCOM_ENABLE_CONFIG_SAVE_STRING
        End If
        Return Nothing
    End Function                        ' QCOM_ToolStripConfigFileButtonClicked
    '-------------------------------------------------------------------------
    ' QCOM_ToolStripConfigFileButtonMouseEntered
    '
    ' Handles the mouse entering the Don't Save Config File button
    '
    ' Called by:    QCOM_InstallToolStrip
    '-------------------------------------------------------------------------
    Public Function QCOM_ToolStripConfigFileButtonMouseEntered(ByVal sender As Object, ByVal evt As EventArgs)
        If (QCOM_GeneralInfo.flags And QCOM_GENERAL_CONFIG_DONT_SAVE) <> 0 Then
            QCOM_UpdateStatusLine("Allows the software to save the config file when you exit the program")
        Else
            QCOM_UpdateStatusLine("Prevents the software from saving the config file when you exit the program")
        End If
        Return Nothing
    End Function                        ' QCOM_ToolStripConfigFileButtonMouseEntered
    '-------------------------------------------------------------------------
    ' QCOM_ToolStripDropDownsMouseExited
    '
    ' Handles the event of mousing away from the Toolstrip drop-down menus
    '
    ' Called by:    QCOM_InstallToolStrip
    '-------------------------------------------------------------------------
    Public Function QCOM_ToolStripDropDownsMouseExited(ByVal sender As Object, ByVal evt As EventArgs)
        fileTSDDButton.HideDropDown()
        fileTSDDButton.Tag = QCOM_DROP_DOWN_CLOSED
        functionsTSDDButton.HideDropDown()
        functionsTSDDButton.Tag = QCOM_DROP_DOWN_CLOSED
        helpTSDDButton.HideDropDown()
        helpTSDDButton.Tag = QCOM_DROP_DOWN_CLOSED
        advancedTSDDButton.HideDropDown()
        advancedTSDDButton.Tag = QCOM_DROP_DOWN_CLOSED
        Return Nothing
    End Function                        ' QCOM_ToolStripDropDownsMouseExited
    '-------------------------------------------------------------------------
    ' QCOM_ToolStripFileDropDownMouseEntered
    '
    ' Handles the mouse entering the area over the File tool strip menu
    '
    ' Called by:    QCOM_InstallToolStrip
    '-------------------------------------------------------------------------
    Public Function QCOM_ToolStripFileDropDownMouseEntered(ByVal sender As Object, ByVal evt As EventArgs)
        QCOM_UpdateStatusLine("Presents file-related operations, plus an exit function")
        fileTSDDButton.ShowDropDown()
        Return Nothing
    End Function                        ' QCOM_ToolStripFileDropDownMouseEntered
    '-------------------------------------------------------------------------
    ' QCOM_ToolStripFileDropDownMouseMoved
    '
    ' Handles the mouse moving over the File tool strip menu
    '
    ' Called by:    QCOM_InstallToolStrip
    '-------------------------------------------------------------------------
    Public Function QCOM_ToolStripFileDropDownMouseMoved(ByVal sender As Object, ByVal evt As MouseEventArgs)
        Dim previousVerticalPosition As Integer
        '---------------------------------------------------------------------
        previousVerticalPosition = (fileTSDDButton.Tag And QCOM_DROP_DOWN_VERTICAL_POSITION_MASK)
        fileTSDDButton.Tag = (evt.Y Or QCOM_DROP_DOWN_CLOSED)
        If (evt.Y > previousVerticalPosition) AndAlso (evt.X < (fileTSDDButton.Width - 4)) Then
            fileTSDDButton.Tag = (evt.Y Or QCOM_DROP_DOWN_OPENED)
        End If
        Return Nothing
    End Function                        ' QCOM_ToolStripFileDropDownMouseMoved
    '-------------------------------------------------------------------------
    ' QCOM_ToolStripFunctionsDropDownMouseEntered
    '
    ' Handles the mouse entering the area over the Functions tool strip menu
    '
    ' Called by:    QCOM_InstallToolStrip
    '-------------------------------------------------------------------------
    Public Function QCOM_ToolStripFunctionsDropDownMouseEntered(ByVal sender As Object, ByVal evt As EventArgs)
        QCOM_UpdateStatusLine("Presents functions available to the software")
        functionsTSDDButton.ShowDropDown()
        Return Nothing
    End Function                        ' QCOM_ToolStripFunctionsDropDownMouseEntered
    '-------------------------------------------------------------------------
    ' QCOM_ToolStripFunctionsDropDownMouseMoved
    '
    ' Handles the mouse moving over the Functions tool strip menu
    '
    ' Called by:    QCOM_InstallToolStrip
    '-------------------------------------------------------------------------
    Public Function QCOM_ToolStripFunctionsDropDownMouseMoved(ByVal sender As Object, ByVal evt As MouseEventArgs)
        Dim previousVerticalPosition As Integer
        '---------------------------------------------------------------------
        previousVerticalPosition = (functionsTSDDButton.Tag And QCOM_DROP_DOWN_VERTICAL_POSITION_MASK)
        functionsTSDDButton.Tag = (evt.Y Or QCOM_DROP_DOWN_CLOSED)
        If (evt.Y > previousVerticalPosition) AndAlso (evt.X < (functionsTSDDButton.Width - 4)) Then
            functionsTSDDButton.Tag = (evt.Y Or QCOM_DROP_DOWN_OPENED)
        End If
        Return Nothing
    End Function                        ' QCOM_ToolStripFunctionsDropDownMouseMoved
    '-------------------------------------------------------------------------
    ' QCOM_ToolStripHelpDropDownMouseEntered
    '
    ' Handles the mouse entering the area over the Help tool strip menu
    '
    ' Called by:    QCOM_InstallToolStrip
    '-------------------------------------------------------------------------
    Public Function QCOM_ToolStripHelpDropDownMouseEntered(ByVal sender As Object, ByVal evt As EventArgs)
        QCOM_UpdateStatusLine("Presents help and other program-related information")
        helpTSDDButton.ShowDropDown()
        Return Nothing
    End Function                        ' QCOM_ToolStripHelpDropDownMouseEntered
    '-------------------------------------------------------------------------
    ' QCOM_ToolStripHelpDropDownMouseMoved
    '
    ' Handles the mouse moving over the Help tool strip menu
    '
    ' Called by:    QCOM_InstallToolStrip
    '-------------------------------------------------------------------------
    Public Function QCOM_ToolStripHelpDropDownMouseMoved(ByVal sender As Object, ByVal evt As MouseEventArgs)
        Dim previousVerticalPosition As Integer
        '---------------------------------------------------------------------
        previousVerticalPosition = (helpTSDDButton.Tag And QCOM_DROP_DOWN_VERTICAL_POSITION_MASK)
        helpTSDDButton.Tag = (evt.Y Or QCOM_DROP_DOWN_CLOSED)
        If (evt.Y > previousVerticalPosition) AndAlso (evt.X < (helpTSDDButton.Width - 4)) Then
            helpTSDDButton.Tag = (evt.Y Or QCOM_DROP_DOWN_OPENED)
        End If
        Return Nothing
    End Function                        ' QCOM_ToolStripHelpDropDownMouseMoved
    '-------------------------------------------------------------------------
    ' QCOM_ToolStripMiscControlsButtonClicked
    '
    ' Handles the click of the Advanced > Miscellaneous tool strip button
    '
    ' Called by:    QCOM_InstallToolStrip
    '-------------------------------------------------------------------------
    Public Function QCOM_ToolStripMiscControlsButtonClicked(ByVal sender As Object, ByVal evt As EventArgs)
        If miscControlsWindow.WindowState = FormWindowState.Minimized Then
            miscControlsWindow.WindowState = FormWindowState.Normal
        Else
            miscControlsWindow.Show()
        End If
        miscControlsWindow.BringToFront()
        If miscControlsWindow.CanFocus = True Then
            miscControlsWindow.Focus()
        End If
        If miscControlsWindow.CanSelect = True Then
            miscControlsWindow.Select()
        End If
        Return Nothing
    End Function                        ' QCOM_ToolStripMiscControlsButtonClicked
    '-------------------------------------------------------------------------
    ' QCOM_ToolStripMiscControlsButtonMouseEntered
    '
    ' Handles the mouse entering the Advanced > Miscellaneous tool strip button
    '
    ' Called by:    QCOM_InstallToolStrip
    '-------------------------------------------------------------------------
    Public Function QCOM_ToolStripMiscControlsButtonMouseEntered(ByVal sender As Object, ByVal evt As EventArgs)
        QCOM_UpdateStatusLine("Displays the miscellaneous controls window")
        Return Nothing
    End Function                        ' QCOM_ToolStripMiscControlsButtonMouseEntered
    '-------------------------------------------------------------------------
    ' QCOM_ToolStripOnlineHelpButtonMouseEntered
    '
    ' Handles the mouse entering the Help > Online Help Information tool strip
    ' button
    '
    ' Called by:    QCOM_InstallToolStrip
    '-------------------------------------------------------------------------
    Public Function QCOM_ToolStripOnlineHelpButtonMouseEntered(ByVal sender As Object, ByVal evt As EventArgs)
        QCOM_UpdateStatusLine("Opens the default browser to display the QCOM online help website")
        Return Nothing
    End Function                        ' QCOM_ToolStripOnlineHelpButtonMouseEntered
    '-------------------------------------------------------------------------
    ' QCOM_ToolStripProgramInfoButtonClicked
    '
    ' Handles the click of the Advanced > Program Information tool strip button
    '
    ' Called by:    QCOM_InstallToolStrip
    '-------------------------------------------------------------------------
    Public Function QCOM_ToolStripProgramInfoButtonClicked(ByVal sender As Object, ByVal evt As EventArgs)
        QCOM_DisplayProgramInformation()
        Return Nothing
    End Function                        ' QCOM_ToolStripProgramInfoButtonClicked
    '-------------------------------------------------------------------------
    ' QCOM_ToolStripProgramInfoButtonMouseEntered
    '
    ' Handles the mouse entering the Advanced > Program Information tool strip
    ' button
    '
    ' Called by:    QCOM_InstallToolStrip
    '-------------------------------------------------------------------------
    Public Function QCOM_ToolStripProgramInfoButtonMouseEntered(ByVal sender As Object, ByVal evt As EventArgs)
        QCOM_UpdateStatusLine("Displays the Program Information window")
        Return Nothing
    End Function                        ' QCOM_ToolStripProgramInfoButtonMouseEntered
    '-------------------------------------------------------------------------
    ' QCOM_ToolStripResetButtonClicked
    '
    ' Handles the click of the Functions > Reset tool strip button
    '
    ' Called by:    QCOM_InstallToolStrip
    '-------------------------------------------------------------------------
    Public Function QCOM_ToolStripResetButtonClicked(ByVal sender As Object, ByVal evt As EventArgs)
        QCOM_ResetModules()
        Return Nothing
    End Function                        ' QCOM_ToolStripResetButtonClicked
    '-------------------------------------------------------------------------
    ' QCOM_ToolStripResetButtonMouseEntered
    '
    ' Handles the mouse entering the area over the Functions > Reset button
    '
    ' Called by:    QCOM_InstallToolStrip
    '-------------------------------------------------------------------------
    Public Function QCOM_ToolStripResetButtonMouseEntered(ByVal sender As Object, ByVal evt As EventArgs)
        QCOM_UpdateStatusLine("Resets the FPGA electronics of all attached QCOM modules")
        Return Nothing
    End Function                        ' QCOM_ToolStripResetButtonMouseEntered
    '-------------------------------------------------------------------------
    ' QCOM_TransducerCurrentAreaMouseEntered
    '
    ' Handles the mouse entering the unit transducer current area
    '
    ' Called by:    QCOM_ConstructUnitReadingsDisplay
    '-------------------------------------------------------------------------
    Public Function QCOM_TransducerCurrentAreaMouseEntered(ByVal sender As Object, ByVal evt As EventArgs)
        Dim unitNumber As UInt32 = sender.Tag
        '---------------------------------------------------------------------
        QCOM_UpdateStatusLine( _
            String.Format( _
                "Most recent transducer current draw reading in milliAmperes on unit {0:D}", _
                unitNumber))
        Return Nothing
    End Function                        ' QCOM_TransducerCurrentAreaMouseEntered
    '-------------------------------------------------------------------------
    ' QCOM_TransducerVoltageAreaMouseEntered
    '
    ' Handles the mouse entering the unit transducer voltage area
    '
    ' Called by:    QCOM_ConstructUnitReadingsDisplay
    '-------------------------------------------------------------------------
    Public Function QCOM_TransducerVoltageAreaMouseEntered(ByVal sender As Object, ByVal evt As EventArgs)
        Dim unitNumber As UInt32 = sender.Tag
        '---------------------------------------------------------------------
        QCOM_UpdateStatusLine( _
            String.Format( _
                "Most recent transducer voltage reading in volts on unit {0:D}", _
                unitNumber))
        Return Nothing
    End Function                        ' QCOM_TransducerVoltageAreaMouseEntered
    '-------------------------------------------------------------------------
    ' QCOM_ValidateSamplingIntervalValue
    '
    ' Handles changes of the sample timer interval field; ensures that the
    ' integer value of the field falls within the appropriate limits
    '
    ' Called by:    QCOM_InstallSamplingIntervalEditArea
    '-------------------------------------------------------------------------
    Public Function QCOM_ValidateSamplingIntervalValue(ByVal sender As Object, ByVal evt As CancelEventArgs)
        Dim newInterval As Integer = 0
        '---------------------------------------------------------------------
        If (QCOM_GeneralInfo.flags And QCOM_GENERAL_CURRENTLY_SAMPLING) = 0 Then
            If samplingIntervalBox.Text.Length <> 0 Then
                newInterval = Convert.ToInt32(samplingIntervalBox.Text)
                If (newInterval >= QCOM_MINIMUM_SAMPLING_INTERVAL) AndAlso (newInterval <= QCOM_MAXIMUM_SAMPLING_INTERVAL) Then
                    If (newInterval <> QCOM_GeneralInfo.samplingInterval) Then
                        'Msg("The sampling interval has changed from {0:D} ms to {1:D} ms", QCOM_GeneralInfo.samplingInterval, newInterval)
                        QCOM_GeneralInfo.samplingInterval = newInterval
                        samplingTimer.Interval = QCOM_GeneralInfo.samplingInterval
                    End If              ' If (newInterval <> QCOM_GeneralInfo.samplingInterval) Then
                    evt.Cancel = False
                Else
                    EMsg("Invalid sampling interval value\n\nValid values are {0:D} to {1:D}, inclusive", _
                        QCOM_MINIMUM_SAMPLING_INTERVAL, QCOM_MAXIMUM_SAMPLING_INTERVAL)
                    evt.Cancel = True
                End If                  ' If (newInterval >= QCOM_MINIMUM_SAMPLING_INTERVAL) AndAlso ...
            End If                      ' If samplingIntervalBox.Text.Length <> 0 Then
        End If                          ' If (QCOM_GeneralInfo.flags And QCOM_GENERAL_CURRENTLY_SAMPLING) = 0 Then
        Return Nothing
    End Function                        ' QCOM_ValidateSamplingIntervalValue
End Class                               ' QCOM_Form
'=============================================================================
' End of Events.vb
'=============================================================================
