﻿'=============================================================================
' GUI.vb
'
' Defines the program GUI methods
'
' Copyright (C) 2013 - 2013 Quartzdyne, Inc.  All rights reserved.
'
' Express written permission is hereby granted from Quartzdyne, Inc., to
' copy, modify, reproduce, and transmit this source code in any form and by
' any means, electronic or mechanical, including photocopying, recording, and
' any information storage and retrieval system as needed.  Permission is also
' hereby granted to use this source code in any form and means as needed.
' While every reasonable precaution has been taken in the preparation of this
' source code, Quartzdyne, Inc., assumes no responsibility for errors,
' omissions, or damages from the use of the source code contained herein.
'
' NEITHER QUARTZDYNE, INC., ANY MEMBER OF QUARTZDYNE, INC., NOR ANY PERSON OR
' ORGANIZATION ACTING ON BEHALF OF THEM MAKES ANY WARRANTY OR REPRESENTATION
' WHATSOEVER, EXPRESS OR IMPLIED, INCLUDING ANY WARRANTY OF MERCHANTABILITY
' OR FITNESS FOR ANY PURPOSE WITH RESPECT TO THE SOURCE CODE OR ASSUMES ANY
' LIABILITY WHATSOEVER WITH RESPECT TO ANY USE OF THE SOURCE CODE OR ANY
' PORTION THEREOF OR WITH RESPECT TO ANY DAMAGES THAT MAY RESULT FROM SUCH
' USE.
'
' Noji Ratzlaff
'
' See QCOM.log for a complete history of all updates, changes, and revisions
' of this source and other files and policies regarding QCOM software
'
' Updated 04-23-2013
'=============================================================================
Imports System.IO
'-----------------------------------------------------------------------------
' QCOM_Form
'
' A continuation of the QCOM_Form class
'-----------------------------------------------------------------------------
Partial Class QCOM_Form
    Inherits Form
    '-------------------------------------------------------------------------
    ' QCOM_ConstructHomeWindow
    '
    ' Constructs the home window and its properties
    '
    ' Called by:    QCOM_ConstructAndDisplayUserInterface
    '-------------------------------------------------------------------------
    Public Function QCOM_ConstructHomeWindow()
        Me.MaximizeBox = False
        Me.Icon = QCOM_SoftwareIcon
        Me.BackgroundImage = greenMarbleBackground
        Me.Text = QCOM_PROGRAM_TITLE + " " + QCOM_PROGRAM_VER_STRING
        Me.Width = GUI_HOME_WINDOW_MIN_WIDTH
        Me.Height = GUI_HOME_WINDOW_MIN_HEIGHT + (QCOM_GeneralInfo.numberOfUnits * 100)
        '---------------------------------------------------------------------
        ' Register a method to handle the Closing event
        '---------------------------------------------------------------------
        AddHandler Me.FormClosing, AddressOf QCOM_HomeClosingWindow
        Return Nothing
    End Function                        ' QCOM_ConstructHomeWindow
    '-------------------------------------------------------------------------
    ' QCOM_ConstructUnitReadingsDisplay
    '
    ' Displays the unit description strings, pressure, temperature, and
    ' other transducer readings, with accompanying units
    '
    ' Called by:    QCOM_ConstructAndDisplayUserInterface
    '-------------------------------------------------------------------------
    Public Function QCOM_ConstructUnitReadingsDisplay()
        Dim verticalOffset As Integer = 80
        Dim unit As UnitInfo
        '---------------------------------------------------------------------
        If QCOM_GeneralInfo.numberOfUnits <> 0 Then
            For unitNumber As UInt32 = 0 To QCOM_GeneralInfo.numberOfUnits - 1
                unit = QCOM_UnitInfoArray(unitNumber)
                '-------------------------------------------------------------
                ' Display the description string for each unit / transducer
                '-------------------------------------------------------------
                Dim descriptionLabel As New Label
                descriptionLabel.Text = unit.unitDescriptionString
                descriptionLabel.Font = New Drawing.Font( _
                    FontFamily.GenericSansSerif, _
                    8.0F, _
                    FontStyle.Bold)
                descriptionLabel.Location = New Point(10, verticalOffset)
                descriptionLabel.Size = New Drawing.Size( _
                    GUI_HOME_WINDOW_MIN_WIDTH - 40, _
                    GUI_REGULAR_LABEL_HEIGHT)
                descriptionLabel.TextAlign = Drawing.ContentAlignment.MiddleLeft
                QCOM_SetUnitObjectInterfaceProperties(descriptionLabel, unitNumber)
                AddHandler descriptionLabel.MouseEnter, AddressOf QCOM_DescriptionLabelMouseEntered
                AddHandler descriptionLabel.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
                Me.Controls.Add(descriptionLabel)
                '-------------------------------------------------------------
                ' Display the readout display titles for each unit / transducer
                '-------------------------------------------------------------
                ' Pressure
                Dim pressureLabel As New Label
                pressureLabel.Text = "Pressure"
                pressureLabel.Location = New Point(30, descriptionLabel.Bottom + 10)
                pressureLabel.Size = New Drawing.Size( _
                    GUI_REGULAR_TEXT_BOX_WIDTH, _
                    GUI_REGULAR_LABEL_HEIGHT)
                pressureLabel.TextAlign = Drawing.ContentAlignment.MiddleCenter
                QCOM_SetUnitObjectInterfaceProperties(pressureLabel, unitNumber)
                AddHandler pressureLabel.MouseEnter, AddressOf QCOM_PressurePSIAreaMouseEntered
                AddHandler pressureLabel.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
                Me.Controls.Add(pressureLabel)
                ' Temperature
                Dim temperatureLabel As New Label
                temperatureLabel.Text = "Temperature"
                temperatureLabel.Location = New Point( _
                    pressureLabel.Right + 20, _
                    pressureLabel.Top)
                temperatureLabel.Size = pressureLabel.Size
                temperatureLabel.TextAlign = Drawing.ContentAlignment.MiddleCenter
                QCOM_SetUnitObjectInterfaceProperties(temperatureLabel, unitNumber)
                AddHandler temperatureLabel.MouseEnter, AddressOf QCOM_TemperatureCelsiusAreaMouseEntered
                AddHandler temperatureLabel.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
                Me.Controls.Add(temperatureLabel)
                ' Pressure Count
                Dim pressureCountLabel As New Label
                pressureCountLabel.Text = "Pres Count"
                pressureCountLabel.Location = New Point( _
                    temperatureLabel.Right + 20, _
                    temperatureLabel.Top)
                pressureCountLabel.Size = temperatureLabel.Size
                pressureCountLabel.TextAlign = Drawing.ContentAlignment.MiddleCenter
                QCOM_SetUnitObjectInterfaceProperties(pressureCountLabel, unitNumber)
                AddHandler pressureCountLabel.MouseEnter, AddressOf QCOM_PressureCountAreaMouseEntered
                AddHandler pressureCountLabel.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
                Me.Controls.Add(pressureCountLabel)
                ' Temperature Count
                Dim temperatureCountLabel As New Label
                temperatureCountLabel.Text = "Temp Count"
                temperatureCountLabel.Location = New Point( _
                    pressureCountLabel.Right + 20, _
                    pressureCountLabel.Top)
                temperatureCountLabel.Size = pressureCountLabel.Size
                temperatureCountLabel.TextAlign = Drawing.ContentAlignment.MiddleCenter
                QCOM_SetUnitObjectInterfaceProperties(temperatureCountLabel, unitNumber)
                AddHandler temperatureCountLabel.MouseEnter, AddressOf QCOM_TemperatureCountAreaMouseEntered
                AddHandler temperatureCountLabel.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
                Me.Controls.Add(temperatureCountLabel)
                ' Pressure Frequency
                Dim pressureFreqLabel As New Label
                pressureFreqLabel.Text = "Pres Frequency"
                pressureFreqLabel.Location = New Point( _
                    temperatureCountLabel.Right + 20, _
                    temperatureCountLabel.Top)
                pressureFreqLabel.Size = temperatureCountLabel.Size
                pressureFreqLabel.TextAlign = Drawing.ContentAlignment.MiddleCenter
                QCOM_SetUnitObjectInterfaceProperties(pressureFreqLabel, unitNumber)
                AddHandler pressureFreqLabel.MouseEnter, AddressOf QCOM_PressureFreqAreaMouseEntered
                AddHandler pressureFreqLabel.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
                Me.Controls.Add(pressureFreqLabel)
                ' Temperature Frequency
                Dim temperatureFreqLabel As New Label
                temperatureFreqLabel.Text = "Temp Frequency"
                temperatureFreqLabel.Location = New Point( _
                    pressureFreqLabel.Right + 20, _
                    pressureFreqLabel.Top)
                temperatureFreqLabel.Size = pressureFreqLabel.Size
                temperatureFreqLabel.TextAlign = Drawing.ContentAlignment.MiddleCenter
                QCOM_SetUnitObjectInterfaceProperties(temperatureFreqLabel, unitNumber)
                AddHandler temperatureFreqLabel.MouseEnter, AddressOf QCOM_TemperatureFreqAreaMouseEntered
                AddHandler temperatureFreqLabel.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
                Me.Controls.Add(temperatureFreqLabel)
                ' Voltage
                Dim voltageLabel As New Label
                voltageLabel.Text = "Voltage"
                voltageLabel.Location = New Point( _
                    temperatureFreqLabel.Right + 20, _
                    temperatureFreqLabel.Top)
                voltageLabel.Size = temperatureFreqLabel.Size
                voltageLabel.TextAlign = Drawing.ContentAlignment.MiddleCenter
                QCOM_SetUnitObjectInterfaceProperties(voltageLabel, unitNumber)
                AddHandler voltageLabel.MouseEnter, AddressOf QCOM_TransducerVoltageAreaMouseEntered
                AddHandler voltageLabel.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
                Me.Controls.Add(voltageLabel)
                ' Current
                Dim currentLabel As New Label
                currentLabel.Text = "Current"
                currentLabel.Location = New Point( _
                    voltageLabel.Right + 20, _
                    voltageLabel.Top)
                currentLabel.Size = voltageLabel.Size
                currentLabel.TextAlign = Drawing.ContentAlignment.MiddleCenter
                QCOM_SetUnitObjectInterfaceProperties(currentLabel, unitNumber)
                AddHandler currentLabel.MouseEnter, AddressOf QCOM_TransducerCurrentAreaMouseEntered
                AddHandler currentLabel.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
                Me.Controls.Add(currentLabel)
                '-------------------------------------------------------------
                ' Display the readout display boxes for each unit / transducer
                '-------------------------------------------------------------
                ' Pressure
                Dim pressureBox As New TextBox
                pressureBox.Location = New Point(30, pressureLabel.Bottom + 6)
                pressureBox.Size = New Drawing.Size( _
                    GUI_REGULAR_TEXT_BOX_WIDTH, _
                    GUI_REGULAR_TEXT_BOX_HEIGHT)
                pressureBox.Multiline = False
                pressureBox.AcceptsReturn = False
                pressureBox.AcceptsTab = False
                pressureBox.WordWrap = False
                pressureBox.ReadOnly = True
                pressureBox.TabStop = False
                pressureBox.TextAlign = HorizontalAlignment.Center
                pressureBox.Cursor = Cursors.Default
                QCOM_SetUnitBoxInterfaceProperties(pressureBox, unitNumber)
                AddHandler pressureBox.MouseEnter, AddressOf QCOM_PressurePSIAreaMouseEntered
                AddHandler pressureBox.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
                Me.Controls.Add(pressureBox)
                pressureBoxArray(unitNumber) = pressureBox
                ' Temperature
                Dim temperatureBox As New TextBox
                temperatureBox.Location = New Point( _
                    pressureBox.Right + 20, _
                    pressureBox.Top)
                temperatureBox.Size = pressureBox.Size
                temperatureBox.Multiline = False
                temperatureBox.AcceptsReturn = False
                temperatureBox.AcceptsTab = False
                temperatureBox.WordWrap = False
                temperatureBox.ReadOnly = True
                temperatureBox.TabStop = False
                temperatureBox.TextAlign = HorizontalAlignment.Center
                temperatureBox.Cursor = Cursors.Default
                QCOM_SetUnitBoxInterfaceProperties(temperatureBox, unitNumber)
                AddHandler temperatureBox.MouseEnter, AddressOf QCOM_TemperatureCelsiusAreaMouseEntered
                AddHandler temperatureBox.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
                Me.Controls.Add(temperatureBox)
                temperatureBoxArray(unitNumber) = temperatureBox
                ' Pressure Count
                Dim pressureCountBox As New TextBox
                pressureCountBox.Location = New Point( _
                    temperatureBox.Right + 20, _
                    temperatureBox.Top)
                pressureCountBox.Size = temperatureBox.Size
                pressureCountBox.Multiline = False
                pressureCountBox.AcceptsReturn = False
                pressureCountBox.AcceptsTab = False
                pressureCountBox.WordWrap = False
                pressureCountBox.ReadOnly = True
                pressureCountBox.TabStop = False
                pressureCountBox.TextAlign = HorizontalAlignment.Center
                pressureCountBox.Cursor = Cursors.Default
                QCOM_SetUnitBoxInterfaceProperties(pressureCountBox, unitNumber)
                AddHandler pressureCountBox.MouseEnter, AddressOf QCOM_PressureCountAreaMouseEntered
                AddHandler pressureCountBox.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
                Me.Controls.Add(pressureCountBox)
                pressureCountBoxArray(unitNumber) = pressureCountBox
                ' Temperature Count
                Dim temperatureCountBox As New TextBox
                temperatureCountBox.Location = New Point( _
                    pressureCountBox.Right + 20, _
                    pressureCountBox.Top)
                temperatureCountBox.Size = pressureCountBox.Size
                temperatureCountBox.Multiline = False
                temperatureCountBox.AcceptsReturn = False
                temperatureCountBox.AcceptsTab = False
                temperatureCountBox.WordWrap = False
                temperatureCountBox.ReadOnly = True
                temperatureCountBox.TabStop = False
                temperatureCountBox.TextAlign = HorizontalAlignment.Center
                temperatureCountBox.Cursor = Cursors.Default
                QCOM_SetUnitBoxInterfaceProperties(temperatureCountBox, unitNumber)
                AddHandler temperatureCountBox.MouseEnter, AddressOf QCOM_TemperatureCountAreaMouseEntered
                AddHandler temperatureCountBox.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
                Me.Controls.Add(temperatureCountBox)
                temperatureCountBoxArray(unitNumber) = temperatureCountBox
                ' Pressure Frequency
                Dim pressureFreqBox As New TextBox
                pressureFreqBox.Location = New Point( _
                    temperatureCountBox.Right + 20, _
                    temperatureCountBox.Top)
                pressureFreqBox.Size = temperatureCountBox.Size
                pressureFreqBox.Multiline = False
                pressureFreqBox.AcceptsReturn = False
                pressureFreqBox.AcceptsTab = False
                pressureFreqBox.WordWrap = False
                pressureFreqBox.ReadOnly = True
                pressureFreqBox.TabStop = False
                pressureFreqBox.TextAlign = HorizontalAlignment.Center
                pressureFreqBox.Cursor = Cursors.Default
                QCOM_SetUnitBoxInterfaceProperties(pressureFreqBox, unitNumber)
                AddHandler pressureFreqBox.MouseEnter, AddressOf QCOM_PressureFreqAreaMouseEntered
                AddHandler pressureFreqBox.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
                Me.Controls.Add(pressureFreqBox)
                pressureFreqBoxArray(unitNumber) = pressureFreqBox
                ' Temperature Frequency
                Dim temperatureFreqBox As New TextBox
                temperatureFreqBox.Location = New Point( _
                    pressureFreqBox.Right + 20, _
                    pressureFreqBox.Top)
                temperatureFreqBox.Size = pressureFreqBox.Size
                temperatureFreqBox.Multiline = False
                temperatureFreqBox.AcceptsReturn = False
                temperatureFreqBox.AcceptsTab = False
                temperatureFreqBox.WordWrap = False
                temperatureFreqBox.ReadOnly = True
                temperatureFreqBox.TabStop = False
                temperatureFreqBox.TextAlign = HorizontalAlignment.Center
                temperatureFreqBox.Cursor = Cursors.Default
                QCOM_SetUnitBoxInterfaceProperties(temperatureFreqBox, unitNumber)
                AddHandler temperatureFreqBox.MouseEnter, AddressOf QCOM_TemperatureFreqAreaMouseEntered
                AddHandler temperatureFreqBox.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
                Me.Controls.Add(temperatureFreqBox)
                temperatureFreqBoxArray(unitNumber) = temperatureFreqBox
                ' Voltage
                Dim voltageBox As New TextBox
                voltageBox.Location = New Point( _
                    temperatureFreqBox.Right + 20, _
                    temperatureFreqBox.Top)
                voltageBox.Size = temperatureFreqBox.Size
                voltageBox.Multiline = False
                voltageBox.AcceptsReturn = False
                voltageBox.AcceptsTab = False
                voltageBox.WordWrap = False
                voltageBox.ReadOnly = True
                voltageBox.TabStop = False
                voltageBox.TextAlign = HorizontalAlignment.Center
                voltageBox.Cursor = Cursors.Default
                QCOM_SetUnitBoxInterfaceProperties(voltageBox, unitNumber)
                AddHandler voltageBox.MouseEnter, AddressOf QCOM_TransducerVoltageAreaMouseEntered
                AddHandler voltageBox.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
                Me.Controls.Add(voltageBox)
                voltageBoxArray(unitNumber) = voltageBox
                ' Current
                Dim currentBox As New TextBox
                currentBox.Location = New Point( _
                    voltageBox.Right + 20, _
                    voltageBox.Top)
                currentBox.Size = voltageBox.Size
                currentBox.Multiline = False
                currentBox.AcceptsReturn = False
                currentBox.AcceptsTab = False
                currentBox.WordWrap = False
                currentBox.ReadOnly = True
                currentBox.TabStop = False
                currentBox.TextAlign = HorizontalAlignment.Center
                currentBox.Cursor = Cursors.Default
                QCOM_SetUnitBoxInterfaceProperties(currentBox, unitNumber)
                AddHandler currentBox.MouseEnter, AddressOf QCOM_TransducerCurrentAreaMouseEntered
                AddHandler currentBox.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
                Me.Controls.Add(currentBox)
                currentBoxArray(unitNumber) = currentBox
                QCOM_UpdateReadout(unit)
                '-------------------------------------------------------------
                ' Display the readout display units for each unit / transducer
                '-------------------------------------------------------------
                ' Pressure
                Dim pressureUnitsLabel As New Label
                pressureUnitsLabel.Text = "psi"
                pressureUnitsLabel.Location = New Point(30, pressureBox.Bottom + 4)
                pressureUnitsLabel.Size = New Drawing.Size( _
                    GUI_REGULAR_TEXT_BOX_WIDTH, _
                    GUI_REGULAR_LABEL_HEIGHT)
                pressureUnitsLabel.TextAlign = Drawing.ContentAlignment.MiddleCenter
                QCOM_SetUnitObjectInterfaceProperties(pressureUnitsLabel, unitNumber)
                AddHandler pressureUnitsLabel.MouseEnter, AddressOf QCOM_PressurePSIAreaMouseEntered
                AddHandler pressureUnitsLabel.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
                Me.Controls.Add(pressureUnitsLabel)
                ' Temperature
                Dim temperatureUnitsLabel As New Label
                temperatureUnitsLabel.Text = "°C"
                temperatureUnitsLabel.Location = New Point( _
                    pressureUnitsLabel.Right + 20, _
                    pressureUnitsLabel.Top)
                temperatureUnitsLabel.Size = pressureUnitsLabel.Size
                temperatureUnitsLabel.TextAlign = Drawing.ContentAlignment.MiddleCenter
                QCOM_SetUnitObjectInterfaceProperties(temperatureUnitsLabel, unitNumber)
                AddHandler temperatureUnitsLabel.MouseEnter, AddressOf QCOM_TemperatureCelsiusAreaMouseEntered
                AddHandler temperatureUnitsLabel.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
                Me.Controls.Add(temperatureUnitsLabel)
                ' Pressure Count
                Dim pressureCountUnitsLabel As New Label
                pressureCountUnitsLabel.Text = "Transitions"
                pressureCountUnitsLabel.Location = New Point( _
                    temperatureUnitsLabel.Right + 20, _
                    temperatureUnitsLabel.Top)
                pressureCountUnitsLabel.Size = temperatureUnitsLabel.Size
                pressureCountUnitsLabel.TextAlign = Drawing.ContentAlignment.MiddleCenter
                QCOM_SetUnitObjectInterfaceProperties(pressureCountUnitsLabel, unitNumber)
                AddHandler pressureCountUnitsLabel.MouseEnter, AddressOf QCOM_PressureCountAreaMouseEntered
                AddHandler pressureCountUnitsLabel.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
                Me.Controls.Add(pressureCountUnitsLabel)
                ' Temperature Count
                Dim temperatureCountUnitsLabel As New Label
                temperatureCountUnitsLabel.Text = "Transitions"
                temperatureCountUnitsLabel.Location = New Point( _
                    pressureCountUnitsLabel.Right + 20, _
                    pressureCountUnitsLabel.Top)
                temperatureCountUnitsLabel.Size = pressureCountUnitsLabel.Size
                temperatureCountUnitsLabel.TextAlign = Drawing.ContentAlignment.MiddleCenter
                QCOM_SetUnitObjectInterfaceProperties(temperatureCountUnitsLabel, unitNumber)
                AddHandler temperatureCountUnitsLabel.MouseEnter, AddressOf QCOM_TemperatureCountAreaMouseEntered
                AddHandler temperatureCountUnitsLabel.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
                Me.Controls.Add(temperatureCountUnitsLabel)
                ' Pressure Frequency
                Dim pressureFreqUnitsLabel As New Label
                pressureFreqUnitsLabel.Text = "Hz"
                pressureFreqUnitsLabel.Location = New Point( _
                    temperatureCountUnitsLabel.Right + 20, _
                    temperatureCountUnitsLabel.Top)
                pressureFreqUnitsLabel.Size = temperatureCountUnitsLabel.Size
                pressureFreqUnitsLabel.TextAlign = Drawing.ContentAlignment.MiddleCenter
                QCOM_SetUnitObjectInterfaceProperties(pressureFreqUnitsLabel, unitNumber)
                AddHandler pressureFreqUnitsLabel.MouseEnter, AddressOf QCOM_PressureFreqAreaMouseEntered
                AddHandler pressureFreqUnitsLabel.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
                Me.Controls.Add(pressureFreqUnitsLabel)
                ' Temperature Frequency
                Dim temperatureFreqUnitsLabel As New Label
                temperatureFreqUnitsLabel.Text = "Hz"
                temperatureFreqUnitsLabel.Location = New Point( _
                    pressureFreqUnitsLabel.Right + 20, _
                    pressureFreqUnitsLabel.Top)
                temperatureFreqUnitsLabel.Size = pressureFreqUnitsLabel.Size
                temperatureFreqUnitsLabel.TextAlign = Drawing.ContentAlignment.MiddleCenter
                QCOM_SetUnitObjectInterfaceProperties(temperatureFreqUnitsLabel, unitNumber)
                AddHandler temperatureFreqUnitsLabel.MouseEnter, AddressOf QCOM_TemperatureFreqAreaMouseEntered
                AddHandler temperatureFreqUnitsLabel.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
                Me.Controls.Add(temperatureFreqUnitsLabel)
                ' Voltage
                Dim voltageUnitsLabel As New Label
                voltageUnitsLabel.Text = "V"
                voltageUnitsLabel.Location = New Point( _
                    temperatureFreqUnitsLabel.Right + 20, _
                    temperatureFreqUnitsLabel.Top)
                voltageUnitsLabel.Size = temperatureFreqUnitsLabel.Size
                voltageUnitsLabel.TextAlign = Drawing.ContentAlignment.MiddleCenter
                QCOM_SetUnitObjectInterfaceProperties(voltageUnitsLabel, unitNumber)
                AddHandler voltageUnitsLabel.MouseEnter, AddressOf QCOM_TransducerVoltageAreaMouseEntered
                AddHandler voltageUnitsLabel.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
                Me.Controls.Add(voltageUnitsLabel)
                ' Current
                Dim currentUnitsLabel As New Label
                currentUnitsLabel.Text = "mA"
                currentUnitsLabel.Location = New Point( _
                    voltageUnitsLabel.Right + 20, _
                    voltageUnitsLabel.Top)
                currentUnitsLabel.Size = voltageUnitsLabel.Size
                currentUnitsLabel.TextAlign = Drawing.ContentAlignment.MiddleCenter
                QCOM_SetUnitObjectInterfaceProperties(currentUnitsLabel, unitNumber)
                AddHandler currentUnitsLabel.MouseEnter, AddressOf QCOM_TransducerCurrentAreaMouseEntered
                AddHandler currentUnitsLabel.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
                Me.Controls.Add(currentUnitsLabel)
                verticalOffset += 100
            Next                        ' For unitNumber As UInt32 = 0 To QCOM_GeneralInfo.numberOfUnits - 1
            startStopButton.Enabled = True
        Else
            '-----------------------------------------------------------------
            ' Make the start / stop button unavailable if no units are present
            '-----------------------------------------------------------------
            startStopButton.Enabled = False
        End If                          ' If QCOM_GeneralInfo.numberOfUnits <> 0 Then
        Return Nothing
    End Function                        ' QCOM_ConstructUnitReadingsDisplay
    '-------------------------------------------------------------------------
    ' QCOM_ConstructAndDisplayUserInterface
    '
    ' Constructs the user interface for the main window by displaying the
    ' opening banner, transducer readings and units, a start / stop button,
    ' and an exit button, among other objects, in the home window
    '
    ' Called by:    QCOM_Start
    '-------------------------------------------------------------------------
    Public Function QCOM_ConstructAndDisplayUserInterface()
        '---------------------------------------------------------------------
        ' Set up the base components necessary to start the GUI
        '---------------------------------------------------------------------
        QCOM_InitializeGUIComponents()
        '---------------------------------------------------------------------
        ' Construct the visible home window objects
        '---------------------------------------------------------------------
        QCOM_ConstructHomeWindow()
        '---------------------------------------------------------------------
        ' Create the opening banner, displaying the number of QCOM modules and
        ' transducers detected
        '---------------------------------------------------------------------
        QCOM_PrepareOpeningBanner()
        '---------------------------------------------------------------------
        ' Install a user-selectable tool strip at the top of the window
        '---------------------------------------------------------------------
        QCOM_InstallToolStrip()
        '---------------------------------------------------------------------
        ' Install a status strip at the bottom of the window
        '---------------------------------------------------------------------
        QCOM_InstallStatusStrip()
        '---------------------------------------------------------------------
        ' Install the home window user objects
        '---------------------------------------------------------------------
        QCOM_InstallHomeWindowObjects()
        '---------------------------------------------------------------------
        ' Install the sampling interval text box and associated objects
        '---------------------------------------------------------------------
        QCOM_InstallSamplingIntervalEditArea()
        '---------------------------------------------------------------------
        ' Construct the transducer readings display
        '---------------------------------------------------------------------
        QCOM_ConstructUnitReadingsDisplay()
        '---------------------------------------------------------------------
        ' Set up the miscellanous controls window
        '---------------------------------------------------------------------
        QCOM_SetUpMiscControls()
        '---------------------------------------------------------------------
        ' Remove the "Please wait . . ." window
        '---------------------------------------------------------------------
        QCOM_PleaseWaitRemove()
        Return Nothing
    End Function                        ' QCOM_ConstructAndDisplayUserInterface
    '-------------------------------------------------------------------------
    ' QCOM_InitializeGUIComponents
    '
    ' Initializes the program entities prior to creating the home window
    '
    ' Called by:    QCOM_ConstructAndDisplayUserInterface
    '-------------------------------------------------------------------------
    Public Function QCOM_InitializeGUIComponents()
        '---------------------------------------------------------------------
        ' Initialize global objects
        '---------------------------------------------------------------------
        QCOM_MonthStringArray = New String() _
            {"Unk", "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"}
        '---------------------------------------------------------------------
        ' Set up the one-second (heartbeat) timer
        '
        ' This non-adjustable timer is for anything that needs a one-second
        ' interval
        '---------------------------------------------------------------------
        oneSecondTimer = New Windows.Forms.Timer
        oneSecondTimer.Interval = QCOM_ONE_SECOND_INTERVAL
        oneSecondTimer.Stop()
        AddHandler oneSecondTimer.Tick, AddressOf QCOM_OneSecondTimerElapsed
        '---------------------------------------------------------------------
        ' Set up the sampling timer
        '
        ' This adjustable timer is used to control the interval at which the
        ' program samples the transducers for data
        '---------------------------------------------------------------------
        samplingTimer = New Windows.Forms.Timer
        samplingTimer.Interval = QCOM_GeneralInfo.samplingInterval
        samplingTimer.Stop()
        AddHandler samplingTimer.Tick, AddressOf QCOM_SamplingTimerElapsed
        '---------------------------------------------------------------------
        ' Ensure the software has been properly initialized
        '---------------------------------------------------------------------
        QCOM_AffirmStartupConditions()
        Return Nothing
    End Function                        ' QCOM_InitializeGUIComponents
    '-------------------------------------------------------------------------
    ' QCOM_InstallHomeWindowObjects
    '
    ' Installs the necessary user objects in the home window
    '
    ' Called by:    QCOM_ConstructAndDisplayUserInterface
    '-------------------------------------------------------------------------
    Public Function QCOM_InstallHomeWindowObjects()
        '---------------------------------------------------------------------
        ' Start / stop button
        '---------------------------------------------------------------------
        startStopButton = New Button
        startStopButton.Text = "Start"
        startStopButton.Location = New Point(10, Me.Bottom - 100)
        startStopButton.Size = New Drawing.Size( _
            GUI_REGULAR_BUTTON_WIDTH, _
            GUI_REGULAR_BUTTON_HEIGHT)
        startStopButton.BackColor = Color.Transparent
        startStopButton.Cursor = Cursors.Hand
        startStopButton.Enabled = If(QCOM_GeneralInfo.numberOfUnits <> 0, True, False)
        AddHandler startStopButton.Click, AddressOf QCOM_StartStopButtonClicked
        AddHandler startStopButton.MouseEnter, AddressOf QCOM_StartStopButtonMouseEntered
        AddHandler startStopButton.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
        Me.Controls.Add(startStopButton)
        '---------------------------------------------------------------------
        ' Elapsed time label
        '---------------------------------------------------------------------
        elapsedTimeLabel = New Label()
        elapsedTimeLabel.Location = New Point( _
            Me.Right - 440, _
            startStopButton.Top + 4)
        elapsedTimeLabel.Size = New Drawing.Size(230, GUI_LARGE_LABEL_HEIGHT)
        elapsedTimeLabel.BackColor = Color.Transparent
        elapsedTimeLabel.TextAlign = Drawing.ContentAlignment.MiddleLeft
        AddHandler elapsedTimeLabel.MouseEnter, AddressOf QCOM_ElapsedTimeMouseEntered
        AddHandler elapsedTimeLabel.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
        Me.Controls.Add(elapsedTimeLabel)
        QCOM_GeneralInfo.samplingStartTime = GetTickCount()
        QCOM_UpdateElapsedTime()
        '---------------------------------------------------------------------
        ' Exit button
        '---------------------------------------------------------------------
        Dim exitButton As New Button
        exitButton.Text = "Exit"
        exitButton.Location = New Point( _
            Me.Right - 100, _
            Me.Bottom - 100)
        exitButton.Size = New Drawing.Size( _
            GUI_CLOSE_BUTTON_WIDTH, _
            GUI_REGULAR_BUTTON_HEIGHT)
        exitButton.BackColor = Color.Transparent
        exitButton.Cursor = Cursors.Hand
        AddHandler exitButton.Click, AddressOf QCOM_ExitButtonClicked
        AddHandler exitButton.MouseEnter, AddressOf QCOM_ExitButtonMouseEntered
        AddHandler exitButton.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
        Me.Controls.Add(exitButton)
        Me.AcceptButton = exitButton
        Me.CancelButton = exitButton
        Return Nothing
    End Function                        ' QCOM_InstallHomeWindowObjects
    '-------------------------------------------------------------------------
    ' QCOM_InstallSamplingIntervalEditArea
    '
    ' Installs the sampling interval text box and associated objects in the
    ' home window
    '
    ' Called by:    QCOM_ConstructAndDisplayUserInterface
    '-------------------------------------------------------------------------
    Public Function QCOM_InstallSamplingIntervalEditArea()
        '---------------------------------------------------------------------
        ' Sampling interval label
        '---------------------------------------------------------------------
        Dim samplingIntervalLabel As New Label
        samplingIntervalLabel.Text = "Sampling Interval :"
        samplingIntervalLabel.Location = New Point( _
            startStopButton.Right + 20, _
            startStopButton.Top + 6)
        samplingIntervalLabel.Size = New Drawing.Size(94, GUI_REGULAR_LABEL_HEIGHT)
        samplingIntervalLabel.BackColor = Color.Transparent
        AddHandler samplingIntervalLabel.MouseEnter, AddressOf QCOM_SamplingIntervalAreaMouseEntered
        AddHandler samplingIntervalLabel.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
        Me.Controls.Add(samplingIntervalLabel)
        '---------------------------------------------------------------------
        ' Sampling interval text box
        '---------------------------------------------------------------------
        samplingIntervalBox = New TextBox
        samplingIntervalBox.Text = String.Concat(QCOM_GeneralInfo.samplingInterval)
        samplingIntervalBox.Location = New Point( _
            samplingIntervalLabel.Right + 2, _
            samplingIntervalLabel.Top - 2)
        samplingIntervalBox.Size = New Drawing.Size(38, GUI_SMALL_TEXT_BOX_HEIGHT)
        samplingIntervalBox.Multiline = False
        samplingIntervalBox.AcceptsReturn = False
        samplingIntervalBox.AcceptsTab = False
        samplingIntervalBox.WordWrap = False
        samplingIntervalBox.ReadOnly = False
        samplingIntervalBox.TabStop = False
        samplingIntervalBox.BackColor = Color.Cyan
        samplingIntervalBox.TextAlign = HorizontalAlignment.Center
        samplingIntervalBox.Enabled = If(QCOM_GeneralInfo.numberOfUnits <> 0, True, False)
        AddHandler samplingIntervalBox.Validating, AddressOf QCOM_ValidateSamplingIntervalValue
        AddHandler samplingIntervalBox.MouseEnter, AddressOf QCOM_SamplingIntervalAreaMouseEntered
        AddHandler samplingIntervalBox.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
        Me.Controls.Add(samplingIntervalBox)
        '---------------------------------------------------------------------
        ' Sampling interval units label
        '---------------------------------------------------------------------
        Dim samplingIntervalUnitsLabel As New Label
        samplingIntervalUnitsLabel.Text = String.Format( _
            "ms ({0:D} to {1:D})", _
            QCOM_MINIMUM_SAMPLING_INTERVAL, _
            QCOM_MAXIMUM_SAMPLING_INTERVAL)
        samplingIntervalUnitsLabel.Location = New Point( _
            samplingIntervalBox.Right + 1, _
            samplingIntervalLabel.Top)
        samplingIntervalUnitsLabel.Size = New Drawing.Size(90, GUI_REGULAR_LABEL_HEIGHT)
        samplingIntervalUnitsLabel.BackColor = Color.Transparent
        AddHandler samplingIntervalUnitsLabel.MouseEnter, AddressOf QCOM_SamplingIntervalAreaMouseEntered
        AddHandler samplingIntervalUnitsLabel.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
        Me.Controls.Add(samplingIntervalUnitsLabel)
        Return Nothing
    End Function                        ' QCOM_InstallSamplingIntervalEditArea
    '-------------------------------------------------------------------------
    ' QCOM_InstallStatusStrip
    '
    ' Displays the status strip (bottom status bar) in the home window
    '
    ' Called by:    QCOM_ConstructAndDisplayUserInterface
    '-------------------------------------------------------------------------
    Public Function QCOM_InstallStatusStrip()
        Dim dateTimes As DateTime = DateTime.Now
        '---------------------------------------------------------------------
        ' Create the status strip and set its properties
        '---------------------------------------------------------------------
        homeStatusStrip = New StatusStrip
        homeStatusStrip.GripStyle = ToolStripGripStyle.Hidden
        homeStatusStrip.SizingGrip = False
        homeStatusStrip.LayoutStyle = ToolStripLayoutStyle.Table
        '---------------------------------------------------------------------
        ' Create the left (main) panel for status and mouse-over messages
        '---------------------------------------------------------------------
        homeStatusLineTSSLabel = New ToolStripStatusLabel
        homeStatusLineTSSLabel.Spring = True
        homeStatusLineTSSLabel.Alignment = ToolStripItemAlignment.Left
        homeStatusLineTSSLabel.BorderStyle = Border3DStyle.Sunken
        homeStatusLineTSSLabel.BorderSides = ToolStripStatusLabelBorderSides.All
        homeStatusLineTSSLabel.TextAlign = Drawing.ContentAlignment.MiddleLeft
        AddHandler homeStatusLineTSSLabel.MouseEnter, AddressOf QCOM_StatusStripStatusLineMouseEntered
        AddHandler homeStatusLineTSSLabel.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
        homeStatusStrip.Items.Add(homeStatusLineTSSLabel)
        '---------------------------------------------------------------------
        ' Create a separater line and the middle (progress bar) panel
        '---------------------------------------------------------------------
        Dim line1 As New ToolStripSeparator()
        line1.Width = 4
        homeStatusStrip.Items.Add(line1)
        homeTSProgressBar = New ToolStripProgressBar
        homeTSProgressBar.Minimum = 0
        homeTSProgressBar.Maximum = 100
        homeTSProgressBar.Style = ProgressBarStyle.Continuous
        AddHandler homeTSProgressBar.MouseEnter, AddressOf QCOM_StatusStripProgressBarMouseEntered
        AddHandler homeTSProgressBar.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
        homeStatusStrip.Items.Add(homeTSProgressBar)
        '---------------------------------------------------------------------
        ' Create a separater line and the right (time-of-day) panel
        '---------------------------------------------------------------------
        Dim line2 As New ToolStripSeparator()
        line2.Width = 4
        homeStatusStrip.Items.Add(line2)
        nowStatusLineTSSLabel = New ToolStripStatusLabel
        nowStatusLineTSSLabel.Alignment = ToolStripItemAlignment.Right
        nowStatusLineTSSLabel.BorderStyle = Border3DStyle.Sunken
        nowStatusLineTSSLabel.BorderSides = ToolStripStatusLabelBorderSides.All
        AddHandler nowStatusLineTSSLabel.MouseEnter, AddressOf QCOM_StatusStripDateTimeMouseEntered
        AddHandler nowStatusLineTSSLabel.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
        homeStatusStrip.Items.Add(nowStatusLineTSSLabel)
        '---------------------------------------------------------------------
        ' Add the constructed status strip to the main window
        '---------------------------------------------------------------------
        Me.Controls.Add(homeStatusStrip)
        QCOM_GeneralInfo.flags = (QCOM_GeneralInfo.flags Or QCOM_GENERAL_STATUS_LINE_AVAILABLE)
        '---------------------------------------------------------------------
        ' Initialize the appearance of the status strip
        '---------------------------------------------------------------------
        QCOM_UpdateStatusLine("Welcome to QCOM")
        QCOM_UpdateHomeProgressBar(0, 0)
        nowStatusLineTSSLabel.Text = String.Format( _
            "{0,9} {1:D2} {2} {3:D4} {4:D2}:{5:D2}:{6:D2}", _
            dateTimes.DayOfWeek, _
            dateTimes.Day, _
            QCOM_MonthStringArray(dateTimes.Month), _
            dateTimes.Year, _
            dateTimes.Hour, _
            dateTimes.Minute, _
            dateTimes.Second)
        oneSecondTimer.Start()
        Return Nothing
    End Function                        ' QCOM_InstallStatusStrip
    '-------------------------------------------------------------------------
    ' QCOM_InstallToolStrip
    '
    ' Displays the tool strip (tool bar) in the home window
    '
    ' Called by:    QCOM_ConstructAndDisplayUserInterface
    '-------------------------------------------------------------------------
    Public Function QCOM_InstallToolStrip()
        Dim toolStripBG As Bitmap
        '---------------------------------------------------------------------
        If File.Exists(QCOM_BG_TOOLSTRIP) = True Then
            toolStripBG = New Bitmap(QCOM_BG_TOOLSTRIP, True)
            '-----------------------------------------------------------------
            ' Create the tool strip and set its properties
            '-----------------------------------------------------------------
            Dim homeToolStrip As New ToolStrip()
            homeToolStrip.ShowItemToolTips = False
            homeToolStrip.AutoSize = True
            homeToolStrip.Size = New Drawing.Size(Me.Width, 25)
            homeToolStrip.BackgroundImage = toolStripBG
            '-----------------------------------------------------------------
            ' Set up the Exit drop-down 'button' for the File drop-down
            '-----------------------------------------------------------------
            configFileTSButton = New ToolStripButton(QCOM_DISABLE_CONFIG_SAVE_STRING)
            AddHandler configFileTSButton.Click, AddressOf QCOM_ToolStripConfigFileButtonClicked
            AddHandler configFileTSButton.MouseEnter, AddressOf QCOM_ToolStripConfigFileButtonMouseEntered
            AddHandler configFileTSButton.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
            '-----------------------------------------------------------------
            ' Set up the Exit drop-down 'button' for the File drop-down
            '-----------------------------------------------------------------
            Dim exitProgramTSButton As New ToolStripButton("Exit QCOM")
            AddHandler exitProgramTSButton.Click, AddressOf QCOM_ExitButtonClicked
            AddHandler exitProgramTSButton.MouseEnter, AddressOf QCOM_ExitButtonMouseEntered
            AddHandler exitProgramTSButton.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
            '-----------------------------------------------------------------
            ' Create the File toolstrip drop-down and set its properties
            '-----------------------------------------------------------------
            Dim fileDropDown As New ToolStripDropDown()
            fileDropDown.BackColor = Color.Wheat
            fileDropDown.ShowItemToolTips = False
            AddHandler fileDropDown.MouseLeave, AddressOf QCOM_ToolStripDropDownsMouseExited
            '-----------------------------------------------------------------
            ' Add the tool strip buttons to the File drop-down
            '-----------------------------------------------------------------
            Dim fileDropDownItems = New ToolStripItem() _
                {configFileTSButton, exitProgramTSButton}
            fileDropDown.Items.AddRange(fileDropDownItems)
            '-----------------------------------------------------------------
            ' Create the File toolstrip button as a drop-down
            '-----------------------------------------------------------------
            fileTSDDButton = New ToolStripDropDownButton("File")
            fileTSDDButton.Tag = QCOM_DROP_DOWN_CLOSED
            fileTSDDButton.DropDown = fileDropDown
            fileTSDDButton.ShowDropDownArrow = False
            AddHandler fileTSDDButton.MouseEnter, AddressOf QCOM_ToolStripFileDropDownMouseEntered
            AddHandler fileTSDDButton.MouseLeave, AddressOf QCOM_ToolStripButtonsMouseExited
            AddHandler fileTSDDButton.MouseMove, AddressOf QCOM_ToolStripFileDropDownMouseMoved
            '-----------------------------------------------------------------
            ' Set up the Reset drop-down 'button' for the Functions drop-down
            '-----------------------------------------------------------------
            resetModuleTSButton = New ToolStripButton("Reset QCOM")
            AddHandler resetModuleTSButton.Click, AddressOf QCOM_ToolStripResetButtonClicked
            AddHandler resetModuleTSButton.MouseEnter, AddressOf QCOM_ToolStripResetButtonMouseEntered
            AddHandler resetModuleTSButton.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
            '-----------------------------------------------------------------
            ' Create the Functions toolstrip drop-down and set its properties
            '-----------------------------------------------------------------
            Dim functionsDropDown As New ToolStripDropDown()
            functionsDropDown.BackColor = Color.Wheat
            functionsDropDown.ShowItemToolTips = False
            AddHandler functionsDropDown.MouseLeave, AddressOf QCOM_ToolStripDropDownsMouseExited
            '-----------------------------------------------------------------
            ' Add the tool strip buttons to the Functions drop-down
            '-----------------------------------------------------------------
            Dim functionsDropDownItems = New ToolStripItem() _
                {resetModuleTSButton}
            functionsDropDown.Items.AddRange(functionsDropDownItems)
            '-----------------------------------------------------------------
            ' Create the Functions toolstrip button as a drop-down
            '-----------------------------------------------------------------
            functionsTSDDButton = New ToolStripDropDownButton("Functions")
            functionsTSDDButton.Tag = QCOM_DROP_DOWN_CLOSED
            functionsTSDDButton.DropDown = functionsDropDown
            functionsTSDDButton.ShowDropDownArrow = False
            AddHandler functionsTSDDButton.MouseEnter, AddressOf QCOM_ToolStripFunctionsDropDownMouseEntered
            AddHandler functionsTSDDButton.MouseLeave, AddressOf QCOM_ToolStripButtonsMouseExited
            AddHandler functionsTSDDButton.MouseMove, AddressOf QCOM_ToolStripFunctionsDropDownMouseMoved
            '-----------------------------------------------------------------
            ' Set up the About drop-down 'button' for the Help drop-down
            '-----------------------------------------------------------------
            Dim aboutTSButton As New ToolStripButton("About QCOM")
            AddHandler aboutTSButton.Click, AddressOf QCOM_ToolStripAboutButtonClicked
            AddHandler aboutTSButton.MouseEnter, AddressOf QCOM_ToolStripAboutButtonMouseEntered
            AddHandler aboutTSButton.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
            '-----------------------------------------------------------------
            ' Set up the Online Help drop-down 'button' for the Help drop-down
            '-----------------------------------------------------------------
            Dim onlineHelpTSButton As New ToolStripButton("Online Help")
            AddHandler onlineHelpTSButton.Click, AddressOf QCOM_OnlineHelpClicked
            AddHandler onlineHelpTSButton.MouseEnter, AddressOf QCOM_ToolStripOnlineHelpButtonMouseEntered
            AddHandler onlineHelpTSButton.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
            '-----------------------------------------------------------------
            ' Create the Help toolstrip drop-down and set its properties
            '-----------------------------------------------------------------
            Dim helpDropDown As New ToolStripDropDown()
            helpDropDown.BackColor = Color.Wheat
            helpDropDown.ShowItemToolTips = False
            AddHandler helpDropDown.MouseLeave, AddressOf QCOM_ToolStripDropDownsMouseExited
            '-----------------------------------------------------------------
            ' Add the tool strip buttons to the Help drop-down
            '-----------------------------------------------------------------
            Dim helpDropDownItems = New ToolStripItem() _
                {aboutTSButton, onlineHelpTSButton}
            helpDropDown.Items.AddRange(helpDropDownItems)
            '-----------------------------------------------------------------
            ' Create the Help toolstrip button as a drop-down
            '-----------------------------------------------------------------
            helpTSDDButton = New ToolStripDropDownButton("Help")
            helpTSDDButton.Tag = QCOM_DROP_DOWN_CLOSED
            helpTSDDButton.DropDown = helpDropDown
            helpTSDDButton.ShowDropDownArrow = False
            AddHandler helpTSDDButton.MouseEnter, AddressOf QCOM_ToolStripHelpDropDownMouseEntered
            AddHandler helpTSDDButton.MouseLeave, AddressOf QCOM_ToolStripButtonsMouseExited
            AddHandler helpTSDDButton.MouseMove, AddressOf QCOM_ToolStripHelpDropDownMouseMoved
            '-----------------------------------------------------------------
            ' Set up the Program Information drop-down 'button' for the
            ' Advanced drop - down
            '-----------------------------------------------------------------
            Dim progInfoTSButton As New ToolStripButton("Program Information")
            AddHandler progInfoTSButton.Click, AddressOf QCOM_ToolStripProgramInfoButtonClicked
            AddHandler progInfoTSButton.MouseEnter, AddressOf QCOM_ToolStripProgramInfoButtonMouseEntered
            AddHandler progInfoTSButton.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
            '-----------------------------------------------------------------
            ' Set up the Misc drop-down 'button' for the Advanced drop-down
            '-----------------------------------------------------------------
            Dim miscProgramTSButton As New ToolStripButton("Miscellaneous Controls")
            AddHandler miscProgramTSButton.Click, AddressOf QCOM_ToolStripMiscControlsButtonClicked
            AddHandler miscProgramTSButton.MouseEnter, AddressOf QCOM_ToolStripMiscControlsButtonMouseEntered
            AddHandler miscProgramTSButton.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
            '-----------------------------------------------------------------
            ' Create the Advanced toolstrip drop-down and set its properties
            '-----------------------------------------------------------------
            Dim advancedDropDown As New ToolStripDropDown()
            advancedDropDown.BackColor = Color.Wheat
            advancedDropDown.ShowItemToolTips = False
            AddHandler advancedDropDown.MouseLeave, AddressOf QCOM_ToolStripDropDownsMouseExited
            '-----------------------------------------------------------------
            ' Add the tool strip buttons to the Advanced drop-down
            '-----------------------------------------------------------------
            Dim advancedDropDownItems = New ToolStripItem() _
                {progInfoTSButton, miscProgramTSButton}
            advancedDropDown.Items.AddRange(advancedDropDownItems)
            '-----------------------------------------------------------------
            ' Create the Advanced toolstrip button as a drop-down
            '-----------------------------------------------------------------
            advancedTSDDButton = New ToolStripDropDownButton("Advanced")
            advancedTSDDButton.Tag = QCOM_DROP_DOWN_CLOSED
            advancedTSDDButton.DropDown = advancedDropDown
            advancedTSDDButton.ShowDropDownArrow = False
            AddHandler advancedTSDDButton.MouseEnter, AddressOf QCOM_ToolStripAdvancedButtonMouseEntered
            AddHandler advancedTSDDButton.MouseLeave, AddressOf QCOM_ToolStripButtonsMouseExited
            AddHandler advancedTSDDButton.MouseMove, AddressOf QCOM_ToolStripAdvancedButtonMouseMoved
            '-----------------------------------------------------------------
            ' Attach the drop-down buttons to the tool strip
            '-----------------------------------------------------------------
            Dim toolStripButtons = New ToolStripDropDownButton() _
                {fileTSDDButton, functionsTSDDButton, helpTSDDButton, advancedTSDDButton}
            homeToolStrip.Items.AddRange(toolStripButtons)
            '-----------------------------------------------------------------
            ' Display the resulting tool strip
            '-----------------------------------------------------------------
            Me.Controls.Add(homeToolStrip)
        Else
            EMsg("The following graphical file is missing:\n\n{0}", _
                Path.GetFileName(QCOM_BG_TOOLSTRIP))
        End If
        Return Nothing
    End Function                        ' QCOM_InstallToolStrip
    '-------------------------------------------------------------------------
    ' QCOM_LoadImagesAndSounds
    '
    ' Initializes the background image pointers
    '
    ' Called by:    QCOM_InitializeProgramComponents
    '-------------------------------------------------------------------------
    Public Function QCOM_LoadImagesAndSounds()
        '---------------------------------------------------------------------
        ' Install images
        '---------------------------------------------------------------------
        If File.Exists(QCOM_PROGRAM_ICON) = True Then
            QCOM_SoftwareIcon = New Drawing.Icon(QCOM_PROGRAM_ICON)
        Else
            EMsg("The following graphical file is missing:\n\n{0}", _
                Path.GetFileName(QCOM_PROGRAM_ICON))
        End If
        If File.Exists(QCOM_BG_GREEN_MARBLE) = True Then
            greenMarbleBackground = New Bitmap(QCOM_BG_GREEN_MARBLE, True)
        Else
            EMsg("The following graphical file is missing:\n\n{0}", _
                Path.GetFileName(QCOM_BG_GREEN_MARBLE))
        End If
        If File.Exists(QCOM_BG_WHITE_SAND) = True Then
            whiteSandBackground = New Bitmap(QCOM_BG_WHITE_SAND, True)
        Else
            EMsg("The following graphical file is missing:\n\n{0}", _
                Path.GetFileName(QCOM_BG_WHITE_SAND))
        End If
        If File.Exists(QCOM_BG_WHITE_MARBLE) = True Then
            whiteMarbleBackground = New Bitmap(QCOM_BG_WHITE_MARBLE, True)
        Else
            EMsg("The following graphical file is missing:\n\n{0}", _
                Path.GetFileName(QCOM_BG_WHITE_MARBLE))
        End If
        '---------------------------------------------------------------------
        ' Install sounds
        '---------------------------------------------------------------------
        If File.Exists(QCOM_SOUND_ERROR) = True Then
            errorSound = New Media.SoundPlayer(QCOM_SOUND_ERROR)
            errorSound.Load()
        Else
            EMsg("The following sound file is missing:\n\n{0}", _
                Path.GetFileName(QCOM_SOUND_ERROR))
        End If
        Return Nothing
    End Function                        ' QCOM_LoadImagesAndSounds
    '-------------------------------------------------------------------------
    ' QCOM_PrepareOpeningBanner
    '
    ' Displays the opening banner (showing the number of modules and attached
    ' transducers) in the home window
    '
    ' Called by:    QCOM_ConstructAndDisplayUserInterface
    '-------------------------------------------------------------------------
    Public Function QCOM_PrepareOpeningBanner()
        Dim modulePlurality As String = String.Empty
        Dim transducerPlurality As String = String.Empty
        '---------------------------------------------------------------------
        If QCOM_GeneralInfo.numberOfUnits <> 1 Then
            modulePlurality = "s"
        End If
        If QCOM_GeneralInfo.numberOfTransducers <> 1 Then
            transducerPlurality = "s"
        End If
        Dim openingBannerLabel As New Label
        openingBannerLabel.Text = String.Concat( _
            QCOM_GeneralInfo.numberOfUnits) + " QCOM Module" + _
            modulePlurality + " and " + String.Concat(QCOM_GeneralInfo.numberOfTransducers) + _
            " Transducer" + transducerPlurality + " attached"
        openingBannerLabel.Location = New Point(10, 30)
        openingBannerLabel.Size = New Drawing.Size(GUI_HOME_WINDOW_MIN_WIDTH - 20, 40)
        openingBannerLabel.Font = New Drawing.Font( _
            FontFamily.GenericSansSerif, _
            20.0F, _
            FontStyle.Bold)
        openingBannerLabel.ForeColor = Color.Black
        openingBannerLabel.BackColor = Color.Transparent
        openingBannerLabel.TextAlign = Drawing.ContentAlignment.MiddleLeft
        AddHandler openingBannerLabel.MouseEnter, AddressOf QCOM_OpeningBannerMouseEntered
        AddHandler openingBannerLabel.MouseLeave, AddressOf QCOM_AnyObjectMouseExited
        Me.Controls.Add(openingBannerLabel)
        Return Nothing
    End Function                        ' QCOM_PrepareOpeningBanner
End Class                               ' QCOM_Form
'=============================================================================
' End of GUI.vb
'=============================================================================
