﻿'=============================================================================
' QCOM.vb
'
' Defines the program startup and pre-GUI methods
'
' Copyright (C) 2013 - 2013 Quartzdyne, Inc.  All rights reserved.
'
' Express written permission is hereby granted from Quartzdyne, Inc., to
' copy, modify, reproduce, and transmit this source code in any form and by
' any means, electronic or mechanical, including photocopying, recording, and
' any information storage and retrieval system as needed.  Permission is also
' hereby granted to use this source code in any form and means as needed.
' While every reasonable precaution has been taken in the preparation of this
' source code, Quartzdyne, Inc., assumes no responsibility for errors,
' omissions, or damages from the use of the source code contained herein.
'
' NEITHER QUARTZDYNE, INC., ANY MEMBER OF QUARTZDYNE, INC., NOR ANY PERSON OR
' ORGANIZATION ACTING ON BEHALF OF THEM MAKES ANY WARRANTY OR REPRESENTATION
' WHATSOEVER, EXPRESS OR IMPLIED, INCLUDING ANY WARRANTY OF MERCHANTABILITY
' OR FITNESS FOR ANY PURPOSE WITH RESPECT TO THE SOURCE CODE OR ASSUMES ANY
' LIABILITY WHATSOEVER WITH RESPECT TO ANY USE OF THE SOURCE CODE OR ANY
' PORTION THEREOF OR WITH RESPECT TO ANY DAMAGES THAT MAY RESULT FROM SUCH
' USE.
'
' Noji Ratzlaff
'
' See QCOM.log for a complete history of all updates, changes, and revisions
' of this source and other files and policies regarding QCOM software
'
' Updated 04-23-2013
'=============================================================================
Imports System.IO
'-----------------------------------------------------------------------------
' QCOM_Form
'
' A continuation of the QCOM_Form class
'-----------------------------------------------------------------------------
Partial Class QCOM_Form
    Inherits Form
    '-------------------------------------------------------------------------
    ' QCOM_Start
    '
    ' Starts the QCOM software
    '
    ' Called by:    New
    '-------------------------------------------------------------------------
    Public Function QCOM_Start()
        Dim mutexHandle As Int32 = OpenMutex(SYNCHRONIZE, True, QCOM_MUTEX_STRING)
        If mutexHandle <> Nothing Then
            Msg("QCOM is already running")
            Environment.Exit(EALREADY)  ' Operation already in progress
        Else
            mutexHandle = CreateMutex(Nothing, True, QCOM_MUTEX_STRING)
            Dim mainStatus As UInt32 = QCOM_InitializeQCOM()
            If mainStatus = QCOM_SUCCESS Then
                QCOM_ConstructAndDisplayUserInterface()
            End If
            ReleaseMutex(mutexHandle)
        End If
        Return (Nothing)
    End Function                        ' QCOM_Start
    '-------------------------------------------------------------------------
    ' QCOM_DriverInstalled
    '
    ' Determines whether the QCOM driver has been installed on the current
    ' machine
    '
    ' Returns: true    = The QCOM driver is installed
    '          false   = The QCOM driver can't be found or is not installed
    '
    ' Called by:    QCOM_EnsureMinimumEnvironment
    '-------------------------------------------------------------------------
    Public Function QCOM_DriverInstalled() As Boolean
        Dim driverInstalled As Boolean = False
        Dim lineString As String
        Dim pathString As String
        Dim textReader As StreamReader
        '---------------------------------------------------------------------
        For infNumber As Integer = 0 To 255
            pathString = String.Format("C:\Windows\inf\oem{0:D}.inf", infNumber)
            If File.Exists(pathString) = True Then
                textReader = File.OpenText(pathString)
                lineString = textReader.ReadLine()
                lineString = textReader.ReadLine()
                If lineString.Length >= 10 Then
                    If String.Compare(lineString.Substring(2, 8), "QCOM.inf", True) = 0 Then
                        driverInstalled = True
                        Exit For
                    End If
                End If                  ' If lineString.Length >= 10 Then
                textReader.Close()
            End If                      ' If File.Exists(pathString) = True Then
        Next                            ' For infNumber As Integer = 0 To 255
        If Not driverInstalled = True Then
            EMsg("The QCOM driver is not installed on this computer")
        End If
        Return driverInstalled
    End Function                        ' QCOM_DriverInstalled
    '-------------------------------------------------------------------------
    ' QCOM_EnsureMinimumEnvironment
    '
    ' Determines whether the client environment meets the minimum requirements
    ' necessary for the software to continue
    '
    ' Returns: 0           Success
    '          nonzero     Failure
    '
    ' Called by:    QCOM_InitializeQCOM
    '-------------------------------------------------------------------------
    Public Function QCOM_EnsureMinimumEnvironment() As UInt32
        Dim driverInstalled As Boolean = False
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        ' Retrieve information on the client operating environment
        '---------------------------------------------------------------------
        QCOM_GetOperatingEnvironment()
        '---------------------------------------------------------------------
        ' Determine whether sufficient free disk space is available
        '---------------------------------------------------------------------
        Dim driveInfo As New DriveInfo(Environment.CurrentDirectory)
        Dim availableSpaceMB As Int64 = driveInfo.AvailableFreeSpace / 1000000
        If availableSpaceMB < 2000 Then
            EMsg("Insufficient workstation disk space (2 GB required, but\n" + _
                 "a little more than {0} MB remain)" + _
                "\n\nThe program must exit", availableSpaceMB.ToString("D"))
            Return QCOM_FAILURE
        Else
            'Msg("{0} MB of disk space remaining on drive {1}", _
            '    availableSpaceMB.ToString("D"), driveInfo.Name)
        End If
        '---------------------------------------------------------------------
        ' Determine whether the qdUSB.dll version meets the minimum
        '---------------------------------------------------------------------
        Dim currentMajorVersion As Byte = 0
        Dim currentMinorVersion As Byte = 0
        Dim currentBuildVersion As Byte = 0
        Dim minimumMajorVersion As Byte = QCOM_MINIMUM_QDUSB_DLL_MAJOR_VERSION
        Dim minimumMinorVersion As Byte = QCOM_MINIMUM_QDUSB_DLL_MINOR_VERSION
        Dim minimumBuildVersion As Byte = QCOM_MINIMUM_QDUSB_DLL_BUILD_VERSION
        status = QD_GetQDDLLVersion( _
            currentMajorVersion, _
            currentMinorVersion, _
            currentBuildVersion)
        If status = QCOM_SUCCESS Then
            If ((currentMajorVersion << 16) Or (currentMinorVersion << 8) Or currentBuildVersion) < _
                ((minimumMajorVersion << 16) Or (minimumMinorVersion << 8) Or minimumBuildVersion) Then
                EMsg("The qdUSB.dll version is {0:D}.{1:D}.{2:D} but must be at least {3:D}.{4:D}.{5:D}" + _
                    "\n\nThe program must exit", _
                    currentMajorVersion, _
                    currentMinorVersion, _
                    currentBuildVersion, _
                    minimumMajorVersion, _
                    minimumMinorVersion, _
                    minimumBuildVersion)
                Return QCOM_FAILURE
            Else
                'Msg("The qdUSB version is {0:D}.{1:D}.{2:D}", _
                '    currentMajorVersion, currentMinorVersion, currentBuildVersion)
            End If
        End If
        '---------------------------------------------------------------------
        ' Determine whether the USB driver is installed
        '---------------------------------------------------------------------
        Dim lowerVersion As UInt32 = 0
        Dim upperVersion As UInt32 = 0
        status = QD_GetUSBDriverVersion(upperVersion, lowerVersion)
        If status = QCOM_SUCCESS Then
            QCOM_GeneralInfo.flags = (QCOM_GeneralInfo.flags Or QCOM_GENERAL_USB_DRIVER_AVAILABLE)
            'Msg("USB driver v{0:D}.{1:D}.{2:D}.{3:D} is installed", _
            '    (upperVersion >> 16), (upperVersion And &HFFFF), _
            '    (lowerVersion >> 16), (lowerVersion And &HFFFF))
        Else
            EMsg("QCOM USB driver is not properly installed on this workstation" + _
                "\n\nThe program must exit")
            Return QCOM_FAILURE
        End If
        '---------------------------------------------------------------------
        ' Determine whether the USB software is installed
        '---------------------------------------------------------------------
        status = QD_GetUSBDLLVersion(upperVersion, lowerVersion)
        If status = QCOM_SUCCESS Then
            QCOM_GeneralInfo.flags = (QCOM_GeneralInfo.flags Or QCOM_GENERAL_USB_DLL_AVAILABLE)
            'Msg("USB software v{0:D}.{1:D}.{2:D}.{3:D} is installed", _
            '    (upperVersion >> 16), (upperVersion And &HFFFF), _
            '   (lowerVersion >> 16), (lowerVersion And &HFFFF))
        Else
            EMsg("QCOM USB software is not properly installed on this workstation" + _
                "\n\nThe program must exit")
            Return QCOM_FAILURE
        End If
        '---------------------------------------------------------------------
        ' Determine whether the QCOM driver is installed
        '---------------------------------------------------------------------
        driverInstalled = QCOM_DriverInstalled()
        If driverInstalled = False Then
            EMsg("QCOM USB driver is not properly installed on this workstation" + _
                "\n\nThe program must exit")
            Return QCOM_FAILURE
        End If
        '---------------------------------------------------------------------
        ' Make sure the log directory is present
        '---------------------------------------------------------------------
        QCOM_EstablishLogDirectory()
        Return status
    End Function                        ' QCOM_EnsureMinimumEnvironment
    '-------------------------------------------------------------------------
    ' QCOM_Finalize
    '
    ' Performs the last tasks before shutting down the software, including
    ' the saving of the config file and releasing of resources
    '
    ' Called by:    QCOM_ShutDownSoftware
    '-------------------------------------------------------------------------
    Public Function QCOM_Finalize()
        QCOM_SaveConfigData()
        QCOM_FreeGeneralElements()
        Return Nothing
    End Function                        ' QCOM_Finalize
    '-------------------------------------------------------------------------
    ' QCOM_FreeGeneralElements
    '
    ' Releases the memory previously allocated for all units, and closes all
    ' open device handles
    '
    ' Called by:    QCOM_Finalize
    '-------------------------------------------------------------------------
    Public Function QCOM_FreeGeneralElements()
        For Each unit As UnitInfo In QCOM_UnitInfoArray
            If (unit.flags And QCOM_UNIT_OPEN) <> 0 Then
                QCOM_CloseUnit(unit)
            End If
            unit.flags = QCOM_UNIT_ZERO_FLAG
        Next
        Erase QCOM_UnitInfoArray
        Return Nothing
    End Function                        ' QCOM_FreeGeneralElements
    '-------------------------------------------------------------------------
    ' QCOM_InitializeGeneralFields
    '
    ' Initializes the fields of the QCOM_GeneralInfo structure
    '
    ' Called by:    QCOM_InitializeProgramComponents
    '-------------------------------------------------------------------------
    Public Function QCOM_InitializeGeneralFields()
        QCOM_GeneralInfo = New GeneralInfo
        QCOM_GeneralInfo.programStartTime = GetTickCount()
        QCOM_GeneralInfo.numberOfUnits = 0
        QCOM_GeneralInfo.numberOfTransducers = 0
        QCOM_GeneralInfo.validUnitBitMap = 0
        QCOM_GeneralInfo.readyUnitBitMap = 0
        QCOM_GeneralInfo.samplingInterval = QCOM_DEFAULT_SAMPLING_INTERVAL
        QCOM_GeneralInfo.flags = QCOM_GENERAL_ALLOCATED + QCOM_GENERAL_INITIAL_SETTINGS
        QCOM_GeneralInfo.persistentFlags = QCOM_GENERAL_DEFAULT_PERSISTENT_FLAGS
        QCOM_GeneralInfo.persistentUnitFlags = QCOM_UNIT_DEFAULT_PERSISTENT_FLAGS
        QCOM_GeneralInfo.configFilePath = Nothing
        QCOM_GeneralInfo.generalUsePath = Nothing
        QCOM_GeneralInfo.logDirectory = Nothing
        QCOM_GeneralInfo.windowsVersion = QCOM_STRING_UNKNOWN
        QCOM_GeneralInfo.searchString = Nothing
        QCOM_GeneralInfo.samplingStartTime = 0
        Return Nothing
    End Function                        ' QCOM_InitializeGeneralFields
    '-------------------------------------------------------------------------
    ' QCOM_InitializeProgramComponents
    '
    ' Initializes the base program components
    '
    ' Called by:    QCOM_InitializeQCOM
    '
    ' Note:     This method must be called only once
    '-------------------------------------------------------------------------
    Public Function QCOM_InitializeProgramComponents()
        '---------------------------------------------------------------------
        ' Load the graphic images and program sounds for the software
        '---------------------------------------------------------------------
        QCOM_LoadImagesAndSounds()
        '---------------------------------------------------------------------
        ' Display a "Please wait . . ." window
        '---------------------------------------------------------------------
        QCOM_PleaseWaitDisplay(QCOM_PLEASE_WAIT_INSTALL)
        '---------------------------------------------------------------------
        ' Create and initialize the general data structure
        '---------------------------------------------------------------------
        QCOM_InitializeGeneralFields()
        '---------------------------------------------------------------------
        ' Create and initialize all possible unit data structures
        '---------------------------------------------------------------------
        For Each unit As UnitInfo In QCOM_UnitInfoArray
            unit.unitNumber = Array.IndexOf(QCOM_UnitInfoArray, unit)
            unit.flags = QCOM_UNIT_ALLOCATED
            QCOM_InitializeUnitFields(unit)
            QCOM_UnitInfoArray(unit.unitNumber) = unit
            QCOM_GeneralInfo.maximumNumberOfUnits += 1
        Next
        Return Nothing
    End Function                        ' QCOM_InitializeProgramComponents
    '-------------------------------------------------------------------------
    ' QCOM_InitializeQCOM
    '
    ' Initializes the software and locates all applicable QCOM hardware, then
    ' prepares them for communication with the host
    '
    ' Returns:  0           Success
    '           nonzero     Failure
    '
    ' Called by:    QCOM_Start
    '
    ' Note:     This method must be called only once
    '-------------------------------------------------------------------------
    Public Function QCOM_InitializeQCOM() As UInt32
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        QCOM_InitializeProgramComponents()
        '---------------------------------------------------------------------
        ' The client environment must meet the minimum operating requirements
        '---------------------------------------------------------------------
        status = QCOM_EnsureMinimumEnvironment()
        If status = QCOM_SUCCESS Then
            QD_SetTimeouts(QCOM_TIMEOUT_500MS, QCOM_TIMEOUT_500MS)
            '-----------------------------------------------------------------
            ' The software is fully operational, so check for the presence of
            ' all applicable hardware
            '-----------------------------------------------------------------
            QCOM_ScanForDevices()
            '-----------------------------------------------------------------
            ' Populate the program with configuration information from the
            ' config file, if it is present
            '-----------------------------------------------------------------
            QCOM_RetrieveConfigData()
        End If
        Return status
    End Function                        ' QCOM_InitializeQCOM
    '-------------------------------------------------------------------------
    ' QCOM_InitializeUnitFields
    '
    ' Initializes the fields of the QCOM_UnitInfoArray() structure
    '
    ' Called by:    QCOM_InitializeProgramComponents
    '-------------------------------------------------------------------------
    Public Function QCOM_InitializeUnitFields(ByRef unit As UnitInfo)
        unit.unitHandle = INVALID_HANDLE_VALUE
        unit.moduleSerialNumber = Nothing
        unit.unitDescriptionString = Nothing
        unit.firmwareID = Nothing
        unit.firmwareString = Nothing
        unit.dataRate = 0.0
        unit.pressureCount = 0
        unit.temperatureCount = 0
        unit.pressureValuePSI = 0.0
        unit.temperatureValueCelsius = 0.0
        unit.transducerVoltage = 0.0
        unit.transducerCurrent = 0.0
        unit.transducerType = QD_TRANSDUCER_TYPE_ABSENT
        unit.transducerSerialNumber = Nothing
        unit.transducerPartNumber = Nothing
        unit.transducerChipID = Nothing
        unit.coefficientFilePath = Nothing
        unit.coefficientData = Nothing
        Return Nothing
    End Function                        ' QCOM_InitializeUnitFields
    '-------------------------------------------------------------------------
    ' QCOM_ScanForDevices
    '
    ' Scans the USB interface for all possible QCOM devices
    '
    ' Called by:    QCOM_InitializeQCOM
    '-------------------------------------------------------------------------
    Public Function QCOM_ScanForDevices()
        Dim status As UInt32
        Dim numberOfModules As UInt32 = QD_GetNumberOfModules()
        '---------------------------------------------------------------------
        If numberOfModules <> 0 Then
            QCOM_GeneralInfo.numberOfUnits = numberOfModules
            'Msg("Scan: Found {0:D} module{1}", numberOfModules, If(numberOfModules <> 1, "s", ""))
            For Each unit As UnitInfo In QCOM_UnitInfoArray
                Dim unitNumber As UInt32 = Array.IndexOf(QCOM_UnitInfoArray, unit)
                If unit.unitNumber = unitNumber Then
                    unit.physicalUnitNumber = unitNumber
                    '---------------------------------------------------------
                    ' Retrieve the serial number of the specified module
                    '---------------------------------------------------------
                    QCOM_GetModuleInfo(unit)
                    If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
                        '-----------------------------------------------------
                        ' This is a QCOM module, so set appropriate flags
                        '-----------------------------------------------------
                        QCOM_SetUnitFlagsToInitialValues(unit)
                        '-----------------------------------------------------
                        ' Open a handle to the unit
                        '-----------------------------------------------------
                        QCOM_CloseUnit(unit)
                        status = QCOM_OpenUnit(unit)
                        If status = QCOM_SUCCESS Then
                            unit.flags = (unit.flags Or QCOM_UNIT_OPEN)
                            '-------------------------------------------------
                            ' Check for Boot Loader Mode
                            '-------------------------------------------------
                            If (unit.flags And QCOM_UNIT_BOOT_LOADER_MODE) <> 0 Then
                                QCOM_SwitchFromBootLoaderMode(unit)
                            End If
                            '-------------------------------------------------
                            ' Verify that this module is listed in the Registry
                            '-------------------------------------------------
                            QCOM_CheckModuleRegistryEntry(unit)
                            If (unit.flags And QCOM_UNIT_BOOT_LOADER_MODE) = 0 Then
                                '---------------------------------------------
                                ' Retrieve the QCOM unit firmware version and ID
                                '---------------------------------------------
                                QCOM_RetrieveFirmwareInformation(unit)
                                '---------------------------------------------
                                ' Set the initial I²C data rate to 100 kHz,
                                ' the I²C maximum, whether a transducer is
                                ' present or not
                                '---------------------------------------------
                                QCOM_SetTransducerDataRate(unit)
                                '---------------------------------------------
                                ' Determines whether a transducer is present
                                ' at the current unit, then retrieves info
                                ' about the attached transducer, if one is
                                '---------------------------------------------
                                QCOM_GetTransducerInfo(unit)
                                '---------------------------------------------
                                ' Read coefficient data into memory, from the
                                ' attached transducer if it is a digital
                                ' version that contains valid coefficient
                                ' data, or from the module otherwise
                                '---------------------------------------------
                                status = QCOM_RetrieveCoefficientData(unit)
                                '---------------------------------------------
                                ' Update the map of units ready to produce data
                                '---------------------------------------------
                                If (unit.flags And QCOM_UNIT_READY) <> 0 Then
                                    QCOM_GeneralInfo.readyUnitBitMap = (QCOM_GeneralInfo.readyUnitBitMap Or (1 << unitNumber))
                                End If
                            End If      ' If Not unit.flags And QCOM_UNIT_BOOT_LOADER_MODE Then
                        End If          ' If status = QCOM_SUCCESS Then
                    End If              ' If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
                Else
                    EMsg("Unit numbers don't match: {0:D} vs. {1:D}", unit.unitNumber, unitNumber)
                End If                  ' If unit.unitNumber = unitNumber Then
                '-------------------------------------------------------------
                ' Construct a display string that describes this unit,
                ' including module serial number, transducer serial number,
                ' and transducer type and chip ID
                '-------------------------------------------------------------
                QCOM_ConstructUnitDescriptionString(unit)
                QCOM_UnitInfoArray(unit.unitNumber) = unit
            Next                        ' For Each unit As UnitInfo In QCOM_UnitInfoArray
            '-----------------------------------------------------------------
            ' After all available modules have been opened for I/O, determine
            ' how many transducers are connected
            '-----------------------------------------------------------------
            QCOM_GeneralInfo.numberOfTransducers = QCOM_GetNumberOfTransducers()
        End If                          ' If numberOfModules <> 0 Then
        Return Nothing
    End Function                        ' QCOM_ScanForDevices
    '-------------------------------------------------------------------------
    ' QCOM_SetUnitFlagsToInitialValues
    '
    ' Sets the flags of the specified unit to initial values
    '
    ' Called by:    QCOM_ScanForDevices
    '-------------------------------------------------------------------------
    Public Function QCOM_SetUnitFlagsToInitialValues(ByRef unit As UnitInfo)
        If (unit.flags And QCOM_UNIT_VALID) <> 0 AndAlso _
            (unit.flags And QCOM_UNIT_INITIAL_SETTINGS_SET) = 0 Then
            unit.flags = (unit.flags Or QCOM_UNIT_INITIAL_SETTINGS Or QCOM_UNIT_INITIAL_SETTINGS_SET)
        End If                          ' If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
        Return Nothing
    End Function                        ' QCOM_SetUnitFlagsToInitialValues
End Class                               ' QCOM_Form
'=============================================================================
' End of QCOM.vb
'=============================================================================
