﻿'=============================================================================
' Utility.vb
'
' The methods used by GUI.cpp for utility tasks
'
' Copyright (C) 2013 - 2013 Quartzdyne, Inc.  All rights reserved.
'
' Express written permission is hereby granted from Quartzdyne, Inc., to
' copy, modify, reproduce, and transmit this source code in any form and by
' any means, electronic or mechanical, including photocopying, recording, and
' any information storage and retrieval system as needed.  Permission is also
' hereby granted to use this source code in any form and means as needed.
' While every reasonable precaution has been taken in the preparation of this
' source code, Quartzdyne, Inc., assumes no responsibility for errors,
' omissions, or damages from the use of the source code contained herein.
'
' NEITHER QUARTZDYNE, INC., ANY MEMBER OF QUARTZDYNE, INC., NOR ANY PERSON OR
' ORGANIZATION ACTING ON BEHALF OF THEM MAKES ANY WARRANTY OR REPRESENTATION
' WHATSOEVER, EXPRESS OR IMPLIED, INCLUDING ANY WARRANTY OF MERCHANTABILITY
' OR FITNESS FOR ANY PURPOSE WITH RESPECT TO THE SOURCE CODE OR ASSUMES ANY
' LIABILITY WHATSOEVER WITH RESPECT TO ANY USE OF THE SOURCE CODE OR ANY
' PORTION THEREOF OR WITH RESPECT TO ANY DAMAGES THAT MAY RESULT FROM SUCH
' USE.
'
' Noji Ratzlaff
'
' See QCOM.log for a complete history of all updates, changes, and revisions
' of this source and other files and policies regarding QCOM software
'
' Updated 04-23-2013
'=============================================================================
Imports System.IO
Imports System.Runtime.InteropServices
Imports System.Threading
Imports Microsoft.Win32
'-----------------------------------------------------------------------------
' QCOM_Form
'
' A continuation of the QCOM_Form class
'-----------------------------------------------------------------------------
Partial Class QCOM_Form
    Inherits Form
    '-------------------------------------------------------------------------
    ' EMsg
    '
    ' Displays an error message box with a formatted message and error sound
    '-------------------------------------------------------------------------
    Public Function EMsg(ByVal formatString As String, ByVal ParamArray parameters() As String)
        Dim formattedString As String = String.Format(formatString, parameters)
        '---------------------------------------------------------------------
        If formattedString.Contains("\n") = True Then
            formattedString = formattedString.Replace("\n", Environment.NewLine)
        End If
        QCOM_PlaySound(errorSound)
        MsgBox(formattedString)
        Return Nothing
    End Function                        ' EMsg
    '-------------------------------------------------------------------------
    ' Msg
    '
    ' Displays a message box with a formatted message
    '-------------------------------------------------------------------------
    Public Function Msg(ByVal formatString As String, ByVal ParamArray parameters() As String)
        Dim formattedString As String = String.Format(formatString, parameters)
        '---------------------------------------------------------------------
        If formattedString.Contains("\n") = True Then
            formattedString = formattedString.Replace("\n", Environment.NewLine)
        End If
        MsgBox(formattedString)
        Return Nothing
    End Function                        ' Msg
    '-------------------------------------------------------------------------
    ' QCOM_AffirmStartupConditions
    '
    ' Ensures the operating environment and other conditions are what they
    ' should be prior to displaying the user interface
    '
    ' Called by:    QCOM_InitializeGUIComponents
    '-------------------------------------------------------------------------
    Public Function QCOM_AffirmStartupConditions()
        Return Nothing
    End Function                        ' QCOM_AffirmStartupConditions
    '-------------------------------------------------------------------------
    ' QCOM_ApplyCoefficientDataInfoToProgram
    '
    ' Updates various program entities and displays with the information based
    ' on the coefficient data currently stored in unit.coefficientData
    '
    ' Called by:    QCOM_RetrieveCoefficientData
    '-------------------------------------------------------------------------
    Public Function QCOM_ApplyCoefficientDataInfoToProgram(ByRef unit As UnitInfo)
        Dim status As UInt32 = QCOM_SUCCESS
        Dim functionName As String = "QCOM_ApplyCoefficientDataInfoToProgram"
        '---------------------------------------------------------------------
        If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
            '-----------------------------------------------------------------
            ' Extract the transducer serial number embedded in the coefficient
            ' data
            '-----------------------------------------------------------------
            QCOM_InterpretCoefficientData(unit)
            '-----------------------------------------------------------------
            ' Re-construct the unit description string, now that the
            ' transducer serial number is known
            '-----------------------------------------------------------------
            QCOM_ConstructUnitDescriptionString(unit)
            '-----------------------------------------------------------------
            ' Clear the applicable flags that might have been set previously
            '-----------------------------------------------------------------
            unit.flags = (unit.flags And Not (QCOM_UNIT_INITIAL_READINGS Or QCOM_UNIT_CAN_COMMUNICATE Or QCOM_UNIT_OK_TO_SAMPLE))
            If (unit.flags And QCOM_UNIT_TRANSDUCER_PRESENT) <> 0 Then
                status = QCOM_RetrieveTransducerReadings(unit)
                If status = QCOM_SUCCESS Then
                    unit.flags = (unit.flags Or (QCOM_UNIT_INITIAL_READINGS Or QCOM_UNIT_CAN_COMMUNICATE Or QCOM_UNIT_OK_TO_SAMPLE))
                Else
                    If status <> QCOM_ERROR_UNIT_NOT_READY Then
                        EMsg("{0}({1:D})\n\nError 0x{2} retrieving initial readings", functionName, unit.unitNumber, status.ToString("X8"))
                    End If              ' If status <> QCOM_ERROR_UNIT_NOT_READY Then
                End If                  ' If status = QCOM_SUCCESS Then
            End If                      ' If (unit.flags And QCOM_UNIT_TRANSDUCER_PRESENT) <> 0 Then
        End If                          ' If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
        Return Nothing
    End Function                        ' QCOM_ApplyCoefficientDataInfoToProgram
    '-------------------------------------------------------------------------
    ' QCOM_CalculatePercentage
    '
    ' Calculates an integer (whole number) percentage
    '
    ' Returns:      The result of the integer percentage calculation
    '-------------------------------------------------------------------------
    Public Function QCOM_CalculatePercentage(ByVal dividend As UInt32, ByVal divisor As UInt32) As Integer
        Dim holder As UInt32
        Dim quotient As Integer = 0
        '---------------------------------------------------------------------
        If (dividend <> 0) AndAlso (divisor <> 0) Then
            '-----------------------------------------------------------------
            ' Multiply by 10 twice, using the fast-multiply
            '-----------------------------------------------------------------
            holder = (((dividend << 2) + dividend) << 1)
            holder = (((holder << 2) + holder) << 1)
            '-----------------------------------------------------------------
            ' Perform the integer division
            '-----------------------------------------------------------------
            quotient = (holder / divisor)
        End If
        Return quotient
    End Function                        ' QCOM_CalculatePercentage
    '-------------------------------------------------------------------------
    ' QCOM_CheckModuleRegistryEntry
    '
    ' Checks the Registry for the keys that correspond to the specified module,
    ' then enters and exits BootLoader Mode if either of the keys is not present
    '
    ' Called by:    QCOM_ScanForDevices
    '-------------------------------------------------------------------------
    Public Function QCOM_CheckModuleRegistryEntry(ByRef unit As UnitInfo)
        Dim foundRegularPath As Boolean = False
        Dim foundTildePath As Boolean = False
        Dim registryKey As String
        Dim registryKeyTilde As String
        Dim registryKeyPath As String
        Dim registryKeyString As String
        Dim registryKeyStringTilde As String
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        registryKeyPath = _
            "HKEY_LOCAL_MACHINE" & _
            "\SYSTEM\CurrentControlSet\Enum\USB\VID_" + _
            QCOM_VENDOR_ID_STRING + _
            "&PID_" + _
            QCOM_PRODUCT_ID_STRING + _
            "\"
        registryKeyString = _
            registryKeyPath + _
            unit.moduleSerialNumber.Substring(0, _
                If(unit.moduleSerialNumber.Contains("~"), _
                   unit.moduleSerialNumber.IndexOf("~"), _
                   unit.moduleSerialNumber.Length))
        registryKeyStringTilde = _
            registryKeyPath + _
            If(unit.moduleSerialNumber.Contains("~"), _
                unit.moduleSerialNumber, _
                unit.moduleSerialNumber + "~")
        registryKey = Registry.GetValue(registryKeyString, "Service", Nothing)
        If registryKey = "QCOM" Then
            foundRegularPath = True
        End If
        registryKeyTilde = Registry.GetValue(registryKeyStringTilde, "Service", Nothing)
        If registryKeyTilde = "QCOM" Then
            foundTildePath = True
        End If
        If foundRegularPath = False OrElse foundTildePath = False Then
            QCOM_SwitchToBootLoaderMode(unit)
            Thread.Sleep(1000)
            QCOM_SwitchFromBootLoaderMode(unit)
            Thread.Sleep(2000)
            registryKey = Registry.GetValue(registryKeyString, "Service", Nothing)
            If registryKey = "QCOM" Then
                foundRegularPath = True
            End If
            registryKeyTilde = Registry.GetValue(registryKeyStringTilde, "Service", Nothing)
            If registryKeyTilde = "QCOM" Then
                foundTildePath = True
            End If
        End If                          ' If foundRegularPath = False OrElse foundTildePath = False Then
        If foundRegularPath = True AndAlso foundTildePath = True Then
            unit.flags = (unit.flags Or QCOM_UNIT_REGISTRY_CHECKED)
        Else
            'EMsg("The QCOM driver is not configured properly in the Windows Registry")
        End If
        Return Nothing
    End Function                        ' QCOM_CheckModuleRegistryEntry
    '-------------------------------------------------------------------------
    ' QCOM_CloseUnit
    '
    ' Closes the specified unit for I/O
    '-------------------------------------------------------------------------
    Public Function QCOM_CloseUnit(ByRef unit As UnitInfo)
        If (unit.flags And QCOM_UNIT_VALID) <> 0 AndAlso (unit.flags And QCOM_UNIT_OPEN) <> 0 Then
            If unit.unitHandle <> INVALID_HANDLE_VALUE Then
                QD_Close(unit.unitHandle)
            End If
            unit.unitHandle = INVALID_HANDLE_VALUE
            unit.flags = (unit.flags And Not QCOM_UNIT_OPEN)
        End If                          ' If (unit.flags And QCOM_UNIT_VALID) <> 0 AndAlso ...
        Return Nothing
    End Function                        ' QCOM_CloseUnit
    '-------------------------------------------------------------------------
    ' QCOM_ConstructUnitDescriptionString
    '
    ' Constructs the unit description string from the module and transducer
    ' serial numbers, if they are available
    '-------------------------------------------------------------------------
    Public Function QCOM_ConstructUnitDescriptionString(ByRef unit As UnitInfo)
        Dim transducerTypeString As String
        '---------------------------------------------------------------------
        If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
            Select Case unit.transducerType
                Case 0
                    If (unit.flags And QCOM_UNIT_TRANSDUCER_PRESENT) <> 0 Then
                        transducerTypeString = "Unknown Type"
                    Else
                        transducerTypeString = "No"
                    End If
                Case 1
                    transducerTypeString = "Frequency (Analog)"
                Case 2
                    transducerTypeString = "Digital (older type)"
                Case 3, 4
                    transducerTypeString = "Digital (newer type)"
                Case 5
                    transducerTypeString = "Digital (no memory)"
                Case Else
                    transducerTypeString = "Unknown Type"
            End Select                  ' Select Case unit.transducerType
            If (unit.flags And QCOM_UNIT_MODULE_SN_VALID) <> 0 Then
                unit.unitDescriptionString = String.Format( _
                    "Module {0} [Unit {1:D}] : {2} Transducer S/N {3}", _
                    unit.moduleSerialNumber, _
                    unit.unitNumber, _
                    transducerTypeString, _
                    unit.transducerSerialNumber)
            Else
                unit.unitDescriptionString = String.Format( _
                    "Unit {0:D} : {1} Transducer S/N {2}", _
                    unit.unitNumber, _
                    transducerTypeString, _
                    unit.transducerSerialNumber)
            End If                      ' If (unit.flags And QCOM_UNIT_MODULE_SN_VALID) <> 0 Then
        Else
            unit.unitDescriptionString = String.Format("Unit {0:D}", unit.unitNumber)
        End If                          ' If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
        Return Nothing
    End Function                        ' QCOM_ConstructUnitDescriptionString
    '-------------------------------------------------------------------------
    ' QCOM_ConvertCString
    '
    ' Converts a C-style string to a VB-style string
    '
    ' Note:     The C string must be null-terminated
    '-------------------------------------------------------------------------
    Public Function QCOM_ConvertCString(ByVal cString As Array) As String
        Dim vbString As String = Nothing
        '---------------------------------------------------------------------
        For offset As Integer = 0 To cString.Length - 1
            If cString(offset) <> 0 Then
                vbString += Chr(cString(offset))
            Else
                Exit For
            End If                      ' If cString(offset) <> 0 Then
        Next                            ' For offset As Integer = 0 To cString.Length - 1
        Return vbString
    End Function                        ' QCOM_ConvertCString
    '-------------------------------------------------------------------------
    ' QCOM_ConvertVBString
    '
    ' Converts a VB-style string to a C-style string
    '-------------------------------------------------------------------------
    Public Function QCOM_ConvertVBString(ByVal vbString As String) As Array
        Dim cString(vbString.Length) As Byte
        '---------------------------------------------------------------------
        For offset As Integer = 0 To vbString.Length - 1
            cString(offset) = Asc(vbString(offset))
        Next
        Return cString
    End Function                        ' QCOM_ConvertVBString
    '-------------------------------------------------------------------------
    ' QCOM_DisplayProgramInformation
    '
    ' Retrieves and displays information about the program and its environment
    '
    ' Called by:    QCOM_ToolStripProgramInfoButtonClicked
    '-------------------------------------------------------------------------
    Public Function QCOM_DisplayProgramInformation()
        '---------------------------------------------------------------------
        ' Create a new window form each time this is called, due to the
        ' dynamic nature of the program values it displays
        '---------------------------------------------------------------------
        programInfoWindow = New Form()
        '---------------------------------------------------------------------
        ' Set its appearance
        '---------------------------------------------------------------------
        programInfoWindow.SuspendLayout()
        programInfoWindow.Size = New Drawing.Size( _
            GUI_PROG_INFO_WINDOW_WIDTH, _
            GUI_PROG_INFO_WINDOW_HEIGHT)
        programInfoWindow.MaximizeBox = False
        programInfoWindow.HelpButton = False
        programInfoWindow.Icon = QCOM_SoftwareIcon
        programInfoWindow.BackgroundImage = whiteSandBackground
        programInfoWindow.Text = "QCOM Program Information"
        programInfoWindow.FormBorderStyle = System.Windows.Forms.FormBorderStyle.Fixed3D
        programInfoWindow.StartPosition = FormStartPosition.CenterScreen
        '---------------------------------------------------------------------
        ' Create the General group box
        '---------------------------------------------------------------------
        Dim pInfoGeneralGroupBox As New GroupBox()
        Dim generalGroupBoxDefined As Boolean = True
        '---------------------------------------------------------------------
        If generalGroupBoxDefined = True Then
            pInfoGeneralGroupBox.Text = "General Program Information"
            pInfoGeneralGroupBox.Location = New Point(10, 10)
            pInfoGeneralGroupBox.Size = New Drawing.Size( _
                programInfoWindow.Width - 30,
                GUI_PROG_INFO_GROUP_BOX_HEIGHT)
            pInfoGeneralGroupBox.BackColor = Color.Transparent
            programInfoWindow.Controls.Add(pInfoGeneralGroupBox)
            '-----------------------------------------------------------------
            ' Program name
            '-----------------------------------------------------------------
            Dim pInfoProgramName As New Label()
            pInfoProgramName.Text = "Name: " + QCOM_PROGRAM_TITLE
            pInfoProgramName.Location = New Point(10, 20)
            pInfoProgramName.Size = New Drawing.Size(300, GUI_INFO_LABEL_HEIGHT)
            pInfoProgramName.BackColor = Color.Transparent
            pInfoGeneralGroupBox.Controls.Add(pInfoProgramName)
            '-----------------------------------------------------------------
            ' Program version
            '-----------------------------------------------------------------
            Dim pInfoProgramVersionAndBuild As New Label()
            pInfoProgramVersionAndBuild.Text = "Version: " + QCOM_PROGRAM_VERSION_STRING
            QCOM_PositionAndSizeBelow(pInfoProgramVersionAndBuild, pInfoProgramName, 2)
            pInfoGeneralGroupBox.Controls.Add(pInfoProgramVersionAndBuild)
            '-----------------------------------------------------------------
            ' Current date and time
            '-----------------------------------------------------------------
            Dim lapsedDays As Integer = 0
            Dim lapsedHours As Integer = 0
            Dim lapsedMinutes As Integer = 0
            Dim lapsedSeconds As Integer = 0
            Dim lapsedMilliseconds As Integer = 0
            Dim dateTimes As DateTime = DateTime.Now
            Dim pInfoThisMoment As New Label()
            QCOM_TimeElapsed( _
                GetTickCount() - QCOM_GeneralInfo.programStartTime, _
                lapsedDays, lapsedHours, _
                lapsedMinutes, lapsedSeconds, lapsedMilliseconds)
            pInfoThisMoment.Text = String.Format( _
                "Now: {0:D2} {1} {2:D4} {3:D2}:{4:D2}:{5:D2}   " + _
                "[ up {6:D} d {7:D} h {8:D} m {9:D} s ]", _
                dateTimes.Day, _
                QCOM_MonthStringArray(dateTimes.Month), _
                dateTimes.Year, dateTimes.Hour, dateTimes.Minute, dateTimes.Second, _
                lapsedDays, lapsedHours, lapsedMinutes, lapsedSeconds)
            QCOM_PositionAndSizeBelow(pInfoThisMoment, pInfoProgramVersionAndBuild, 2)
            pInfoGeneralGroupBox.Controls.Add(pInfoThisMoment)
            '-----------------------------------------------------------------
            ' Program author
            '-----------------------------------------------------------------
            Dim pInfoProgramAuthor As New Label()
            pInfoProgramAuthor.Text = "Author: " + QCOM_PROGRAM_AUTHOR
            QCOM_PositionAndSizeBelow(pInfoProgramAuthor, pInfoThisMoment, 2)
            pInfoGeneralGroupBox.Controls.Add(pInfoProgramAuthor)
            '-----------------------------------------------------------------
            '  Valid unit bitmap
            '-----------------------------------------------------------------
            Dim pInfoValidUnitBitMap As New Label()
            pInfoValidUnitBitMap.Text = _
                "Valid Unit Bitmap: " + QCOM_GeneralInfo.validUnitBitMap.ToString("X8")
            pInfoValidUnitBitMap.Location = New Point( _
                pInfoProgramName.Right + 10, _
                pInfoProgramName.Top)
            pInfoValidUnitBitMap.Size = New Drawing.Size( _
                210, GUI_INFO_LABEL_HEIGHT)
            pInfoValidUnitBitMap.BackColor = Color.Transparent
            pInfoGeneralGroupBox.Controls.Add(pInfoValidUnitBitMap)
            '-----------------------------------------------------------------
            ' Ready unit bitmap
            '-----------------------------------------------------------------
            Dim pInfoReadyUnitBitMap As New Label()
            pInfoReadyUnitBitMap.Text = _
                "Ready Unit Bitmap: " + QCOM_GeneralInfo.readyUnitBitMap.ToString("X8")
            QCOM_PositionAndSizeBelow(pInfoReadyUnitBitMap, pInfoValidUnitBitMap, 2)
            pInfoGeneralGroupBox.Controls.Add(pInfoReadyUnitBitMap)
            '-----------------------------------------------------------------
            ' Windows version
            '-----------------------------------------------------------------
            Dim pInfoWindowsVersionLabel As New Label()
            pInfoWindowsVersionLabel.Text = "OS: " + QCOM_GeneralInfo.windowsVersion
            pInfoWindowsVersionLabel.Location = New Point( _
                pInfoValidUnitBitMap.Right + 10, _
                pInfoValidUnitBitMap.Top)
            pInfoWindowsVersionLabel.Size = New Drawing.Size(220, GUI_INFO_LABEL_HEIGHT)
            pInfoWindowsVersionLabel.BackColor = Color.Transparent
            pInfoGeneralGroupBox.Controls.Add(pInfoWindowsVersionLabel)
            '-----------------------------------------------------------------
            ' Memory information
            '-----------------------------------------------------------------
            Dim pInfoClientMemoryLabel As New Label()
            Dim memory As MEMORYSTATUSEX
            memory.Length = Marshal.SizeOf(memory)
            GlobalMemoryStatusEx(memory)
            Dim availablePhysicalMB As UInt64 = memory.AvailPhys / 1000000
            Dim totalPhysicalMB As UInt64 = memory.TotalPhys / 1000000
            pInfoClientMemoryLabel.Text = String.Format( _
                "Avail Memory: {0} MB / {1} MB", _
                availablePhysicalMB.ToString("D"), _
                totalPhysicalMB.ToString("D"))
            QCOM_PositionAndSizeBelow(pInfoClientMemoryLabel, pInfoWindowsVersionLabel, 2)
            pInfoGeneralGroupBox.Controls.Add(pInfoClientMemoryLabel)
            '-----------------------------------------------------------------
            ' Disk space information
            '-----------------------------------------------------------------
            Dim pInfoDiskSpaceLabel As New Label()
            Dim driveInfo As New DriveInfo(Environment.CurrentDirectory)
            Dim availableSpaceMB As Int64 = driveInfo.AvailableFreeSpace / 1000000
            Dim totalSpaceMB As Int64 = driveInfo.TotalSize / 1000000
            pInfoDiskSpaceLabel.Text = String.Format( _
                "Avail Storage: {0} MB / {1} MB", _
                availableSpaceMB.ToString("D"), _
                totalSpaceMB.ToString("D"))
            QCOM_PositionAndSizeBelow(pInfoDiskSpaceLabel, pInfoClientMemoryLabel, 2)
            pInfoGeneralGroupBox.Controls.Add(pInfoDiskSpaceLabel)
            '-----------------------------------------------------------------
            ' qdUSB.dll version
            '-----------------------------------------------------------------
            Dim pInfoQDDLLVersionLabel As New Label()
            Dim majorVersion As Byte = 0
            Dim minorVersion As Byte = 0
            Dim buildVersion As Byte = 0
            QD_GetQDDLLVersion(majorVersion, minorVersion, buildVersion)
            pInfoQDDLLVersionLabel.Text = String.Format( _
                "QCOM DLL Version: {0:D}.{1:D}.{2:D}", _
                majorVersion, minorVersion, buildVersion)
            QCOM_PositionAndSizeBelow(pInfoQDDLLVersionLabel, pInfoDiskSpaceLabel, 2)
            pInfoGeneralGroupBox.Controls.Add(pInfoQDDLLVersionLabel)
            '-----------------------------------------------------------------
            ' Silicon Labs driver version
            '-----------------------------------------------------------------
            Dim lowerVersion As UInt32 = 0
            Dim upperVersion As UInt32 = 0
            Dim pInfoDriverVersionLabel As New Label()
            QD_GetUSBDriverVersion(upperVersion, lowerVersion)
            pInfoDriverVersionLabel.Text = String.Format( _
                "Silicon Labs USB Driver Version: {0:D}.{1:D}.{2:D}.{3:D}", _
                (upperVersion >> 16), (upperVersion And &HFFFF), _
                (lowerVersion >> 16), (lowerVersion And &HFFFF))
            QCOM_PositionAndSizeBelow(pInfoDriverVersionLabel, pInfoQDDLLVersionLabel, 2)
            pInfoGeneralGroupBox.Controls.Add(pInfoDriverVersionLabel)
            '-----------------------------------------------------------------
            ' Silicon Labs DLL version
            '-----------------------------------------------------------------
            Dim pInfoUSBDLLVersionLabel As New Label()
            QD_GetUSBDLLVersion(upperVersion, lowerVersion)
            pInfoUSBDLLVersionLabel.Text = String.Format( _
                "Silicon Labs DLL Version: {0:D}.{1:D}.{2:D}.{3:D}", _
                (upperVersion >> 16), (upperVersion And &HFFFF), _
                (lowerVersion >> 16), (lowerVersion And &HFFFF))
            QCOM_PositionAndSizeBelow(pInfoUSBDLLVersionLabel, pInfoDriverVersionLabel, 2)
            pInfoGeneralGroupBox.Controls.Add(pInfoUSBDLLVersionLabel)
            '-----------------------------------------------------------------
            ' Program startup path
            '-----------------------------------------------------------------
            Dim pInfoStartupPathLabel As New Label()
            pInfoStartupPathLabel.Size = New Drawing.Size( _
                pInfoProgramName.Width + pInfoValidUnitBitMap.Width + 10, _
                GUI_INFO_LABEL_HEIGHT)
            Dim startupPathCaption As String = "Startup Path: "
            Dim startupPath As String = QCOM_LimitFilePathStringWidth( _
                Application.ExecutablePath, _
                pInfoStartupPathLabel.Width - QCOM_StringWidth(startupPathCaption))
            pInfoStartupPathLabel.Text = startupPathCaption + startupPath
            QCOM_PositionBelow(pInfoStartupPathLabel, pInfoProgramAuthor, 2)
            pInfoGeneralGroupBox.Controls.Add(pInfoStartupPathLabel)
            '-----------------------------------------------------------------
            ' Config file path
            '-----------------------------------------------------------------
            Dim pInfoConfigFilePathLabel As New Label()
            QCOM_PositionAndSizeBelow(pInfoConfigFilePathLabel, pInfoStartupPathLabel, 2)
            Dim cfgPathPrefix As String = "Config File Path: "
            Dim cfgPath As String
            If (QCOM_GeneralInfo.flags And QCOM_GENERAL_CONFIG_FILE_SPECIFIED) <> 0 AndAlso _
                QCOM_GeneralInfo.configFilePath.Length > 0 Then
                cfgPath = QCOM_LimitFilePathStringWidth( _
                    QCOM_GeneralInfo.configFilePath, _
                    pInfoConfigFilePathLabel.Width - QCOM_StringWidth(cfgPathPrefix))
            Else
                cfgPath = QCOM_STRING_NONE
            End If
            pInfoConfigFilePathLabel.Text = cfgPathPrefix + cfgPath
            pInfoGeneralGroupBox.Controls.Add(pInfoConfigFilePathLabel)
            '-----------------------------------------------------------------
            ' General flags
            '-----------------------------------------------------------------
            Dim pInfoGeneralFlags As New Label()
            pInfoGeneralFlags.Text = "Gen Flags: " + QCOM_GeneralInfo.flags.ToString("X8")
            QCOM_PositionBelow(pInfoGeneralFlags, pInfoConfigFilePathLabel, 2)
            pInfoGeneralFlags.Size = New Drawing.Size(130, GUI_INFO_LABEL_HEIGHT)
            pInfoGeneralGroupBox.Controls.Add(pInfoGeneralFlags)
            '-----------------------------------------------------------------
            ' General persistent flags
            '-----------------------------------------------------------------
            Dim pInfoGeneralPersistentFlags As New Label()
            pInfoGeneralPersistentFlags.Text = String.Format( _
                "Gen Persist: {0:X8}", QCOM_GENERAL_DEFAULT_PERSISTENT_FLAGS)
            pInfoGeneralPersistentFlags.Location = New Point( _
                pInfoGeneralFlags.Right + 10, _
                pInfoGeneralFlags.Top)
            pInfoGeneralPersistentFlags.Size = pInfoGeneralFlags.Size
            pInfoGeneralPersistentFlags.BackColor = Color.Transparent
            pInfoGeneralGroupBox.Controls.Add(pInfoGeneralPersistentFlags)
            '-----------------------------------------------------------------
            ' Unit persistent flags
            '-----------------------------------------------------------------
            Dim pInfoUnitPersistentFlags As New Label()
            pInfoUnitPersistentFlags.Text = String.Format( _
                "Unit Persist: {0:X8}", QCOM_UNIT_DEFAULT_PERSISTENT_FLAGS)
            pInfoUnitPersistentFlags.Location = New Point( _
                pInfoGeneralPersistentFlags.Right + 10, _
                pInfoGeneralPersistentFlags.Top)
            pInfoUnitPersistentFlags.Size = pInfoGeneralPersistentFlags.Size
            pInfoUnitPersistentFlags.BackColor = Color.Transparent
            pInfoGeneralGroupBox.Controls.Add(pInfoUnitPersistentFlags)
        End If                          ' If generalGroupBoxDefined = True Then
        '---------------------------------------------------------------------
        ' Set up the tab controls
        '---------------------------------------------------------------------
        pInfoTabControl = New TabControl()
        Dim tabsDefined As Boolean = True
        '---------------------------------------------------------------------
        If tabsDefined = True Then
            pInfoTabControl.Appearance = TabAppearance.Normal
            pInfoTabControl.Alignment = TabAlignment.Top
            pInfoTabControl.Size = New Drawing.Size( _
                pInfoGeneralGroupBox.Width, _
                programInfoWindow.Height - pInfoGeneralGroupBox.Height - 92)
            QCOM_PositionBelow(pInfoTabControl, pInfoGeneralGroupBox, 10)
            programInfoWindow.Controls.Add(pInfoTabControl)
            '-----------------------------------------------------------------
            ' Set up and populate the tabs for all possible units
            '-----------------------------------------------------------------
            For Each unit As UnitInfo In QCOM_UnitInfoArray
                QCOM_DisplayProgramInformationTab(unit)
            Next                        ' For Each unit As UnitInfo In QCOM_UnitInfoArray
        End If                          ' If tabsDefined = True Then
        '---------------------------------------------------------------------
        ' Add a Close button
        '---------------------------------------------------------------------
        Dim pInfoCloseButton As New Button()
        pInfoCloseButton.Text = "Close"
        pInfoCloseButton.Location = New Point( _
            programInfoWindow.Width - 95, _
            programInfoWindow.Height - 65)
        pInfoCloseButton.Size = New Drawing.Size( _
            GUI_CLOSE_BUTTON_WIDTH, _
            GUI_REGULAR_BUTTON_HEIGHT)
        pInfoCloseButton.BackColor = Color.Transparent
        pInfoCloseButton.Cursor = Cursors.Hand
        AddHandler pInfoCloseButton.Click, AddressOf QCOM_ProgramInfoCloseWindow
        programInfoWindow.AcceptButton = pInfoCloseButton
        programInfoWindow.CancelButton = pInfoCloseButton
        programInfoWindow.Controls.Add(pInfoCloseButton)
        '---------------------------------------------------------------------
        ' Display the newly created window
        '---------------------------------------------------------------------
        programInfoWindow.ResumeLayout(False)
        programInfoWindow.ShowDialog()
        programInfoWindow.BringToFront()
        If programInfoWindow.CanFocus = True Then
            programInfoWindow.Focus()
        End If
        Return Nothing
    End Function                        ' QCOM_DisplayProgramInformation
    '-------------------------------------------------------------------------
    ' QCOM_DisplayProgramInformationTab
    '
    ' Retrieves and displays information about the specified unit
    '
    ' Called by:    QCOM_DisplayProgramInformation
    '-------------------------------------------------------------------------
    Public Function QCOM_DisplayProgramInformationTab(ByRef unit As UnitInfo)
        Dim unitNumber As UInt32 = unit.unitNumber
        '---------------------------------------------------------------------
        ' Create the unit tab page
        '---------------------------------------------------------------------
        Dim pInfoUnitTabPage As New TabPage()
        Dim unitTabPageDefined As Boolean = True
        '---------------------------------------------------------------------
        If unitTabPageDefined = True Then
            If (unit.flags And QCOM_UNIT_MODULE_SN_VALID) <> 0 Then
                pInfoUnitTabPage.Text = unit.moduleSerialNumber
            Else
                pInfoUnitTabPage.Text = String.Concat("Unit ", unitNumber)
            End If
            pInfoUnitTabPage.TabIndex = unitNumber
            pInfoUnitTabPage.BackgroundImage = whiteMarbleBackground
            pInfoTabPageArray(unitNumber) = pInfoUnitTabPage
            pInfoTabControl.Controls.Add(pInfoUnitTabPage)
            '-----------------------------------------------------------------
            ' Create the unit group box
            '-----------------------------------------------------------------
            Dim pInfoUnitGroupBox As New GroupBox()
            Dim unitGroupBoxDefined As Boolean = True
            '-----------------------------------------------------------------
            If unitGroupBoxDefined = True Then
                pInfoUnitGroupBox.Text = unit.unitDescriptionString
                pInfoUnitGroupBox.Location = New Point(10, 10)
                pInfoUnitGroupBox.Size = New Drawing.Size( _
                    pInfoTabControl.Width - 30, _
                    pInfoTabControl.Height - 50)
                pInfoUnitGroupBox.BackColor = Color.Transparent
                pInfoUnitTabPage.Controls.Add(pInfoUnitGroupBox)
                '-------------------------------------------------------------
                ' Logical unit number
                '-------------------------------------------------------------
                Dim pInfoLogicalUnitNumberLabel As New Label()
                pInfoLogicalUnitNumberLabel.Text = _
                    "Logical Unit Number: " + String.Concat(unit.unitNumber)
                pInfoLogicalUnitNumberLabel.Location = New Point(10, 20)
                pInfoLogicalUnitNumberLabel.Size = New Drawing.Size( _
                    148, GUI_INFO_LABEL_HEIGHT)
                pInfoLogicalUnitNumberLabel.BackColor = Color.Transparent
                pInfoUnitGroupBox.Controls.Add(pInfoLogicalUnitNumberLabel)
                '-------------------------------------------------------------
                ' Physical unit number
                '-------------------------------------------------------------
                Dim pInfoPhysicalUnitNumberLabel As New Label()
                pInfoPhysicalUnitNumberLabel.Text = _
                    "Physical Unit Number: " + String.Concat(unit.physicalUnitNumber)
                pInfoPhysicalUnitNumberLabel.Location = New Point( _
                    pInfoLogicalUnitNumberLabel.Right + 10, _
                    pInfoLogicalUnitNumberLabel.Top)
                pInfoPhysicalUnitNumberLabel.Size = pInfoLogicalUnitNumberLabel.Size
                pInfoPhysicalUnitNumberLabel.BackColor = Color.Transparent
                pInfoUnitGroupBox.Controls.Add(pInfoPhysicalUnitNumberLabel)
                '-------------------------------------------------------------
                ' Firmware ID
                '-------------------------------------------------------------
                Dim pInfoFirmwareIDLabel As New Label()
                pInfoFirmwareIDLabel.Text = "Firmware ID Bytes: "
                If (unit.flags And QCOM_UNIT_FIRMWARE_ID_PRESENT) <> 0 Then
                    pInfoFirmwareIDLabel.Text += String.Format( _
                        "{0:X2}  {1:X2}  {2:X2}  {3:X2}", _
                        unit.firmwareID(0), unit.firmwareID(1),
                        unit.firmwareID(2), unit.firmwareID(3))
                Else
                    pInfoFirmwareIDLabel.Text += QCOM_STRING_UNAVAILABLE
                End If
                pInfoFirmwareIDLabel.Location = New Point( _
                    pInfoPhysicalUnitNumberLabel.Right + 10, _
                    pInfoPhysicalUnitNumberLabel.Top)
                pInfoFirmwareIDLabel.Size = New Drawing.Size( _
                    188, GUI_INFO_LABEL_HEIGHT)
                pInfoFirmwareIDLabel.BackColor = Color.Transparent
                pInfoUnitGroupBox.Controls.Add(pInfoFirmwareIDLabel)
                '-------------------------------------------------------------
                ' Transducer part number
                '-------------------------------------------------------------
                Dim pInfoXDPartNumberLabel As New Label()
                pInfoXDPartNumberLabel.Text = "Transducer Part Number: "
                If (unit.flags And QCOM_UNIT_TRANSDUCER_PRESENT) <> 0 Then
                    pInfoXDPartNumberLabel.Text += unit.transducerPartNumber
                Else
                    pInfoXDPartNumberLabel.Text += QCOM_STRING_NA
                End If
                QCOM_PositionBelow(pInfoXDPartNumberLabel, pInfoLogicalUnitNumberLabel, 4)
                pInfoXDPartNumberLabel.Size = New Drawing.Size( _
                    240, GUI_INFO_LABEL_HEIGHT)
                pInfoUnitGroupBox.Controls.Add(pInfoXDPartNumberLabel)
                '-------------------------------------------------------------
                ' Transducer type
                '-------------------------------------------------------------
                Dim pInfoXDTypeLabel As New Label()
                pInfoXDTypeLabel.Text = "Transducer Type: "
                If (unit.flags And QCOM_UNIT_TRANSDUCER_PRESENT) <> 0 Then
                    pInfoXDTypeLabel.Text += String.Concat(unit.transducerType)
                Else
                    pInfoXDTypeLabel.Text += QCOM_STRING_NA
                End If
                pInfoXDTypeLabel.Location = New Point( _
                    pInfoXDPartNumberLabel.Right + 10, _
                    pInfoXDPartNumberLabel.Top)
                pInfoXDTypeLabel.Size = New Drawing.Size( _
                    120, GUI_INFO_LABEL_HEIGHT)
                pInfoXDTypeLabel.BackColor = Color.Transparent
                pInfoUnitGroupBox.Controls.Add(pInfoXDTypeLabel)
                '-------------------------------------------------------------
                ' Transducer memory type
                '-------------------------------------------------------------
                Dim pInfoMemTypeLabel As New Label()
                pInfoMemTypeLabel.Text = "Memory Type: "
                Dim memoryType As Byte = 0
                QD_GetMemoryType( _
                    unit.unitHandle, _
                    QD_DEVICE_TRANSDUCER, _
                    memoryType)
                If (unit.flags And QCOM_UNIT_TRANSDUCER_PRESENT) <> 0 Then
                    If memoryType <> 0 Then
                        pInfoMemTypeLabel.Text += String.Format("{0:D} ({1:D})", _
                            memoryType, _
                            unit.transducerMemorySize)
                    Else
                        If unit.transducerType <> QD_TRANSDUCER_TYPE_FREQUENCY Then
                            pInfoMemTypeLabel.Text += QCOM_STRING_ABSENT
                        Else
                            pInfoMemTypeLabel.Text += QCOM_STRING_NA
                        End If
                    End If
                Else
                    pInfoMemTypeLabel.Text += QCOM_STRING_NA
                End If
                pInfoMemTypeLabel.Location = New Point( _
                    pInfoXDTypeLabel.Right + 10, _
                    pInfoXDTypeLabel.Top)
                pInfoMemTypeLabel.Size = New Drawing.Size( _
                    124, GUI_INFO_LABEL_HEIGHT)
                pInfoMemTypeLabel.BackColor = Color.Transparent
                pInfoUnitGroupBox.Controls.Add(pInfoMemTypeLabel)
                '-------------------------------------------------------------
                ' Coefficient file pathname
                '-------------------------------------------------------------
                Dim pInfoCoefficientFilePathLabel As New Label()
                QCOM_PositionBelow(pInfoCoefficientFilePathLabel, pInfoXDPartNumberLabel, 4)
                pInfoCoefficientFilePathLabel.Size = New Drawing.Size( _
                    504, GUI_INFO_LABEL_HEIGHT)
                pInfoCoefficientFilePathLabel.Text = "Coefficient File: "
                If (unit.flags And QCOM_UNIT_COEFFICIENT_FILE_SPECIFIED) <> 0 Then
                    pInfoCoefficientFilePathLabel.Text += QCOM_LimitFilePathStringWidth( _
                        unit.coefficientFilePath, _
                        pInfoCoefficientFilePathLabel.Width - QCOM_StringWidth(pInfoCoefficientFilePathLabel.Text))
                Else
                    pInfoCoefficientFilePathLabel.Text += QCOM_STRING_NONE
                End If
                pInfoCoefficientFilePathLabel.BackColor = Color.Transparent
                pInfoUnitGroupBox.Controls.Add(pInfoCoefficientFilePathLabel)
                '-------------------------------------------------------------
                ' I²C Data Rate
                '-------------------------------------------------------------
                Dim pInfoDataRateLabel As New Label()
                pInfoDataRateLabel.Text = "I² C Data Rate: "
                If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
                    pInfoDataRateLabel.Text += String.Format( _
                        "{0:F1} kHz", _
                        unit.dataRate)
                Else
                    pInfoDataRateLabel.Text += QCOM_STRING_UNAVAILABLE
                End If
                pInfoDataRateLabel.Location = New Point( _
                    pInfoFirmwareIDLabel.Right + 10, _
                    pInfoFirmwareIDLabel.Top)
                pInfoDataRateLabel.Size = New Drawing.Size(220, GUI_INFO_LABEL_HEIGHT)
                pInfoDataRateLabel.BackColor = Color.Transparent
                pInfoUnitGroupBox.Controls.Add(pInfoDataRateLabel)
                '-------------------------------------------------------------
                ' Transducer Chip ID
                '-------------------------------------------------------------
                Dim pInfoXDChipIDLabel As New Label()
                pInfoXDChipIDLabel.Text = "Transducer Chip ID Bytes: "
                If (unit.flags And QCOM_UNIT_TRANSDUCER_PRESENT) <> 0 Then
                    If unit.transducerChipID(0) = QD_QUARTZDYNE_ID Then
                        pInfoXDChipIDLabel.Text += String.Format( _
                            "{0:X2}  {1:X2}  {2:X2}  {3:X2}", _
                            unit.transducerChipID(0), unit.transducerChipID(1),
                            unit.transducerChipID(2), unit.transducerChipID(3))
                    Else
                        pInfoXDChipIDLabel.Text += QCOM_STRING_INVALID
                    End If
                Else
                    pInfoXDChipIDLabel.Text += QCOM_STRING_NA
                End If
                QCOM_PositionAndSizeBelow(pInfoXDChipIDLabel, pInfoDataRateLabel, 4)
                pInfoUnitGroupBox.Controls.Add(pInfoXDChipIDLabel)
                '-------------------------------------------------------------
                ' Unit flags
                '-------------------------------------------------------------
                Dim pInfoUnitFlagsLabel As New Label()
                pInfoUnitFlagsLabel.Text = "Unit Flags: " + unit.flags.ToString("X8")
                QCOM_PositionAndSizeBelow(pInfoUnitFlagsLabel, pInfoXDChipIDLabel, 4)
                pInfoUnitGroupBox.Controls.Add(pInfoUnitFlagsLabel)
            End If                      ' If unitGroupBoxDefined = True Then
        End If                          ' If unitTabPageDefined = True Then
        Return Nothing
    End Function                        ' QCOM_DisplayProgramInformationTab
    '-------------------------------------------------------------------------
    ' QCOM_EstablishLogDirectory
    '
    ' Ensures the QLog directory exists in the current working directory, and
    ' stores the directory path string
    '
    ' Called by:    QCOM_EnsureMinimumEnvironment
    '-------------------------------------------------------------------------
    Public Function QCOM_EstablishLogDirectory()
        If QCOM_GeneralInfo.logDirectory = Nothing Then
            Dim pathString As String = Application.StartupPath + "\" + QCOM_LOG_DIRECTORY
            If Directory.Exists(pathString) = True Then
                QCOM_GeneralInfo.logDirectory = pathString
            Else
                Directory.CreateDirectory(pathString)
                Thread.Sleep(100)
                If Directory.Exists(pathString) = True Then
                    QCOM_GeneralInfo.logDirectory = pathString
                Else
                    QCOM_GeneralInfo.logDirectory = Nothing
                End If                  ' If Directory.Exists(pathString) = True Then
            End If                      ' If Directory.Exists(pathString) = True Then
        End If                          ' If QCOM_GeneralInfo.logDirectory = Nothing Then
        Return Nothing
    End Function                        ' QCOM_EstablishLogDirectory
    '-------------------------------------------------------------------------
    ' QCOM_GetModuleInfo
    '
    ' Retrieves the serial number of the specified module, determines whether
    ' the module is currently in Boot Loader mode, and sets the appropriate
    ' flags as a result
    '
    ' Called by:    QCOM_ScanForDevices
    '               QCOM_SwitchFromBootLoaderMode
    '               QCOM_SwitchToBootLoaderMode
    '
    ' Note:     This function can be called before a device is open for I/O,
    '           so it cannot assume that a handle to the device is available
    '-------------------------------------------------------------------------
    Public Function QCOM_GetModuleInfo(ByRef unit As UnitInfo)
        Dim invalidModule As Boolean = False
        Dim moduleSerialNumber(QD_MAXIMUM_SERIAL_NUMBER_SIZE) As Byte
        Dim productInfo(QD_MAXIMUM_PRODUCT_STRING_SIZE) As Byte
        Dim productInfoString As String = Nothing
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        ' Retrieve the serial number of this module
        '---------------------------------------------------------------------
        status = QD_GetModuleSerialNumber(unit.physicalUnitNumber, moduleSerialNumber(0))
        If status = QCOM_SUCCESS Then
            '-----------------------------------------------------------------
            ' This is a valid unit, complete with a module serial number
            '-----------------------------------------------------------------
            unit.moduleSerialNumber = QCOM_ConvertCString(moduleSerialNumber)
            unit.flags = (unit.flags Or QCOM_UNIT_MODULE_SN_VALID)
            '-----------------------------------------------------------------
            ' Retrieve the PID of this driver
            '-----------------------------------------------------------------
            status = QD_GetProductInfo(unit.physicalUnitNumber, productInfo(0), QD_RETURN_PID)
            If status = QCOM_SUCCESS Then
                productInfoString = QCOM_ConvertCString(productInfo)
                '-------------------------------------------------------------
                ' Make sure this PID matches the expected one
                '-------------------------------------------------------------
                If String.Compare(productInfoString, QCOM_PRODUCT_ID_STRING, True) = 0 Then
                    unit.flags = (unit.flags Or QCOM_UNIT_VALID)
                    QCOM_GeneralInfo.validUnitBitMap = (QCOM_GeneralInfo.validUnitBitMap Or (1 << unit.unitNumber))
                    '---------------------------------------------------------
                    ' Check for Boot Loader Mode
                    '---------------------------------------------------------
                    If unit.moduleSerialNumber.EndsWith("~") = True Then
                        unit.flags = (unit.flags Or QCOM_UNIT_BOOT_LOADER_MODE)
                    Else
                        unit.flags = (unit.flags And Not QCOM_UNIT_BOOT_LOADER_MODE)
                        unit.moduleMemorySize = 8192
                        unit.numberOfModuleMemoryPages = unit.moduleMemorySize / QD_COEFFICIENT_DATA_SIZE
                    End If              ' If unit.moduleSerialNumber.EndsWith("~") = True Then
                End If                  ' If String.Compare(productInfoString, QCOM_PRODUCT_ID_STRING, True) = 0 Then
            Else
                invalidModule = True
            End If                      ' If status = QCOM_SUCCESS Then
        Else
            invalidModule = True
        End If                          ' If status = QCOM_SUCCESS Then
        Erase productInfo
        Erase moduleSerialNumber
        If invalidModule = True Then
            unit.flags = (unit.flags And Not QCOM_UNIT_VALID)
            unit.moduleSerialNumber = Nothing
            unit.unitDescriptionString = Nothing
        End If                          ' If invalidModule = True Then
        Return Nothing
    End Function                        ' QCOM_GetModuleInfo
    '-------------------------------------------------------------------------
    ' QCOM_GetNumberOfTransducers
    '
    ' Returns the number of physical transducers detected in the system
    '
    ' Called by:    QCOM_ScanForDevices
    '-------------------------------------------------------------------------
    Public Function QCOM_GetNumberOfTransducers() As UInt32
        Dim attempts As Integer = 3
        Dim numberOfTransducers As UInt32 = 0
        Dim status As UInt32 = QCOM_SUCCESS
        Dim statusRegister As Byte = 0
        '---------------------------------------------------------------------
        For Each unit As UnitInfo In QCOM_UnitInfoArray
            If (unit.flags And QCOM_UNIT_VALID) <> 0 AndAlso (unit.flags And QCOM_UNIT_OPEN) <> 0 Then
                While attempts <> 0
                    attempts -= 1
                    status = QD_GetModuleStatusRegister(unit.unitHandle, statusRegister)
                    If status = QCOM_SUCCESS Then
                        If (statusRegister And (QD_SREG_TRANSDUCER_PRESENT Or QD_SREG_TRANSDUCER_TYPE_MASK)) <> 0 Then
                            numberOfTransducers += 1
                            attempts = 0
                            Exit While
                        End If
                    Else
                        If attempts <> 0 Then
                            status = QCOM_SUCCESS
                            Thread.Sleep(100)
                        End If
                    End If              ' If status = QCOM_SUCCESS Then
                End While               ' While attempts <> 0
            End If                      ' If (unit.flags And QCOM_UNIT_VALID) <> 0 AndAlso ...
        Next
        Return numberOfTransducers
    End Function                        ' QCOM_GetNumberOfTransducers
    '-------------------------------------------------------------------------
    ' QCOM_GetOperatingEnvironment
    '
    ' Retrieves information about the client operating environment
    '
    ' Called by:    QCOM_EnsureMinimumEnvironment
    '-------------------------------------------------------------------------
    Public Function QCOM_GetOperatingEnvironment()
        Dim plurality As String = String.Empty
        '---------------------------------------------------------------------
        ' Retrieve the Windows version of the client
        '---------------------------------------------------------------------
        QCOM_GetWindowsVersion()
        '---------------------------------------------------------------------
        ' Retrieve the client user and machine name
        '---------------------------------------------------------------------
        'Msg("Operator {0} is using client machine {1} ({2:D} processor{3})", _
        '    Environment.UserName, _
        '    Environment.MachineName, _
        '    Environment.ProcessorCount, _
        '    If(Environment.ProcessorCount <> 1, "s", ""))
        Return Nothing
    End Function                        ' QCOM_GetOperatingEnvironment
    '-------------------------------------------------------------------------
    ' QCOM_GetTransducerInfo
    '
    ' Retrieves various information about the attached transducer, and returns
    ' the transducer type
    '
    ' Returns the transducer type
    '
    ' Called by:    QCOM_ScanForDevices
    '-------------------------------------------------------------------------
    Public Function QCOM_GetTransducerInfo(ByRef unit As UnitInfo) As UInt32
        Dim commandString() As Byte
        Dim rawCommandString As String
        Dim replyString(QCOM_MAXIMUM_I2C_REPLY_STRING_SIZE) As Byte
        Dim memoryType As Byte = 0
        Dim statusRegister As Byte = 0
        Dim transducerChipID(QD_TRANSDUCER_CHIP_ID_LENGTH) As Integer
        Dim transducerType As Byte = QD_TRANSDUCER_TYPE_ABSENT
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        If (unit.flags And QCOM_UNIT_VALID) <> 0 AndAlso (unit.flags And QCOM_UNIT_OPEN) <> 0 Then
            status = QD_GetModuleStatusRegister(unit.unitHandle, statusRegister)
            If status = QCOM_SUCCESS Then
                If (statusRegister And QD_SREG_TRANSDUCER_POWER_STATE) <> 0 Then
                    unit.flags = (unit.flags Or QCOM_UNIT_TRANSDUCER_POWER_ENABLED)
                Else
                    status = QD_SetTransducerPowerState( _
                        unit.unitHandle, _
                        QD_ENABLE_TRANSDUCER_POWER)
                End If                  ' If (statusRegister And QD_SREG_TRANSDUCER_POWER_STATE) <> 0 Then
            End If                      ' If status = QCOM_SUCCESS Then
            If (statusRegister And (QD_SREG_TRANSDUCER_PRESENT Or QD_SREG_TRANSDUCER_TYPE_MASK)) <> 0 Then
                If (statusRegister And QD_SREG_TRANSDUCER_PRESENT) <> 0 AndAlso _
                    (statusRegister And QD_SREG_TRANSDUCER_TYPE_MASK) <> 0 Then
                    unit.flags = (unit.flags Or QCOM_UNIT_TRANSDUCER_PRESENT)
                    transducerType = statusRegister And QD_SREG_TRANSDUCER_TYPE_MASK
                    unit.transducerType = transducerType
                    If (unit.flags And QCOM_UNIT_TRANSDUCER_POWER_ENABLED) <> 0 Then
                        '-----------------------------------------------------
                        ' Send an I²C command to retrieve the transducer chip
                        ' ID (without the checksum, because at this point it's
                        ' unknown whether the embedded ASIC/FPGA supports the
                        ' checksum return)
                        '-----------------------------------------------------
                        If (unit.transducerType <> QD_TRANSDUCER_TYPE_FREQUENCY) _
                            AndAlso (unit.transducerType <> QD_TRANSDUCER_TYPE_DIGITAL_NOMEM) Then
                            rawCommandString = "S9CR9DFFFFFFFFP"
                        Else
                            rawCommandString = "S98R99FFFFFFFFP"
                        End If          ' If (unit.transducerType <> QD_TRANSDUCER_TYPE_FREQUENCY) AndAlso ...
                        commandString = QCOM_ConvertVBString(rawCommandString)
                        status = QD_ExecuteI2CCommand( _
                            unit.unitHandle, _
                            commandString(0), _
                            replyString(0))
                        If status = QCOM_SUCCESS Then
                            If (replyString(14) = 80) AndAlso (replyString(15) = 13) AndAlso (replyString(16) = 10) Then
                                For index As Integer = 0 To QD_TRANSDUCER_CHIP_ID_LENGTH - 1
                                    Dim highByte As Integer = replyString(6 + (index * 2))
                                    Dim lowByte As Integer = replyString(7 + (index * 2))
                                    If (highByte >= 48) AndAlso (highByte <= 57) Then
                                        highByte -= 48
                                    Else
                                        highByte -= 55
                                    End If
                                    If (lowByte >= 48) AndAlso (lowByte <= 57) Then
                                        lowByte -= 48
                                    Else
                                        lowByte -= 55
                                    End If
                                    transducerChipID(index) = (highByte << 4) Or (lowByte)
                                Next    ' For index As Integer = 0 To QD_TRANSDUCER_CHIP_ID_LENGTH - 1
                                unit.transducerChipID = transducerChipID
                                'Msg("XD ID = {0} - {1} - {2} - {3}",
                                'String.Format("{0:X2}", transducerChipID(0)), _
                                'String.Format("{0:X2}", transducerChipID(1)), _
                                'String.Format("{0:X2}", transducerChipID(2)), _
                                'String.Format("{0:X2}", transducerChipID(3)))
                                If unit.transducerChipID(0) = QD_QUARTZDYNE_ID Then
                                    QD_GetMemoryType( _
                                        unit.unitHandle, _
                                        QD_DEVICE_TRANSDUCER, _
                                        memoryType)
                                    If memoryType <> QD_MEMORY_TYPE_ABSENT Then
                                        '-------------------------------------
                                        ' Hard-code these for now
                                        '-------------------------------------
                                        unit.transducerMemorySize = 8192
                                        unit.numberOfTransducerMemoryPages = _
                                            unit.transducerMemorySize / QD_COEFFICIENT_DATA_SIZE
                                    End If
                                Else
                                    Msg("Firmware is invalid")
                                End If  ' If unit.transducerChipID(0) = QD_QUARTZDYNE_ID Then
                            End If      ' If (replyString(14) = 80) AndAlso ...
                        Else
                            EMsg("QD_ExecuteI2CCommand({0:D}) returned 0x{1}", unit.unitNumber, status.ToString("X2"))
                        End If          ' If status = QCOM_SUCCESS Then
                    Else
                        EMsg("Module {0} is unable to apply power to the transducer", _
                            unit.moduleSerialNumber)
                    End If              ' If (unit.flags And QCOM_UNIT_TRANSDUCER_POWER_ENABLED) <> 0 Then
                Else
                    'EMsg("GTI({0:D}): sReg = 0x{1} and XD type = 0x{2}", unit.unitNumber, statusRegister.ToString("X2"), unit.transducerType.ToString("X2"))
                End If                  ' If (statusRegister And QD_SREG_TRANSDUCER_PRESENT) <> 0 AndAlso ...
            Else
                EMsg("QD_GetModuleStatusRegister returned register value 0x{0}\n" + _
                    "No transducer detected attached to the module", _
                    statusRegister.ToString("X2"))
            End If                      ' If (statusRegister And (QD_SREG_TRANSDUCER_PRESENT Or QD_SREG_TRANSDUCER_TYPE_MASK)) <> 0 Then
        End If                          ' If (unit.flags And QCOM_UNIT_VALID) <> 0 AndAlso ...
        Return transducerType
    End Function                        ' QCOM_GetTransducerInfo
    '-------------------------------------------------------------------------
    ' QCOM_GetWindowsVersion
    '
    ' Discovers the currently running Windows version and sets the general
    ' QCOM_GeneralInfo.windowsVersion string accordingly
    '
    ' Called by:    QCOM_GetOperatingEnvironment
    '-------------------------------------------------------------------------
    Public Function QCOM_GetWindowsVersion()
        Dim isServerOS = False
        Dim versionString As String = Nothing
        Dim status As UInt32
        '---------------------------------------------------------------------
        Dim OSVersionInfo As New OSVERSIONINFOEX()
        OSVersionInfo.OSVersionInfoSize = Marshal.SizeOf(OSVersionInfo)
        status = GetVersionExA(OSVersionInfo)
        If status <> 0 Then
            Select Case OSVersionInfo.MajorVersion
                Case 5
                    Select Case OSVersionInfo.MinorVersion
                        Case 0
                            versionString = "Windows 2000"
                        Case 1
                            versionString = "Windows XP"
                        Case 2
                            If GetSystemMetrics(SM_SERVERR2) = 0 Then
                                versionString = "Windows Server 2003"
                            Else
                                versionString = "Windows Server 2003 R2"
                            End If
                            isServerOS = True
                        Case Else
                            versionString = QCOM_STRING_UNSUPPORTED
                    End Select          ' Select Case OSVersionInfo.MinorVersion
                Case 6
                    Select Case OSVersionInfo.MinorVersion
                        Case 0
                            If OSVersionInfo.ProductType = VER_NT_WORKSTATION Then
                                versionString = "Windows Vista"
                            Else
                                versionString = "Windows Server 2008"
                                isServerOS = True
                            End If
                        Case 1
                            If OSVersionInfo.ProductType = VER_NT_WORKSTATION Then
                                versionString = "Windows 7"
                            Else
                                versionString = "Windows Server 2008 R2"
                                isServerOS = True
                            End If
                        Case 2
                            If OSVersionInfo.ProductType = VER_NT_WORKSTATION Then
                                versionString = "Windows 8"
                            Else
                                versionString = "Windows Server 8"
                                isServerOS = True
                            End If
                        Case Else
                            versionString = QCOM_STRING_UNSUPPORTED
                    End Select          ' Select Case OSVersionInfo.MinorVersion
                Case Else
                    versionString = QCOM_STRING_UNSUPPORTED
            End Select                  ' Select Case OSVersionInfo.MajorVersion
            '-----------------------------------------------------------------
            ' Determine the service pack version, if available
            '-----------------------------------------------------------------
            If OSVersionInfo.CSDVersion.Length > 0 AndAlso _
                isServerOS = False Then
                Dim SPString As String = OSVersionInfo.CSDVersion
                If SPString.Contains("Service Pack") = True Then
                    SPString = SPString.Replace("Service Pack", "SP")
                End If
                QCOM_GeneralInfo.windowsVersion = String.Format( _
                    "{0} {1}-bit with {2}", _
                    versionString, _
                    If(Environment.Is64BitOperatingSystem = True, "64", "32"), _
                    SPString)
            Else
                QCOM_GeneralInfo.windowsVersion = String.Format( _
                    "{0} {1}-bit", _
                    versionString, _
                    If(Environment.Is64BitOperatingSystem = True, "64", "32"))
            End If                      ' If OSVersionInfo.CSDVersion.Length > 0 AndAlso _
        Else
            EMsg("status = {0:D}", Err.LastDllError)
        End If                          ' If status <> 0 Then
        Return Nothing
    End Function                        ' QCOM_GetWindowsVersion
    '-------------------------------------------------------------------------
    ' QCOM_HelpAbout
    '
    ' Displays a small window that describes the QCOM software and version,
    ' plus provides some links to Quartzdyne websites
    '
    ' Called by:    QCOM_InstallToolStrip
    '-------------------------------------------------------------------------
    Public Function QCOM_HelpAbout()
        Dim lineLength As Integer = 0
        '---------------------------------------------------------------------
        aboutHelpWindow = New Form()
        aboutHelpWindow.Icon = QCOM_SoftwareIcon
        aboutHelpWindow.BackgroundImage = whiteSandBackground
        aboutHelpWindow.Text = "About QCOM"
        aboutHelpWindow.Size = New Drawing.Size(440, 208)
        aboutHelpWindow.FormBorderStyle = System.Windows.Forms.FormBorderStyle.Fixed3D
        aboutHelpWindow.StartPosition = FormStartPosition.CenterScreen
        aboutHelpWindow.MaximizeBox = False
        aboutHelpWindow.HelpButton = False
        Dim QCOMLogo As Image = Image.FromFile(QCOM_PROGRAM_LOGO)
        Dim logoLabel As New Label()
        logoLabel.Location = New Point(aboutHelpWindow.Width - 110, 10)
        logoLabel.Size = New Drawing.Size(QCOMLogo.Width, QCOMLogo.Height)
        logoLabel.Image = QCOMLogo
        logoLabel.BackColor = Color.Transparent
        logoLabel.Cursor = Cursors.Hand
        AddHandler logoLabel.Click, AddressOf QCOM_OnlineHelpClicked
        aboutHelpWindow.Controls.Add(logoLabel)
        lineLength = aboutHelpWindow.Width - 20 - QCOMLogo.Width
        '---------------------------------------------------------------------
        ' Display the program name and version
        '---------------------------------------------------------------------
        Dim helpLine1Label As New Label()
        helpLine1Label.Text = String.Concat("QCOM, Version ", QCOM_PROGRAM_VERSION_STRING)
        helpLine1Label.Font = New Drawing.Font( _
            FontFamily.GenericSansSerif, _
            12.0F, _
            FontStyle.Bold)
        helpLine1Label.Location = New Point(10, 10)
        helpLine1Label.Size = New Drawing.Size(lineLength, GUI_LARGE_LABEL_HEIGHT)
        helpLine1Label.BackColor = Color.Transparent
        aboutHelpWindow.Controls.Add(helpLine1Label)
        '---------------------------------------------------------------------
        ' Display the company name and copyright
        '---------------------------------------------------------------------
        Dim helpLine2Label As New Label()
        helpLine2Label.Text = "Copyright © 2013 Quartzdyne, Inc."
        helpLine2Label.Font = New Drawing.Font( _
            FontFamily.GenericSansSerif, _
            12.0F, _
            FontStyle.Bold)
        QCOM_PositionAndSizeBelow(helpLine2Label, helpLine1Label, 6)
        aboutHelpWindow.Controls.Add(helpLine2Label)
        '---------------------------------------------------------------------
        ' Display the company URL
        '---------------------------------------------------------------------
        Dim helpLine3LinkLabel As New LinkLabel()
        helpLine3LinkLabel.Text = "Visit us at www.quartzdyne.com"
        helpLine3LinkLabel.Font = New Drawing.Font( _
            FontFamily.GenericSansSerif, _
            12.0F, _
            FontStyle.Bold)
        helpLine3LinkLabel.LinkArea = New LinkArea(12, 18)
        helpLine3LinkLabel.AutoSize = True
        helpLine3LinkLabel.LinkColor = Color.Blue
        helpLine3LinkLabel.DisabledLinkColor = Color.Red
        helpLine3LinkLabel.VisitedLinkColor = Color.Purple
        helpLine3LinkLabel.LinkBehavior = LinkBehavior.HoverUnderline
        helpLine3LinkLabel.Links(0).LinkData = QUARTZDYNE_URL
        QCOM_PositionAndSizeBelow(helpLine3LinkLabel, helpLine2Label, 6)
        AddHandler helpLine3LinkLabel.LinkClicked, AddressOf QCOM_InternetHyperlinkClicked
        aboutHelpWindow.Controls.Add(helpLine3LinkLabel)
        '---------------------------------------------------------------------
        ' Display the remaining text
        '---------------------------------------------------------------------
        Dim helpLine4Label As New Label()
        helpLine4Label.Text = "Transducer information communication"
        helpLine4Label.Font = New Drawing.Font( _
            FontFamily.GenericSansSerif, _
            12.0F, _
            FontStyle.Bold)
        QCOM_PositionAndSizeBelow(helpLine4Label, helpLine3LinkLabel, 32)
        aboutHelpWindow.Controls.Add(helpLine4Label)
        Dim helpLine5Label As New Label()
        helpLine5Label.Text = "software for QCOM modules"
        helpLine5Label.Font = New Drawing.Font( _
            FontFamily.GenericSansSerif, _
            12.0F, _
            FontStyle.Bold)
        QCOM_PositionAndSizeBelow(helpLine5Label, helpLine4Label, 6)
        aboutHelpWindow.Controls.Add(helpLine5Label)
        '---------------------------------------------------------------------
        ' Add an OK button
        '---------------------------------------------------------------------
        Dim okButton As New Button()
        okButton.Text = "OK"
        okButton.Location = New Point( _
            aboutHelpWindow.Width - 80, _
            aboutHelpWindow.Height - 66)
        okButton.Size = New Drawing.Size(60, GUI_REGULAR_BUTTON_HEIGHT)
        okButton.BackColor = Color.Transparent
        okButton.Cursor = Cursors.Hand
        AddHandler okButton.Click, AddressOf QCOM_HelpCloseWindow
        aboutHelpWindow.Controls.Add(okButton)
        aboutHelpWindow.AcceptButton = okButton
        aboutHelpWindow.CancelButton = okButton
        '---------------------------------------------------------------------
        ' Display the new window
        '---------------------------------------------------------------------
        aboutHelpWindow.ShowDialog()
        Return Nothing
    End Function                        ' QCOM_HelpAbout
    '-------------------------------------------------------------------------
    ' QCOM_I2CChecksumVerified
    '
    ' Verifies that the fifth byte in the array of BYTEs is the correct checksum
    ' of the preceding four BYTEs, with each BYTE being represented by two
    ' hexadecimal Bytes
    '
    ' Called by:    QCOM_
    '-------------------------------------------------------------------------
    Public Function QCOM_I2CChecksumVerified(ByVal fiveByteArray() As Byte) As Boolean
        Dim checksumVerified As Boolean = False
        Dim highByte As Integer
        Dim lowByte As Integer
        Dim calculatedChecksum As Integer = 0
        Dim readChecksum As Integer
        '---------------------------------------------------------------------
        For offset As Integer = 0 To 3
            highByte = If(fiveByteArray(offset * 2) > &H39, fiveByteArray(offset * 2) - 55, fiveByteArray(offset * 2) - 48)
            lowByte = If(fiveByteArray((offset * 2) + 1) > &H39, fiveByteArray((offset * 2) + 1) - 55, fiveByteArray((offset * 2) + 1) - 48)
            calculatedChecksum += ((highByte << 4) Or lowByte)
        Next                            ' For offset As Integer = 0 To 3
        '---------------------------------------------------------------------
        ' Keep only the lowest byte as the calculated checksum
        '---------------------------------------------------------------------
        calculatedChecksum = (calculatedChecksum And &HFF)
        highByte = If(fiveByteArray(8) > &H39, fiveByteArray(8) - 55, fiveByteArray(8) - 48)
        lowByte = If(fiveByteArray(9) > &H39, fiveByteArray(9) - 55, fiveByteArray(9) - 48)
        readChecksum = (((highByte << 4) Or lowByte) And &HFF)
        'Msg("calc = {0} and read = {1}", calculatedChecksum.ToString("X2"), readChecksum.ToString("X2"))
        '---------------------------------------------------------------------
        ' If the calculated checksum and the returned (fifth-byte) checksum
        ' add to zero, then the returned checksum appears correct
        '---------------------------------------------------------------------
        If ((calculatedChecksum + readChecksum) And &HFF) = 0 Then
            '-----------------------------------------------------------------
            ' Perform a redundant check for checksum confirmation by comparing
            ' the two's complement of the calculated checksum with the
            ' returned checksum
            '-----------------------------------------------------------------
            calculatedChecksum = (((Not calculatedChecksum) + 1) And &HFF)
            If calculatedChecksum = readChecksum Then
                '-------------------------------------------------------------
                ' The checksum is confirmed
                '-------------------------------------------------------------
                checksumVerified = True
                'Msg("The checksum of {0} is good", readChecksum.ToString("X2"))
            End If
        End If
        Return checksumVerified
    End Function                        ' QCOM_I2CChecksumVerified
    '-------------------------------------------------------------------------
    ' QCOM_InterpretCoefficientData
    '
    ' Parses select fields of the coefficient data for transducer information
    ' and populates the appropriate transducer fields
    '
    ' Called by:    QCOM_ApplyCoefficientDataInfoToProgram
    '-------------------------------------------------------------------------
    Public Function QCOM_InterpretCoefficientData(ByRef unit As UnitInfo)
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
            '-----------------------------------------------------------------
            ' Interpret the bytes from the little-endian value into a string
            '-----------------------------------------------------------------
            unit.transducerSerialNumber = Nothing
            For offset As Integer = 5 To 7
                unit.transducerSerialNumber += String.Format( _
                    "{0:X2}", unit.coefficientData(offset))
            Next
            If Convert.ToInt32(unit.transducerSerialNumber) > 0 Then
                unit.flags = (unit.flags Or QCOM_UNIT_TRANSDUCER_SN_VALID)
            End If
            '-----------------------------------------------------------------
            ' The part number must be converted from ASCII values to text
            '-----------------------------------------------------------------
            unit.transducerPartNumber = Nothing
            For offset As Integer = 8 To 15
                If unit.coefficientData(offset) <> 0 Then
                    unit.transducerPartNumber += Chr(unit.coefficientData(offset))
                Else
                    Exit For
                End If
            Next
            'Msg("XDSN = {0} and XDPN = {1}", unit.transducerSerialNumber, unit.transducerPartNumber)
        End If                          ' If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
        Return Nothing
    End Function                        ' QCOM_InterpretCoefficientData
    '-------------------------------------------------------------------------
    ' QCOM_LimitFilePathStringWidth
    '
    ' Parses select fields of the coefficient data for transducer information
    ' and populates the appropriate transducer fields
    '
    ' Called by:    QCOM_ApplyCoefficientDataInfoToProgram
    '-------------------------------------------------------------------------
    Public Function QCOM_LimitFilePathStringWidth(ByVal filePath As String, ByVal maximumWidth As UInt32) As String
        Dim minimumSpaceWidths As UInt32 = 28
        '---------------------------------------------------------------------
        If Environment.Is64BitOperatingSystem = True Then
            minimumSpaceWidths += 10
        End If
        If maximumWidth > minimumSpaceWidths Then
            '-----------------------------------------------------------------
            ' Subtract the pixels taken by padding, spaces, and a colon
            '-----------------------------------------------------------------
            maximumWidth -= minimumSpaceWidths
            If QCOM_StringWidth(filePath) > maximumWidth Then
                '-------------------------------------------------------------
                ' Insert the ellipsis at the front of the string
                '-------------------------------------------------------------
                filePath = filePath.PadLeft(filePath.Length + 3, ".")
                While QCOM_StringWidth(filePath.Substring(3)) > maximumWidth
                    If filePath.Contains("\") = True Then
                        If filePath(3) = "\" Then
                            If filePath.Substring(4).Contains("\") = True Then
                                filePath = filePath.Remove(4, filePath.Substring(4).IndexOf("\") + 1)
                            Else
                                filePath = filePath.Remove(3, 1)
                            End If      ' If filePath.Substring(4).Contains("\") = True Then
                        Else
                            If filePath.IndexOf("\") > 3 Then
                                filePath = filePath.Remove(3, filePath.IndexOf("\") - 3)
                            Else
                                filePath = filePath.Remove(3, 1)
                            End If      ' If filePath.IndexOf("\") > 3 Then
                        End If          ' If filePath(3) = "\" Then
                    Else
                        '-----------------------------------------------------
                        ' No more back-slashes in the string to reference, so
                        ' simply remove enough characters from the front of the
                        ' string, except for the ellipsis, to limit its width
                        '-----------------------------------------------------
                        While QCOM_StringWidth(filePath.Substring(3)) > maximumWidth
                            filePath = filePath.Remove(3, 1)
                        End While
                    End If              ' If filePath.Contains("\") = True Then
                End While               ' While QCOM_StringWidth(filePath.Substring(3)) > maximumWidth
            End If                      ' If QCOM_StringWidth(filePath) > maximumWidth Then
        End If                          ' If maximumWidth > minimumSpaceWidths Then
        Return filePath
    End Function                        ' QCOM_LimitFilePathStringWidth
    '-------------------------------------------------------------------------
    ' QCOM_OpenUnit
    '
    ' Opens the specified unit for I/O
    '
    ' Returns: 0            Success
    '          nonzero      Failure
    '-------------------------------------------------------------------------
    Public Function QCOM_OpenUnit(ByRef unit As UnitInfo) As UInt32
        Dim attempts As Integer = 10
        Dim status As UInt32 = QCOM_SUCCESS
        Dim unitHandle As Integer = INVALID_HANDLE_VALUE
        '---------------------------------------------------------------------
        Thread.Sleep(100)
        If unit.unitHandle = INVALID_HANDLE_VALUE Then
            While attempts <> 0
                attempts -= 1
                status = QD_Open(unit.physicalUnitNumber, unitHandle)
                If (status = QCOM_SUCCESS) AndAlso (unitHandle <> INVALID_HANDLE_VALUE) Then
                    unit.unitHandle = unitHandle
                    unit.flags = (unit.flags Or QCOM_UNIT_OPEN)
                    attempts = 0
                    Exit While
                Else
                    If attempts <> 0 Then
                        status = QCOM_SUCCESS
                        Thread.Sleep(100)
                    Else
                        If status = QCOM_SUCCESS Then
                            If unitHandle = INVALID_HANDLE_VALUE Then
                                status = QCOM_ERROR_INVALID_HANDLE
                            Else
                                status = QCOM_ERROR_UNREACHABLE_CONDITION
                            End If
                        End If          ' If status = QCOM_SUCCESS Then
                    End If              ' If attempts <> 0 Then
                End If                  ' If (status = QCOM_SUCCESS) AndAlso (unitHandle <> INVALID_HANDLE_VALUE) Then
            End While                   ' While attempts <> 0
        End If                          ' If unit.unitHandle = INVALID_HANDLE_VALUE Then
        Return status
    End Function                        ' QCOM_OpenUnit
    '-------------------------------------------------------------------------
    ' QCOM_PlaySound
    '
    ' Plays the sound of the specified WAV file
    '-------------------------------------------------------------------------
    Public Function QCOM_PlaySound(ByVal soundFile As Media.SoundPlayer)
        If (QCOM_GeneralInfo.flags And QCOM_GENERAL_SOUNDS_ENABLED) <> 0 Then
            soundFile.Play()
        Else
            soundFile.Stop()
        End If
        Return Nothing
    End Function                        ' QCOM_PlaySound
    '-------------------------------------------------------------------------
    ' QCOM_PleaseWaitDisplay
    '
    ' Displays a "Please wait . . ." window
    '
    ' Called by:    QCOM_InitializeProgramComponents
    '-------------------------------------------------------------------------
    Public Function QCOM_PleaseWaitDisplay(ByVal installWindow As Boolean)
        If installWindow = True Then
            pleaseWaitWindow = New Form()
            pleaseWaitWindow.Text = "Searching for Devices"
            pleaseWaitWindow.Size = New Drawing.Size(200, 100)
            pleaseWaitWindow.TopMost = True
            pleaseWaitWindow.Icon = QCOM_SoftwareIcon
            pleaseWaitWindow.BackgroundImage = whiteSandBackground
            pleaseWaitWindow.FormBorderStyle = System.Windows.Forms.FormBorderStyle.Fixed3D
            pleaseWaitWindow.StartPosition = FormStartPosition.CenterScreen
            pleaseWaitWindow.MaximizeBox = False
            pleaseWaitWindow.MinimizeBox = False
            pleaseWaitWindow.HelpButton = False
            pleaseWaitLabel = New Label()
            pleaseWaitLabel.Text = "Please wait . . ."
            pleaseWaitLabel.Size = New Drawing.Size(130, 20)
            pleaseWaitLabel.Location = New Point( _
                ((pleaseWaitWindow.Width - pleaseWaitLabel.Width) / 2), 22)
            pleaseWaitLabel.Font = New Drawing.Font( _
                FontFamily.GenericSansSerif, _
                12.0F, _
                FontStyle.Bold)
            pleaseWaitLabel.BackColor = Color.Transparent
            pleaseWaitWindow.Controls.Add(pleaseWaitLabel)
            AddHandler pleaseWaitWindow.FormClosing, AddressOf QCOM_PleaseWaitClosingWindow
        End If                          ' If installWindow = True Then
        pleaseWaitWindow.Show()
        pleaseWaitWindow.Update()
        Return Nothing
    End Function                        ' QCOM_PleaseWaitDisplay
    '-------------------------------------------------------------------------
    ' QCOM_PleaseWaitRemove
    '
    ' Removes (hides) the "Please wait . . ." window
    '
    ' Called by:    QCOM_ConstructAndDisplayUserInterface
    '-------------------------------------------------------------------------
    Public Function QCOM_PleaseWaitRemove()
        pleaseWaitWindow.Hide()
        pleaseWaitWindow.Update()
        Return Nothing
    End Function                        ' QCOM_PleaseWaitRemove
    '-------------------------------------------------------------------------
    ' QCOM_PositionAndSizeBelow
    '
    ' Sets the Location and Size properties of the specified object in
    ' reference to another object
    '-------------------------------------------------------------------------
    Public Function QCOM_PositionAndSizeBelow( _
        ByRef targetObject As Object, _
        ByVal referenceObject As Object, _
        ByVal spacing As Integer)
        QCOM_PositionBelow(targetObject, referenceObject, spacing)
        targetObject.Size = referenceObject.Size
        Return Nothing
    End Function                        ' QCOM_PositionAndSizeBelow
    '-------------------------------------------------------------------------
    ' QCOM_PositionBelow
    '
    ' Sets the Location property of the specified object in reference to
    ' another object
    '-------------------------------------------------------------------------
    Public Function QCOM_PositionBelow( _
        ByRef targetObject As Object, _
        ByVal referenceObject As Object, _
        ByVal spacing As Integer)
        targetObject.Location = New Point( _
            referenceObject.Left, _
            referenceObject.Bottom + spacing)
        targetObject.BackColor = Color.Transparent
        Return Nothing
    End Function                        ' QCOM_PositionBelow
    '-------------------------------------------------------------------------
    ' QCOM_ReadFilePathFromLine
    '
    ' Returns the file path string identified by parameter in the specified line
    '
    ' Called by:    QCOM_RetrieveConfigData
    '-------------------------------------------------------------------------
    Public Function QCOM_ReadFilePathFromLine(ByVal lineString As String, ByVal parameter As String) As String
        If lineString.Length > 0 AndAlso parameter.Length > 0 Then
            If lineString.Contains(parameter) = True Then
                Dim pathString As String = _
                    lineString.Substring(lineString.IndexOf(parameter) + parameter.Length)
                If pathString.Length > 0 AndAlso String.Compare(pathString, QCOM_STRING_NONE, True) <> 0 Then
                    If File.Exists(pathString) = True Then
                        Return pathString
                    End If              ' If File.Exists(pathString) = True Then
                End If                  ' If pathString.Length > 0 AndAlso ...
            End If                      ' If lineString.Contains(parameter) = True Then
        End If                          ' If lineString.Length > 0 AndAlso parameter.Length > 0 Then
        Return Nothing
    End Function                        ' QCOM_ReadFilePathFromLine
    '-------------------------------------------------------------------------
    ' QCOM_ReadHexValueFromLine
    '
    ' Returns the hexadecimal value identified by parameter in the specified
    ' line
    '
    ' Called by:    QCOM_RetrieveConfigData
    '-------------------------------------------------------------------------
    Public Function QCOM_ReadHexValueFromLine(ByVal lineString As String, ByVal parameter As String) As Integer
        Dim inputValue As Integer = 0
        '---------------------------------------------------------------------
        If lineString.Length > 0 AndAlso parameter.Length > 0 Then
            If lineString.Contains(parameter) = True Then
                Dim valueString As String = _
                    lineString.Substring(lineString.IndexOf(parameter) + parameter.Length)
                If valueString.Length = 8 AndAlso String.Compare(valueString, "00000000") <> 0 Then
                    inputValue = Convert.ToInt32(valueString, 16)
                End If                  ' If pathString.Length > 0 AndAlso ...
            End If                      ' If lineString.Contains(parameter) = True Then
        End If                          ' If lineString.Length > 0 AndAlso parameter.Length > 0 Then
        Return inputValue
    End Function                        ' QCOM_ReadHexValueFromLine
    '-------------------------------------------------------------------------
    ' QCOM_ResetModules
    '
    ' Resets all attached QCOM modules
    '
    ' Called by:    QCOM_ToolStripResetButtonClicked
    '-------------------------------------------------------------------------
    Public Function QCOM_ResetModules()
        Dim unitNumber As UInt32 = 0
        Dim unit As UnitInfo
        '---------------------------------------------------------------------
        For percent As Integer = 1 To 100
            If (percent Mod 25) = 0 Then
                unitNumber = (percent / 25) - 1
                unit = QCOM_UnitInfoArray(unitNumber)
                If (unit.flags And QCOM_UNIT_OPEN) <> 0 Then
                    QD_ResetModule(unit.unitHandle)
                    Thread.Sleep(100)
                End If                  ' If (unit.flags And QCOM_UNIT_OPEN) <> 0 Then
            End If                      ' If (percent Mod 25) = 0 Then
            QCOM_UpdateHomeProgressBar(percent, 100)
            Thread.Sleep(10)
        Next                            ' For percent As Integer = 1 To 100
        QCOM_UpdateHomeProgressBar(0, 0)
        QCOM_UpdateStatusLine("All QCOM modules have been reset")
        Return Nothing
    End Function                        ' QCOM_ResetModules
    '-------------------------------------------------------------------------
    ' QCOM_RetrieveConfigData
    '
    ' Retrieves the information stored in the configuration file, if present
    '
    ' Called by:    QCOM_InitializeQCOM
    '-------------------------------------------------------------------------
    Public Function QCOM_RetrieveConfigData()
        Dim proceedToReadConfigFile As Boolean = False
        Dim processingGeneralSection As Boolean = True
        Dim transducerMatch(QCOM_MAXIMUM_NUMBER_OF_UNITS) As Boolean
        Dim unitMatch(QCOM_MAXIMUM_NUMBER_OF_UNITS) As Boolean
        Dim usingOutdatedConfigFile As Boolean = False
        Dim generalFlags As Integer = QCOM_GENERAL_ZERO_FLAG
        Dim inputValue As Integer
        Dim unitGeneralFlags(QCOM_MAXIMUM_NUMBER_OF_UNITS) As Integer
        Dim data As String
        Dim lineString As String
        Dim parameter As String
        Dim pathString As String
        '---------------------------------------------------------------------
        If (QCOM_GeneralInfo.flags And QCOM_GENERAL_CONFIG_DONT_LOAD) = 0 Then
            For unitNumber As Byte = 0 To QCOM_MAXIMUM_NUMBER_OF_UNITS - 1
                transducerMatch(unitNumber) = False
                unitMatch(unitNumber) = False
                unitGeneralFlags(unitNumber) = QCOM_UNIT_ZERO_FLAG
            Next
            pathString = QCOM_GeneralInfo.logDirectory + "\QCOM-" + QCOM_PROGRAM_VER_STRING + ".config"
            If File.Exists(pathString) = True Then
                proceedToReadConfigFile = True
                QCOM_GeneralInfo.configFilePath = pathString
                QCOM_GeneralInfo.flags = (QCOM_GeneralInfo.flags Or QCOM_GENERAL_CONFIG_FILE_SPECIFIED)
            Else
                '-------------------------------------------------------------
                ' The version-appropriate config file is not found, so search
                ' for one from an older version
                '-------------------------------------------------------------
                Dim latestConfigPath As String = String.Empty
                Dim logDir As String = QCOM_GeneralInfo.logDirectory + "\"
                Dim configFiles As String() = Directory.GetFiles(logDir, "QCOM-*.config")
                If configFiles.Length > 0 Then
                    For Each oldConfigPath As String In configFiles
                        If latestConfigPath.Length > 0 Then
                            If String.Compare(oldConfigPath, latestConfigPath, True) > 0 Then
                                latestConfigPath = oldConfigPath
                            End If
                        Else
                            latestConfigPath = oldConfigPath
                        End If
                    Next
                    If latestConfigPath.Length > 0 AndAlso File.Exists(latestConfigPath) = True Then
                        '-----------------------------------------------------
                        ' A config file from an older software version is
                        ' found, so load as many parameters from the old one
                        ' as possible and appropriate
                        '-----------------------------------------------------
                        proceedToReadConfigFile = True
                        usingOutdatedConfigFile = True
                        pathString = latestConfigPath
                    End If              ' If latestConfigPath.Length > 0 AndAlso ...
                End If                  ' If configFiles.Length > 0 Then
            End If                      ' If File.Exists(pathString) = True Then
            If proceedToReadConfigFile = True Then
                Dim textReader As StreamReader = File.OpenText(pathString)
                While textReader.Peek() > -1
                    lineString = textReader.ReadLine()
                    lineString = lineString.Trim()
                    '---------------------------------------------------------
                    ' Locate any comments and zero out the line from that
                    ' point to the end of the line
                    '---------------------------------------------------------
                    If lineString.Contains("#") = True Then
                        lineString = lineString.Substring(0, lineString.IndexOf("#"))
                    End If              ' If lineString.Contains("#") = True Then
                    '---------------------------------------------------------
                    ' Begin parsing non-comment lines
                    '---------------------------------------------------------
                    If lineString.Length > 0 Then
                        If processingGeneralSection = True Then
                            '-------------------------------------------------
                            ' General search string
                            '-------------------------------------------------
                            parameter = "SearchString="
                            If lineString.Contains(parameter) = True Then
                                data = lineString.Substring(lineString.IndexOf(parameter) + parameter.Length)
                                If data.Length > 0 Then
                                    If String.Compare(data, QCOM_STRING_NONE, True) <> 0 Then
                                        QCOM_GeneralInfo.searchString = data
                                    End If
                                End If
                                Continue While
                            End If
                            '-------------------------------------------------
                            ' General use path
                            '-------------------------------------------------
                            parameter = "GeneralUsePath="
                            If lineString.Contains(parameter) = True Then
                                QCOM_GeneralInfo.generalUsePath = _
                                    QCOM_ReadFilePathFromLine(lineString, parameter)
                                QCOM_GeneralInfo.flags = _
                                    (QCOM_GeneralInfo.flags Or QCOM_GENERAL_USE_PATH_SPECIFIED)
                                Continue While
                            End If
                            '-------------------------------------------------
                            ' Import the persistent flags only from the
                            ' current config file
                            '-------------------------------------------------
                            If usingOutdatedConfigFile = False Then
                                parameter = "GeneralFlags="
                                If lineString.Contains(parameter) = True Then
                                    generalFlags = QCOM_ReadHexValueFromLine(lineString, parameter)
                                    Continue While
                                End If
                                parameter = "GeneralPersistentFlags="
                                If lineString.Contains(parameter) = True Then
                                    QCOM_GeneralInfo.persistentFlags = _
                                        QCOM_ReadHexValueFromLine(lineString, parameter)
                                    Continue While
                                End If
                                parameter = "UnitPersistentFlags="
                                If lineString.Contains(parameter) = True Then
                                    QCOM_GeneralInfo.persistentUnitFlags = _
                                        QCOM_ReadHexValueFromLine(lineString, parameter)
                                    Continue While
                                End If
                            End If      ' If usingOutdatedConfigFile = False Then
                            '-------------------------------------------------
                            ' Sampling interval
                            '-------------------------------------------------
                            parameter = "SamplingInterval="
                            If lineString.Contains(parameter) = True Then
                                data = lineString.Substring(lineString.IndexOf(parameter) + parameter.Length)
                                If data.Length > 0 Then
                                    inputValue = Convert.ToInt32(data)
                                    If inputValue >= QCOM_MINIMUM_SAMPLING_INTERVAL AndAlso
                                        inputValue <= QCOM_MAXIMUM_SAMPLING_INTERVAL Then
                                        QCOM_GeneralInfo.samplingInterval = inputValue
                                    End If
                                End If
                                Continue While
                            End If
                            parameter = "[Unit_Information]"
                            If lineString.Contains(parameter) = True Then
                                processingGeneralSection = False
                                Continue While
                            End If      ' If lineString.Contains(parameter) = True Then
                        Else            ' If processingGeneralSection = True Then
                            '-------------------------------------------------
                            ' Read in the per-unit information
                            '-------------------------------------------------
                            If String.Compare(lineString.Substring(0, 4), "Unit", True) = 0 AndAlso _
                                lineString.IndexOf(" : ") = 6 Then
                                Dim unitNumber As Int32 = _
                                    Convert.ToInt32(lineString.Substring(lineString.IndexOf("Unit ") + 5, 2))
                                If unitNumber < 0 OrElse unitNumber >= QCOM_MAXIMUM_NUMBER_OF_UNITS Then
                                    Continue While
                                End If
                                Dim unit As UnitInfo = QCOM_UnitInfoArray(unitNumber)
                                '---------------------------------------------
                                ' Read the stored module serial number to test
                                ' for a match
                                '---------------------------------------------
                                parameter = "ModuleSerialNumber="
                                If lineString.Contains(parameter) = True Then
                                    data = lineString.Substring(lineString.IndexOf(parameter) + parameter.Length)
                                    If data.Length > 0 Then
                                        If String.Compare(data, QCOM_STRING_NONE, True) <> 0 Then
                                            If String.Compare(data, unit.moduleSerialNumber, True) = 0 Then
                                                unitMatch(unitNumber) = True
                                            End If
                                        End If
                                    End If
                                    Continue While
                                End If
                                '---------------------------------------------
                                ' Read the stored transducer serial number to
                                ' test for a match
                                '---------------------------------------------
                                parameter = "TransducerSerialNumber="
                                If lineString.Contains(parameter) = True Then
                                    data = lineString.Substring(lineString.IndexOf(parameter) + parameter.Length)
                                    If data.Length > 0 Then
                                        If String.Compare(data, QCOM_STRING_NONE, True) <> 0 Then
                                            If String.Compare(data, unit.transducerSerialNumber, True) = 0 Then
                                                transducerMatch(unitNumber) = True
                                            End If
                                        End If
                                    End If
                                    Continue While
                                End If
                                If unitMatch(unitNumber) = True AndAlso transducerMatch(unitNumber) = True Then
                                    '-----------------------------------------
                                    ' Because the current unit (module and
                                    ' transducer) hardware configuration seems
                                    ' to match that stored in the config file
                                    ' for those of the same unit number, the
                                    ' following parameters are considered
                                    ' valid for this unit number
                                    '-----------------------------------------
                                    parameter = "CoefficientFilePath="
                                    If lineString.Contains(parameter) = True Then
                                        data = QCOM_ReadFilePathFromLine(lineString, parameter)
                                        unit.coefficientFilePath = data
                                        unit.flags = (unit.flags Or QCOM_UNIT_COEFFICIENT_FILE_SPECIFIED)
                                        Continue While
                                    End If
                                    If usingOutdatedConfigFile = False Then
                                        parameter = "UnitFlags="
                                        If lineString.Contains(parameter) = True Then
                                            unitGeneralFlags(unitNumber) = _
                                                QCOM_ReadHexValueFromLine(lineString, parameter)
                                            Continue While
                                        End If
                                    End If
                                End If  ' If unitMatch(unitNumber) = True AndAlso transducerMatch(unitNumber) = True Then
                                unit.flags = (unit.flags Or QCOM_UNIT_CONFIG_INFO_LOADED)
                            End If      ' If String.Compare(lineString.Substring(0, 4), "Unit", True) = 0 AndAlso ...
                        End If          ' If processingGeneralSection = True Then
                    End If              ' If lineString.Length > 0 Then
                End While               ' While textReader.Peek() > -1
                textReader.Close()
                QCOM_GeneralInfo.flags = (QCOM_GeneralInfo.flags Or QCOM_GENERAL_CONFIG_INFO_LOADED)
            Else
                QCOM_GeneralInfo.configFilePath = Nothing
                QCOM_GeneralInfo.flags = (QCOM_GeneralInfo.flags And Not QCOM_GENERAL_CONFIG_FILE_SPECIFIED)
                QCOM_GeneralInfo.flags = (QCOM_GeneralInfo.flags And Not QCOM_GENERAL_CONFIG_INFO_LOADED)
            End If                      ' If proceedToReadConfigFile = True Then
            '-----------------------------------------------------------------
            ' Clear invalid pathnames, in case their flags are set
            '-----------------------------------------------------------------
            For Each unit As UnitInfo In QCOM_UnitInfoArray
                If (unit.flags And QCOM_UNIT_COEFFICIENT_FILE_SPECIFIED) <> 0 Then
                    If unit.coefficientFilePath.Length = 0 OrElse File.Exists(unit.coefficientFilePath) = False Then
                        unit.flags = (unit.flags And Not QCOM_UNIT_COEFFICIENT_FILE_SPECIFIED)
                        unit.coefficientFilePath = Nothing
                    End If              ' If unit.coefficientFilePath.Length = 0 OrElse ...
                Else
                    unit.coefficientFilePath = Nothing
                End If                  ' If (unit.flags And QCOM_UNIT_COEFFICIENT_FILE_SPECIFIED) <> 0 Then
            Next                        ' For Each unit As UnitInfo In QCOM_UnitInfoArray
            '-----------------------------------------------------------------
            ' Set the persistent flags if they are not set
            '-----------------------------------------------------------------
            If QCOM_GeneralInfo.persistentFlags = 0 Then
                QCOM_GeneralInfo.persistentFlags = QCOM_GENERAL_DEFAULT_PERSISTENT_FLAGS
            End If
            If QCOM_GeneralInfo.persistentUnitFlags = 0 Then
                QCOM_GeneralInfo.persistentUnitFlags = QCOM_UNIT_DEFAULT_PERSISTENT_FLAGS
            End If
            '-----------------------------------------------------------------
            ' Apply the effects of the persistent flags
            '-----------------------------------------------------------------
            QCOM_GeneralInfo.flags = QCOM_GeneralInfo.flags Or (generalFlags And QCOM_GeneralInfo.persistentFlags)
            For Each unit As UnitInfo In QCOM_UnitInfoArray
                unit.flags = (unit.flags Or (unitGeneralFlags(unit.unitNumber) And QCOM_GeneralInfo.persistentUnitFlags))
            Next
        End If                          ' If (QCOM_GeneralInfo.flags And QCOM_GENERAL_CONFIG_DONT_LOAD) = 0 Then
        Return Nothing
    End Function                        ' QCOM_RetrieveConfigData
    '-------------------------------------------------------------------------
    ' QCOM_RetrieveCoefficientData
    '
    ' Retrieves the coefficient data for client storage, to be used for readings
    '
    ' Returns: 0            Success
    '          nonzero      Failure
    '
    ' Called by:    QCOM_ScanForDevices
    '-------------------------------------------------------------------------
    Public Function QCOM_RetrieveCoefficientData(ByRef unit As UnitInfo) As UInt32
        Dim attemptToReadModuleMemory As Boolean = False
        Dim dataIsVerified As Boolean = False
        Dim coefficientData(QD_COEFFICIENT_DATA_SIZE) As Byte
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
            If (unit.transducerType <> QD_TRANSDUCER_TYPE_ABSENT) _
                AndAlso (unit.transducerType <> QD_TRANSDUCER_TYPE_FREQUENCY) _
                AndAlso (unit.transducerType <> QD_TRANSDUCER_TYPE_DIGITAL_NOMEM) Then
                '-----------------------------------------------------
                ' This digital transducer contains onboard memory, so
                ' attempt to read the coefficient data stored in that
                ' memory
                '-----------------------------------------------------
                status = QD_ReadCoefficientDataFromDevice(unit.unitHandle, coefficientData(0))
                If status = QCOM_SUCCESS Then
                    dataIsVerified = QD_CoefficientDataIsValid(coefficientData(0))
                    If dataIsVerified = True Then
                        unit.coefficientData = coefficientData
                        unit.flags = (unit.flags Or QCOM_UNIT_COEFFICIENTS_VERIFIED)
                    Else
                        attemptToReadModuleMemory = True
                        status = QCOM_ERROR_INVALID_COEFFICIENT_DATA
                    End If
                Else
                    attemptToReadModuleMemory = True
                End If                  ' If status = QCOM_SUCCESS Then
            Else
                attemptToReadModuleMemory = True
            End If                      ' If unit.transducerType <> QD_TRANSDUCER_TYPE_ABSENT AndAlso ...
            If attemptToReadModuleMemory = True Then
                '-------------------------------------------------------------
                ' Attempt to read the coefficient data from the QCOM module's
                ' memory at the first valid memory page encountered
                '-------------------------------------------------------------
                For pageNumber As Byte = 0 To QD_NUMBER_OF_LOWER_QMEM_PAGES - 1
                    status = QD_ReadCoefficientDataFromModulePage(unit.unitHandle, pageNumber, coefficientData(0))
                    If status = QCOM_SUCCESS Then
                        dataIsVerified = QD_CoefficientDataIsValid(coefficientData(0))
                        If dataIsVerified = True Then
                            unit.coefficientData = coefficientData
                            unit.flags = (unit.flags Or QCOM_UNIT_COEFFICIENTS_VERIFIED)
                            Exit For
                        Else
                            status = QCOM_ERROR_INVALID_COEFFICIENT_DATA
                        End If
                    End If              ' If status = QCOM_SUCCESS Then
                Next
            End If                      ' If attemptToReadModuleMemory = True Then
            If (unit.flags And QCOM_UNIT_COEFFICIENTS_VERIFIED) <> 0 Then
                QCOM_ApplyCoefficientDataInfoToProgram(unit)
            End If
        End If                          ' If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
        'If (status = QCOM_SUCCESS) <> 0 AndAlso (unit.flags And QCOM_UNIT_COEFFICIENTS_VERIFIED) <> 0 Then
        '    Msg("Scan CF({0:D}) = {1} - {2} - {3} - {4} - {5} - {6} - {7} - {8} - {9} - {10} - {11} - {12} - {13} - {14} - {15:D} - {16:D}", _
        '        unit.unitNumber, _
        '        String.Format("{0:X2}", unit.coefficientData(0)), _
        '        String.Format("{0:X2}", unit.coefficientData(1)), _
        '        String.Format("{0:X2}", unit.coefficientData(2)), _
        '        String.Format("{0:X2}", unit.coefficientData(3)), _
        '        String.Format("{0:X2}", unit.coefficientData(4)), _
        '        String.Format("{0:X2}", unit.coefficientData(5)), _
        '        String.Format("{0:X2}", unit.coefficientData(6)), _
        '        String.Format("{0:X2}", unit.coefficientData(7)), _
        '        Chr(unit.coefficientData(8)), _
        '        Chr(unit.coefficientData(9)), _
        '        Chr(unit.coefficientData(10)), _
        '        Chr(unit.coefficientData(11)), _
        '        Chr(unit.coefficientData(12)), _
        '        Chr(unit.coefficientData(13)), _
        '        unit.coefficientData(14), _
        '        unit.coefficientData(15))
        'End If  ' If (status = QCOM_SUCCESS) <> 0 AndAlso ...
        Return status
    End Function                        ' QCOM_RetrieveCoefficientData
    '-------------------------------------------------------------------------
    ' QCOM_RetrieveFirmwareInformation
    '
    ' Retrieves the firmware version and ID of the specified unit and populates
    ' the appropriate firmware fields
    '
    ' Called by:    QCOM_ScanForDevices
    '-------------------------------------------------------------------------
    Public Function QCOM_RetrieveFirmwareInformation(ByRef unit As UnitInfo)
        Dim companyString As String = QCOM_STRING_UNKNOWN
        Dim firmwareIDString(QD_FIRMWARE_ID_LENGTH) As Byte
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        status = QD_GetModuleFirmwareID(unit.unitHandle, firmwareIDString(0))
        If status = QCOM_SUCCESS Then
            unit.firmwareID = firmwareIDString
            If (firmwareIDString(0) = QD_FIRMWARE_QUARTZDYNE_ID) AndAlso _
                (firmwareIDString(1) = QD_FIRMWARE_QCOM_ID) Then
                companyString = "Quartzdyne QCOM"
            End If
            unit.firmwareString = String.Format( _
                "{0} v{1:D}.{2:D}", _
                companyString, _
                firmwareIDString(2), _
                firmwareIDString(3))
            unit.flags = (unit.flags Or QCOM_UNIT_FIRMWARE_ID_PRESENT)
        End If                          ' If status = QCOM_SUCCESS Then
        'Msg("FWID({0:D}) = {1}", unit.unitNumber, BitConverter.ToString(unit.firmwareID))
        Return Nothing
    End Function                        ' QCOM_RetrieveFirmwareInformation
    '-------------------------------------------------------------------------
    ' QCOM_RetrieveTransducerReadings
    '
    ' Retrieves one set of transducer readings for the specified unit
    '
    ' Returns:  0           Success
    '           nonzero     Failure
    '
    ' Called by:    QCOM_ApplyCoefficientDataInfoToProgram
    '               QCOM_SamplingTimerElapsed
    '-------------------------------------------------------------------------
    Public Function QCOM_RetrieveTransducerReadings(ByRef unit As UnitInfo) As UInt32
        Dim attempts As Integer
        Dim pressureCount As UInt32 = 0
        Dim temperatureCount As UInt32 = 0
        Dim pressureValuePSI As Double = 0.0
        Dim temperatureValueCelsius As Double = 0.0
        Dim transducerCurrent As Double = 0.0
        Dim transducerVoltage As Double = 0.0
        Dim coefficientData(QD_COEFFICIENT_DATA_SIZE) As Byte
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        If (unit.flags And QCOM_UNIT_VALID) <> 0 AndAlso (unit.flags And QCOM_UNIT_COEFFICIENTS_VERIFIED) <> 0 Then
            '-----------------------------------------------------------------
            ' Reset all stored values to prevent displaying of stale data
            '-----------------------------------------------------------------
            unit.pressureCount = 0
            unit.temperatureCount = 0
            unit.pressureValuePSI = 0.0
            unit.temperatureValueCelsius = 0.0
            unit.transducerVoltage = 0.0
            unit.transducerCurrent = 0.0
            '-----------------------------------------------------------------
            ' Retrieve the transducer counts
            '-----------------------------------------------------------------
            attempts = 3
            While attempts > 0
                attempts -= 1
                status = QD_GetTransducerCounts( _
                    unit.unitHandle, _
                    pressureCount, _
                    temperatureCount)
                If status = QCOM_SUCCESS Then
                    unit.pressureCount = pressureCount
                    unit.temperatureCount = temperatureCount
                    attempts = 0
                    Exit While
                Else
                    If attempts <> 0 Then
                        status = QCOM_SUCCESS
                        Thread.Sleep(1000)
                    End If
                End If
            End While
            '-----------------------------------------------------------------
            ' Calculate the pressure and temperature using the coefficient
            ' data and the counts
            '-----------------------------------------------------------------
            If status = QCOM_SUCCESS Then
                coefficientData = unit.coefficientData
                status = QD_CalculatePressureAndTemperature( _
                    coefficientData(0), _
                    pressureCount, _
                    temperatureCount, _
                    pressureValuePSI, _
                    temperatureValueCelsius)
                If status = QCOM_SUCCESS Then
                    'Msg("Success, and pres = {0:F3}", pressureValuePSI)
                    unit.pressureValuePSI = pressureValuePSI
                    unit.temperatureValueCelsius = temperatureValueCelsius
                Else
                    EMsg("Failed with status = 0x{0}", status.ToString("X8"))
                End If
            End If                      ' If status = QCOM_SUCCESS Then
            '-----------------------------------------------------------------
            ' Retrieve the transdcuer voltage
            '-----------------------------------------------------------------
            If status = QCOM_SUCCESS Then
                attempts = 3
                While attempts > 0
                    attempts -= 1
                    status = QD_GetTransducerVoltage( _
                        unit.unitHandle, _
                        transducerVoltage)
                    If status = QCOM_SUCCESS Then
                        unit.transducerVoltage = transducerVoltage
                        attempts = 0
                        Exit While
                    Else
                        If attempts <> 0 Then
                            status = QCOM_SUCCESS
                            Thread.Sleep(1000)
                        End If
                    End If
                End While
            End If                      ' If status = QCOM_SUCCESS Then
            '-----------------------------------------------------------------
            ' Retrieve the transdcuer current
            '-----------------------------------------------------------------
            If status = QCOM_SUCCESS Then
                attempts = 3
                While attempts > 0
                    attempts -= 1
                    status = QD_GetTransducerCurrent( _
                        unit.unitHandle, _
                        transducerCurrent)
                    If status = QCOM_SUCCESS Then
                        unit.transducerCurrent = transducerCurrent
                        attempts = 0
                        Exit While
                    Else
                        If attempts <> 0 Then
                            status = QCOM_SUCCESS
                            Thread.Sleep(1000)
                        End If
                    End If
                End While
            End If                      ' If status = QCOM_SUCCESS Then
        End If                          ' If (unit.flags And QCOM_UNIT_VALID) <> 0 AndAlso ...
        Erase coefficientData
        Return status
    End Function                        ' QCOM_RetrieveTransducerReadings
    '-------------------------------------------------------------------------
    ' QCOM_SaveConfigData
    '
    ' Saves program information in the configuration file
    '
    ' Called by:    QCOM_Finalize
    '-------------------------------------------------------------------------
    Public Function QCOM_SaveConfigData()
        Dim border As String = "#" + New String("=", 77)
        Dim configFileName As String
        Dim halfBorder As String = "#" + New String("-", 77)
        Dim lineString As String
        Dim pathString As String
        Dim dateTimes As DateTime = DateTime.Now
        '---------------------------------------------------------------------
        If (QCOM_GeneralInfo.flags And QCOM_GENERAL_CONFIG_FILE_SPECIFIED) <> 0 Then
            pathString = QCOM_GeneralInfo.configFilePath
            configFileName = Path.GetFileName(pathString)
        Else
            configFileName = "QCOM-" + QCOM_PROGRAM_VER_STRING + ".config"
            pathString = QCOM_GeneralInfo.logDirectory + "\" + configFileName
        End If
        If (QCOM_GeneralInfo.flags And QCOM_GENERAL_CONFIG_DELETE_ON_EXIT) <> 0 Then
            If File.Exists(pathString) = True Then
                File.Delete(pathString)
            End If
        End If
        If (QCOM_GeneralInfo.flags And QCOM_GENERAL_CONFIG_DONT_SAVE) = 0 Then
            Dim textWriter As StreamWriter = File.CreateText(pathString)
            textWriter.WriteLine(border)
            '-----------------------------------------------------------------
            ' Write out the header, including the program name and the current
            ' date and time
            '-----------------------------------------------------------------
            lineString = String.Format( _
                "# {0}\n#\n" + _
                "# QCOM Configuration Data File\n#\n" + _
                "# Created on {1:D2} {2} {3:D4} at {4:D2}:{5:D2}:{6:D2}", _
                configFileName, _
                dateTimes.Day, _
                QCOM_MonthStringArray(dateTimes.Month), _
                dateTimes.Year, _
                dateTimes.Hour, _
                dateTimes.Minute, _
                dateTimes.Second)
            textWriter.WriteLine(lineString.Replace(QCOM_STRING_LF, Environment.NewLine))
            textWriter.WriteLine(border)
            '-----------------------------------------------------------------
            ' Save general program information
            '-----------------------------------------------------------------
            Dim currentMajorVersion As Byte = 0
            Dim currentMinorVersion As Byte = 0
            Dim currentBuildVersion As Byte = 0
            QD_GetQDDLLVersion( _
                currentMajorVersion, _
                currentMinorVersion, _
                currentBuildVersion)
            Dim lowerVersion As UInt32 = 0
            Dim upperVersion As UInt32 = 0
            QD_GetUSBDriverVersion(upperVersion, lowerVersion)
            lineString = String.Format( _
                "[QCOM_Start]\n" + _
                "    ProgramVersion={0}\n" + _
                "    QCOMDLL={1:D}.{2:D}.{3:D}\n" + _
                "    SiUSBDriver={4:D}.{5:D}.{6:D}.{7:D}\n" + _
                "    OS={8}", _
                QCOM_PROGRAM_VERSION_STRING, _
                currentMajorVersion, currentMinorVersion, currentBuildVersion, _
                (upperVersion >> 16), (upperVersion And &HFFFF), _
                (lowerVersion >> 16), (lowerVersion And &HFFFF), _
                QCOM_GeneralInfo.windowsVersion)
            textWriter.WriteLine(lineString.Replace(QCOM_STRING_LF, Environment.NewLine))
            '-----------------------------------------------------------------
            ' Save global flags
            '-----------------------------------------------------------------
            lineString = String.Format( _
                "[QCOM_Parameters]\n" + _
                "    ConfigFilePath={0}\n" + _
                "    GeneralFlags={1}\n" + _
                "    GeneralPersistentFlags={2}\n" + _
                "    UnitPersistentFlags={3}\n" + _
                "    SamplingInterval={4}", _
                If((QCOM_GeneralInfo.flags And QCOM_GENERAL_CONFIG_FILE_SPECIFIED) <> 0 AndAlso QCOM_GeneralInfo.configFilePath.Length > 0, _
                   QCOM_GeneralInfo.configFilePath, QCOM_STRING_NONE),
                QCOM_GeneralInfo.flags.ToString("X8"), _
                QCOM_GeneralInfo.persistentFlags.ToString("X8"), _
                QCOM_GeneralInfo.persistentUnitFlags.ToString("X8"), _
                QCOM_GeneralInfo.samplingInterval.ToString("D"))
            textWriter.WriteLine(lineString.Replace(QCOM_STRING_LF, Environment.NewLine))
            '-----------------------------------------------------------------
            ' Save unit and transducer information for all possible units
            '-----------------------------------------------------------------
            textWriter.WriteLine("[Unit_Information]")
            For Each unit As UnitInfo In QCOM_UnitInfoArray
                lineString = String.Format( _
                    "Unit {0} : ModuleSerialNumber={1}\n" + _
                    "Unit {0} : ModuleFirmwareVersion={2}\n" + _
                    "Unit {0} : TransducerSerialNumber={3}\n" + _
                    "Unit {0} : TransducerType={4}\n" + _
                    "Unit {0} : CoefficientFilePath={5}\n" + _
                    "Unit {0} : UnitFlags={6}", _
                    unit.unitNumber.ToString("D"), _
                    If((unit.flags And QCOM_UNIT_VALID) <> 0, unit.moduleSerialNumber, QCOM_STRING_NONE), _
                    If((unit.flags And QCOM_UNIT_FIRMWARE_ID_PRESENT) <> 0, unit.firmwareString, QCOM_STRING_NONE), _
                    If((unit.flags And QCOM_UNIT_TRANSDUCER_SN_VALID) <> 0, unit.transducerSerialNumber, QCOM_STRING_NONE), _
                    If((unit.flags And QCOM_UNIT_TRANSDUCER_PRESENT) <> 0, unit.transducerType.ToString("D"), QCOM_STRING_NA), _
                    If((unit.flags And QCOM_UNIT_COEFFICIENT_FILE_SPECIFIED) <> 0 AndAlso unit.coefficientFilePath.Length > 0, unit.coefficientFilePath, QCOM_STRING_NONE), _
                    unit.flags.ToString("X8"))
                textWriter.WriteLine(lineString.Replace(QCOM_STRING_LF, Environment.NewLine))
                textWriter.WriteLine(halfBorder)
            Next                        ' For Each unit As UnitInfo In QCOM_UnitInfoArray
            '-----------------------------------------------------------------
            ' Write out the footer
            '-----------------------------------------------------------------
            textWriter.WriteLine("[QCOM_End]")
            textWriter.WriteLine(border)
            lineString = "# End of " + configFileName
            textWriter.WriteLine(lineString)
            textWriter.WriteLine(border)
            textWriter.Close()
        End If                          ' If (QCOM_GeneralInfo.flags And QCOM_GENERAL_CONFIG_DONT_SAVE) = 0 Then
        Return Nothing
    End Function                        ' QCOM_SaveConfigData
    '-------------------------------------------------------------------------
    ' QCOM_SetTransducerDataRate
    '
    ' Checks the I²C Data rate set by the module to communicate with the
    ' transducer, then sets it to the maximum if it isn't already set to that
    ' value
    '
    ' Returns the data rate
    '
    ' Called by:    QCOM_ScanForDevices
    '-------------------------------------------------------------------------
    Public Function QCOM_SetTransducerDataRate(ByRef unit As UnitInfo) As Double
        Dim currentDataRate As Double = 0.0
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        status = QD_GetI2CDataRate(unit.unitHandle, currentDataRate)
        If status = QCOM_SUCCESS Then
            If currentDataRate < QD_MAXIMUM_I2C_DATA_RATE Then
                status = QD_SetI2CDataRate(unit.unitHandle, QD_MAXIMUM_I2C_DATA_RATE)
                If status = QCOM_SUCCESS Then
                    status = QD_GetI2CDataRate(unit.unitHandle, currentDataRate)
                    If status = QCOM_SUCCESS AndAlso currentDataRate = QD_MAXIMUM_I2C_DATA_RATE Then
                        'Msg("I²C data rate successfully set to {0:F2}", currentDataRate)
                    Else
                        EMsg("Setting the I²C data rate to {0:F2} failed with 0x{1}", _
                             QD_MAXIMUM_I2C_DATA_RATE, _
                             status.ToString("X8"))
                        currentDataRate = 0.0
                    End If              ' If status = QCOM_SUCCESS AndAlso ...
                Else
                    currentDataRate = 0.0
                End If                  ' If status = QCOM_SUCCESS Then
            End If                      ' If currentDataRate < QD_MAXIMUM_I2C_DATA_RATE Then
            unit.dataRate = currentDataRate
        Else
            currentDataRate = 0.0
        End If                          ' If status = QCOM_SUCCESS Then
        Return currentDataRate
    End Function                        ' QCOM_SetTransducerDataRate
    '-------------------------------------------------------------------------
    ' QCOM_SetUnitBoxInterfaceProperties
    '
    ' Sets the Tag and BackColor properties of the specified box object
    '-------------------------------------------------------------------------
    Public Function QCOM_SetUnitBoxInterfaceProperties( _
        ByRef targetObject As Object, _
        ByRef unitNumber As UInt32)
        targetObject.Tag = unitNumber
        targetObject.BackColor = Color.Lavender
        Return Nothing
    End Function                        ' QCOM_SetUnitBoxInterfaceProperties
    '-------------------------------------------------------------------------
    ' QCOM_SetUnitObjectInterfaceProperties
    '
    ' Sets the Tag and BackColor properties of the specified object
    '-------------------------------------------------------------------------
    Public Function QCOM_SetUnitObjectInterfaceProperties( _
        ByRef targetObject As Object, _
        ByRef unitNumber As UInt32)
        targetObject.Tag = unitNumber
        targetObject.BackColor = Color.Transparent
        Return Nothing
    End Function                        ' QCOM_SetUnitObjectInterfaceProperties
    '-------------------------------------------------------------------------
    ' QCOM_SetUpMiscControls
    '
    ' Displays a window that allows the tester to manually control discrete
    ' QCOM functions
    '
    ' Called by:    QCOM_ConstructAndDisplayUserInterface
    '-------------------------------------------------------------------------
    Public Function QCOM_SetUpMiscControls()
        miscControlsWindow = New Form()
        miscControlsWindow.SuspendLayout()
        miscControlsWindow.Icon = QCOM_SoftwareIcon
        miscControlsWindow.BackgroundImage = whiteSandBackground
        miscControlsWindow.Text = "Miscellaneous Controls and Utilities"
        miscControlsWindow.Size = New Drawing.Size( _
            GUI_MISC_WINDOW_WIDTH, _
            GUI_MISC_WINDOW_HEIGHT)
        miscControlsWindow.FormBorderStyle = System.Windows.Forms.FormBorderStyle.Fixed3D
        miscControlsWindow.StartPosition = FormStartPosition.CenterScreen
        miscControlsWindow.MaximizeBox = False
        miscControlsWindow.HelpButton = False
        '---------------------------------------------------------------------
        ' General Controls group box
        '---------------------------------------------------------------------
        Dim miscControlsGeneralGroupBox As New GroupBox()
        Dim generalGroupBoxDefined As Boolean = True
        '---------------------------------------------------------------------
        If generalGroupBoxDefined = True Then
            miscControlsGeneralGroupBox.Text = "Controls That Affect All Units"
            miscControlsGeneralGroupBox.Location = New Point(10, 10)
            miscControlsGeneralGroupBox.Size = New Drawing.Size( _
                miscControlsWindow.Width - 30, 80)
            miscControlsGeneralGroupBox.BackColor = Color.Transparent
            miscControlsWindow.Controls.Add(miscControlsGeneralGroupBox)
            '-----------------------------------------------------------------
            ' General check boxes
            '-----------------------------------------------------------------
            miscEnableSoundsCheck = New CheckBox()
            miscEnableSoundsCheck.Text = "Enable program sounds"
            miscEnableSoundsCheck.Location = New Point(10, 20)
            miscEnableSoundsCheck.Size = New Drawing.Size(222, GUI_REGULAR_CHECK_BOX_HEIGHT)
            miscEnableSoundsCheck.BackColor = Color.Transparent
            miscEnableSoundsCheck.Cursor = Cursors.Hand
            AddHandler miscEnableSoundsCheck.Click, AddressOf QCOM_MiscEnableSoundsChecked
            If (QCOM_GeneralInfo.flags And QCOM_GENERAL_SOUNDS_ENABLED) <> 0 Then
                miscEnableSoundsCheck.Checked = True
            Else
                miscEnableSoundsCheck.Checked = False
            End If
            miscControlsGeneralGroupBox.Controls.Add(miscEnableSoundsCheck)
            '-----------------------------------------------------------------
            ' Tool tip
            '-----------------------------------------------------------------
            Dim miscEnableSoundsCheckToolTip As New ToolTip()
            miscEnableSoundsCheckToolTip.ShowAlways = True
            miscEnableSoundsCheckToolTip.AutoPopDelay = 10000
            miscEnableSoundsCheckToolTip.ToolTipTitle = "Enable Program Sounds"
            Dim miscEnableSoundsCheckToolTipText As String = String.Concat( _
                "Check to enable program sounds made when errors are", Environment.NewLine, _
                "encountered or when the program exits, for example.", Environment.NewLine, _
                Environment.NewLine, _
                "Program sounds are enabled by default.")
            miscEnableSoundsCheckToolTip.SetToolTip(miscEnableSoundsCheck, miscEnableSoundsCheckToolTipText)
            '-----------------------------------------------------------------
            ' Timeout Settings group box
            '-----------------------------------------------------------------
            Dim miscGetSetTimeoutsGroupBox As New GroupBox()
            Dim timeoutGroupBoxDefined As Boolean = True
            '-----------------------------------------------------------------
            If timeoutGroupBoxDefined = True Then
                miscGetSetTimeoutsGroupBox.Text = "Timeout Settings"
                miscGetSetTimeoutsGroupBox.Location = New Point( _
                    miscControlsGeneralGroupBox.Width - 248, 14)
                miscGetSetTimeoutsGroupBox.Size = New Drawing.Size(238, 56)
                miscGetSetTimeoutsGroupBox.BackColor = Color.Transparent
                miscControlsGeneralGroupBox.Controls.Add(miscGetSetTimeoutsGroupBox)
                '-------------------------------------------------------------
                ' Get button
                '-------------------------------------------------------------
                Dim getTimeoutButton As New Button()
                getTimeoutButton.Text = "Get"
                getTimeoutButton.Location = New Point(6, 24)
                getTimeoutButton.Size = New Drawing.Size( _
                    GUI_GET_SET_BUTTON_WIDTH, _
                    GUI_REGULAR_BUTTON_HEIGHT)
                getTimeoutButton.Cursor = Cursors.Hand
                getTimeoutButton.BackColor = Color.Transparent
                AddHandler getTimeoutButton.Click, AddressOf QCOM_MiscGetTimeoutsButtonClicked
                miscGetSetTimeoutsGroupBox.Controls.Add(getTimeoutButton)
                '-------------------------------------------------------------
                ' Read label
                '-------------------------------------------------------------
                Dim readTimeoutValueLabel As New Label()
                readTimeoutValueLabel.Text = "Read"
                readTimeoutValueLabel.Location = New Point( _
                    getTimeoutButton.Right + 5, 14)
                readTimeoutValueLabel.Size = New Drawing.Size(50, GUI_REGULAR_LABEL_HEIGHT)
                readTimeoutValueLabel.BackColor = Color.Transparent
                readTimeoutValueLabel.TextAlign = HorizontalAlignment.Center
                miscGetSetTimeoutsGroupBox.Controls.Add(readTimeoutValueLabel)
                '-------------------------------------------------------------
                ' Write label
                '-------------------------------------------------------------
                Dim writetimeoutValueLabel As New Label()
                writetimeoutValueLabel.Text = "Write"
                writetimeoutValueLabel.Location = New Point( _
                    getTimeoutButton.Right + 81, _
                    readTimeoutValueLabel.Top)
                writetimeoutValueLabel.Size = readTimeoutValueLabel.Size
                writetimeoutValueLabel.BackColor = Color.Transparent
                writetimeoutValueLabel.TextAlign = HorizontalAlignment.Center
                miscGetSetTimeoutsGroupBox.Controls.Add(writetimeoutValueLabel)
                '-------------------------------------------------------------
                ' Read text box
                '-------------------------------------------------------------
                miscReadTimeoutBox = New TextBox()
                miscReadTimeoutBox.Location = New Point( _
                    getTimeoutButton.Right + 5, _
                    readTimeoutValueLabel.Bottom + 2)
                miscReadTimeoutBox.Size = New Drawing.Size(50, GUI_REGULAR_TEXT_BOX_HEIGHT)
                miscReadTimeoutBox.Multiline = False
                miscReadTimeoutBox.AcceptsReturn = False
                miscReadTimeoutBox.AcceptsTab = False
                miscReadTimeoutBox.WordWrap = False
                miscReadTimeoutBox.ReadOnly = False
                miscReadTimeoutBox.TextAlign = HorizontalAlignment.Right
                miscReadTimeoutBox.Cursor = Cursors.Hand
                miscReadTimeoutBox.BackColor = Color.Lavender
                miscGetSetTimeoutsGroupBox.Controls.Add(miscReadTimeoutBox)
                '-------------------------------------------------------------
                ' Read ms label
                '-------------------------------------------------------------
                Dim readMSLabel As New Label()
                readMSLabel.Text = "ms"
                readMSLabel.Location = New Point( _
                    miscReadTimeoutBox.Right + 2, _
                    miscReadTimeoutBox.Top + 3)
                readMSLabel.Size = New Drawing.Size(20, GUI_REGULAR_LABEL_HEIGHT)
                readMSLabel.BackColor = Color.Transparent
                miscGetSetTimeoutsGroupBox.Controls.Add(readMSLabel)
                '-------------------------------------------------------------
                ' Write text box
                '-------------------------------------------------------------
                miscWriteTimeoutBox = New TextBox()
                miscWriteTimeoutBox.Location = New Point( _
                    readMSLabel.Right + 4, _
                    miscReadTimeoutBox.Top)
                miscWriteTimeoutBox.Size = miscReadTimeoutBox.Size
                miscWriteTimeoutBox.Multiline = False
                miscWriteTimeoutBox.AcceptsReturn = False
                miscWriteTimeoutBox.AcceptsTab = False
                miscWriteTimeoutBox.WordWrap = False
                miscWriteTimeoutBox.ReadOnly = False
                miscWriteTimeoutBox.TextAlign = HorizontalAlignment.Right
                miscWriteTimeoutBox.Cursor = Cursors.Hand
                miscWriteTimeoutBox.BackColor = Color.Lavender
                miscGetSetTimeoutsGroupBox.Controls.Add(miscWriteTimeoutBox)
                '-------------------------------------------------------------
                ' Write ms label
                '-------------------------------------------------------------
                Dim writeMSLabel As New Label()
                writeMSLabel.Text = "ms"
                writeMSLabel.Location = New Point( _
                    miscWriteTimeoutBox.Right + 2, _
                    miscWriteTimeoutBox.Top + 3)
                writeMSLabel.Size = readMSLabel.Size
                writeMSLabel.BackColor = Color.Transparent
                miscGetSetTimeoutsGroupBox.Controls.Add(writeMSLabel)
                '-------------------------------------------------------------
                ' Set button
                '-------------------------------------------------------------
                Dim setTimeoutButton As New Button()
                setTimeoutButton.Text = "Set"
                setTimeoutButton.Location = New Point( _
                    writeMSLabel.Right + 4, _
                    getTimeoutButton.Top)
                setTimeoutButton.Size = getTimeoutButton.Size
                setTimeoutButton.Cursor = Cursors.Hand
                setTimeoutButton.BackColor = Color.Transparent
                AddHandler setTimeoutButton.Click, AddressOf QCOM_MiscSetTimeoutsButtonClicked
                miscGetSetTimeoutsGroupBox.Controls.Add(setTimeoutButton)
                '-------------------------------------------------------------
                ' Tool tip
                '-------------------------------------------------------------
                Dim miscGetSetTimeoutsToolTip As New ToolTip()
                miscGetSetTimeoutsToolTip.ShowAlways = True
                miscGetSetTimeoutsToolTip.AutoPopDelay = 10000
                miscGetSetTimeoutsToolTip.ToolTipTitle = "Timeout Settings"
                Dim miscGetSetTimeoutsToolTipText As String = String.Concat( _
                    "Controls the amount of time allowed by an operation before", Environment.NewLine, _
                    "the software considers that operation expired.", Environment.NewLine, _
                    Environment.NewLine, _
                    "Default timeouts are 500 ms for both Read and Write operations.")
                miscGetSetTimeoutsToolTip.SetToolTip(miscGetSetTimeoutsGroupBox, miscGetSetTimeoutsToolTipText)
                miscGetSetTimeoutsToolTip.SetToolTip(getTimeoutButton, miscGetSetTimeoutsToolTipText)
                miscGetSetTimeoutsToolTip.SetToolTip(readTimeoutValueLabel, miscGetSetTimeoutsToolTipText)
                miscGetSetTimeoutsToolTip.SetToolTip(writetimeoutValueLabel, miscGetSetTimeoutsToolTipText)
                miscGetSetTimeoutsToolTip.SetToolTip(miscReadTimeoutBox, miscGetSetTimeoutsToolTipText)
                miscGetSetTimeoutsToolTip.SetToolTip(readMSLabel, miscGetSetTimeoutsToolTipText)
                miscGetSetTimeoutsToolTip.SetToolTip(miscWriteTimeoutBox, miscGetSetTimeoutsToolTipText)
                miscGetSetTimeoutsToolTip.SetToolTip(writeMSLabel, miscGetSetTimeoutsToolTipText)
                miscGetSetTimeoutsToolTip.SetToolTip(setTimeoutButton, miscGetSetTimeoutsToolTipText)
            End If                      ' If timeoutGroupBoxDefined = True Then
        End If                          ' If generalGroupBoxDefined = True Then
        '---------------------------------------------------------------------
        ' Unit tab controls
        '---------------------------------------------------------------------
        miscTabControl = New TabControl()
        Dim tabsDefined As Boolean = True
        '---------------------------------------------------------------------
        If tabsDefined = True Then
            miscTabControl.Appearance = TabAppearance.Normal
            miscTabControl.Alignment = TabAlignment.Top
            miscTabControl.Size = New Drawing.Size( _
                miscControlsGeneralGroupBox.Width, _
                miscControlsWindow.Height - miscControlsGeneralGroupBox.Height - 92)
            QCOM_PositionBelow(miscTabControl, miscControlsGeneralGroupBox, 10)
            miscControlsWindow.Controls.Add(miscTabControl)
            '-----------------------------------------------------------------
            ' Set up and populate the tabs for all possible units
            '-----------------------------------------------------------------
            For Each unit As UnitInfo In QCOM_UnitInfoArray
                QCOM_SetUpUnitMiscControls(unit)
            Next                    ' For Each unit As UnitInfo In QCOM_UnitInfoArray
        End If                      ' If tabsDefined = True Then
        '---------------------------------------------------------------------
        ' Add a Close button
        '---------------------------------------------------------------------
        Dim miscCloseButton As New Button()
        miscCloseButton.Text = "Close"
        miscCloseButton.Location = New Point( _
            miscControlsWindow.Width - 95, _
            miscControlsWindow.Height - 65)
        miscCloseButton.Size = New Drawing.Size( _
            GUI_CLOSE_BUTTON_WIDTH, _
            GUI_REGULAR_BUTTON_HEIGHT)
        miscCloseButton.BackColor = Color.Transparent
        miscCloseButton.Cursor = Cursors.Hand
        AddHandler miscCloseButton.Click, AddressOf QCOM_MiscControlsCloseWindow
        '---------------------------------------------------------------------
        ' Remaining window closing controls
        '---------------------------------------------------------------------
        AddHandler miscControlsWindow.FormClosing, AddressOf QCOM_MiscControlsClosingWindow
        miscControlsWindow.Controls.Add(miscCloseButton)
        miscControlsWindow.AcceptButton = miscCloseButton
        miscControlsWindow.CancelButton = miscCloseButton
        miscControlsWindow.ResumeLayout(False)
        Return Nothing
    End Function                        ' QCOM_SetUpMiscControls
    '-------------------------------------------------------------------------
    ' QCOM_SetUpUnitMiscControls
    '
    ' Builds a unit Misc controls tab for the Misc Controls window
    '
    ' Called by:    QCOM_SetUpMiscControls
    '-------------------------------------------------------------------------
    Public Function QCOM_SetUpUnitMiscControls(ByRef unit As UnitInfo)
        Dim unitNumber As UInt32 = unit.unitNumber
        '---------------------------------------------------------------------
        ' Unit tab page
        '---------------------------------------------------------------------
        Dim miscUnitTabPage As New TabPage()
        Dim unitTabPageDefined As Boolean = True
        '---------------------------------------------------------------------
        If unitTabPageDefined = True Then
            If (unit.flags And QCOM_UNIT_MODULE_SN_VALID) <> 0 Then
                miscUnitTabPage.Text = unit.moduleSerialNumber
            Else
                miscUnitTabPage.Text = String.Concat("Unit ", unitNumber)
            End If
            miscUnitTabPage.TabIndex = unitNumber
            miscUnitTabPage.BackgroundImage = whiteMarbleBackground
            miscTabPageArray(unitNumber) = miscUnitTabPage
            miscTabControl.Controls.Add(miscUnitTabPage)
            '-----------------------------------------------------------------
            ' Unit group box
            '-----------------------------------------------------------------
            Dim miscUnitGroupBox As New GroupBox()
            Dim unitGroupBoxDefined As Boolean = True
            '-----------------------------------------------------------------
            If unitGroupBoxDefined = True Then
                miscUnitGroupBox.Text = unit.unitDescriptionString
                miscUnitGroupBox.Location = New Point(10, 10)
                miscUnitGroupBox.Size = New Drawing.Size( _
                    miscTabControl.Width - 30, _
                    miscTabControl.Height - 50)
                miscUnitGroupBox.BackColor = Color.Transparent
                miscUnitTabPage.Controls.Add(miscUnitGroupBox)
                '-------------------------------------------------------------
                ' Module serial number group box
                '-------------------------------------------------------------
                Dim miscGetModuleSerialNumberGroupBox As New GroupBox()
                Dim moduleSerialNumberGroupBoxDefined As Boolean = True
                '-------------------------------------------------------------
                If moduleSerialNumberGroupBoxDefined = True Then
                    miscGetModuleSerialNumberGroupBox.Text = "Module Serial Number"
                    miscGetModuleSerialNumberGroupBox.Location = New Point(10, 20)
                    miscGetModuleSerialNumberGroupBox.Size = New Drawing.Size(140, 50)
                    miscUnitGroupBox.Controls.Add(miscGetModuleSerialNumberGroupBox)
                    '---------------------------------------------------------
                    ' Get button
                    '---------------------------------------------------------
                    Dim miscGetModuleSerialNumberButton As New Button()
                    miscGetModuleSerialNumberButton.Text = "Get"
                    miscGetModuleSerialNumberButton.Location = New Point(6, 16)
                    miscGetModuleSerialNumberButton.Size = New Drawing.Size( _
                        GUI_GET_SET_BUTTON_WIDTH, _
                        GUI_REGULAR_BUTTON_HEIGHT)
                    QCOM_SetUnitObjectInterfaceProperties(miscGetModuleSerialNumberButton, unitNumber)
                    AddHandler miscGetModuleSerialNumberButton.Click, _
                        AddressOf QCOM_MiscUnitReadModuleSerialNumberButtonClicked
                    miscGetModuleSerialNumberGroupBox.Controls.Add(miscGetModuleSerialNumberButton)
                    '---------------------------------------------------------
                    ' Text box
                    '---------------------------------------------------------
                    Dim miscGetModuleSerialNumberBox = New TextBox()
                    miscGetModuleSerialNumberBox.Location = New Point( _
                        miscGetModuleSerialNumberButton.Right + 5, _
                        miscGetModuleSerialNumberButton.Top + 2)
                    miscGetModuleSerialNumberBox.Size = New Drawing.Size(50, GUI_REGULAR_TEXT_BOX_HEIGHT)
                    miscGetModuleSerialNumberBox.BackColor = Color.Lavender
                    miscGetModuleSerialNumberBox.Multiline = False
                    miscGetModuleSerialNumberBox.AcceptsReturn = False
                    miscGetModuleSerialNumberBox.AcceptsTab = False
                    miscGetModuleSerialNumberBox.WordWrap = False
                    miscGetModuleSerialNumberBox.TextAlign = HorizontalAlignment.Center
                    miscGetModuleSerialNumberBox.ReadOnly = True
                    miscGetModuleSerialNumberBox.Cursor = Cursors.Default
                    miscGetModuleSerialNumberBox.BackColor = Color.Lavender
                    miscGetModuleSerialNumberBoxArray(unitNumber) = miscGetModuleSerialNumberBox
                    miscGetModuleSerialNumberGroupBox.Controls.Add(miscGetModuleSerialNumberBox)
                    '---------------------------------------------------------
                    ' Tool tip
                    '---------------------------------------------------------
                    Dim miscModuleSerialNumberToolTip As New ToolTip()
                    miscModuleSerialNumberToolTip.ShowAlways = True
                    miscModuleSerialNumberToolTip.AutoPopDelay = 10000
                    miscModuleSerialNumberToolTip.ToolTipTitle = "Module Serial Number"
                    Dim miscModuleSerialNumberToolTipText As String = String.Concat( _
                        "Displays the serial number of the QCOM module", Environment.NewLine, _
                        "for unit ", unitNumber, ". If the module is in Boot Loader Mode,", Environment.NewLine, _
                        "a ~ symbol will be appended to the serial number")
                    miscModuleSerialNumberToolTip.SetToolTip(miscGetModuleSerialNumberGroupBox, miscModuleSerialNumberToolTipText)
                    miscModuleSerialNumberToolTip.SetToolTip(miscGetModuleSerialNumberButton, miscModuleSerialNumberToolTipText)
                    miscModuleSerialNumberToolTip.SetToolTip(miscGetModuleSerialNumberBox, miscModuleSerialNumberToolTipText)
                End If                  ' If moduleSerialNumberGroupBoxDefined = True Then
                '-------------------------------------------------------------
                ' I²C data rate group box
                '-------------------------------------------------------------
                Dim miscGetSetI2CDataRateGroupBox As New GroupBox()
                Dim I2CDataRateGroupBoxDefined As Boolean = True
                '-------------------------------------------------------------
                If I2CDataRateGroupBoxDefined = True Then
                    miscGetSetI2CDataRateGroupBox.Text = "I² C Data Rate"
                    miscGetSetI2CDataRateGroupBox.Location = New Point( _
                        miscGetModuleSerialNumberGroupBox.Right + 10, _
                        miscGetModuleSerialNumberGroupBox.Top)
                    miscGetSetI2CDataRateGroupBox.Size = New Drawing.Size( _
                        160, miscGetModuleSerialNumberGroupBox.Height)
                    miscGetSetI2CDataRateGroupBox.BackColor = Color.Transparent
                    miscUnitGroupBox.Controls.Add(miscGetSetI2CDataRateGroupBox)
                    '---------------------------------------------------------
                    ' Get button
                    '---------------------------------------------------------
                    Dim miscGetI2CDataRateButton As New Button()
                    miscGetI2CDataRateButton.Text = "Get"
                    miscGetI2CDataRateButton.Location = New Point(6, 16)
                    miscGetI2CDataRateButton.Size = New Drawing.Size( _
                        GUI_GET_SET_BUTTON_WIDTH, _
                        GUI_REGULAR_BUTTON_HEIGHT)
                    QCOM_SetUnitObjectInterfaceProperties(miscGetI2CDataRateButton, unitNumber)
                    AddHandler miscGetI2CDataRateButton.Click, AddressOf QCOM_MiscUnitGetI2CDataRateButtonClicked
                    miscGetSetI2CDataRateGroupBox.Controls.Add(miscGetI2CDataRateButton)
                    '---------------------------------------------------------
                    ' Text box
                    '---------------------------------------------------------
                    Dim miscGetSetI2CDataRateBox As New TextBox()
                    miscGetSetI2CDataRateBox.Location = New Point( _
                        miscGetI2CDataRateButton.Right + 5, _
                        miscGetI2CDataRateButton.Top + 2)
                    miscGetSetI2CDataRateBox.Size = New Drawing.Size(40, GUI_REGULAR_TEXT_BOX_HEIGHT)
                    miscGetSetI2CDataRateBox.Multiline = False
                    miscGetSetI2CDataRateBox.AcceptsReturn = False
                    miscGetSetI2CDataRateBox.AcceptsTab = False
                    miscGetSetI2CDataRateBox.WordWrap = False
                    miscGetSetI2CDataRateBox.ReadOnly = False
                    miscGetSetI2CDataRateBox.TextAlign = HorizontalAlignment.Right
                    miscGetSetI2CDataRateBox.Cursor = Cursors.Hand
                    miscGetSetI2CDataRateBox.BackColor = Color.Lavender
                    miscGetSetI2CDataRateBoxArray(unitNumber) = miscGetSetI2CDataRateBox
                    miscGetSetI2CDataRateGroupBox.Controls.Add(miscGetSetI2CDataRateBox)
                    '---------------------------------------------------------
                    ' Units label
                    '---------------------------------------------------------
                    Dim miscGetSetI2CpInfoDataRateLabel As New Label()
                    miscGetSetI2CpInfoDataRateLabel.Text = "kHz"
                    miscGetSetI2CpInfoDataRateLabel.Location = New Point( _
                        miscGetSetI2CDataRateBox.Right + 2, _
                        miscGetSetI2CDataRateBox.Top + 3)
                    miscGetSetI2CpInfoDataRateLabel.Size = New Drawing.Size(26, GUI_REGULAR_LABEL_HEIGHT)
                    miscGetSetI2CpInfoDataRateLabel.BackColor = Color.Transparent
                    miscGetSetI2CDataRateGroupBox.Controls.Add(miscGetSetI2CpInfoDataRateLabel)
                    '---------------------------------------------------------
                    ' Set button
                    '---------------------------------------------------------
                    Dim miscSetI2CDataRateButton As New Button()
                    miscSetI2CDataRateButton.Text = "Set"
                    miscSetI2CDataRateButton.Location = New Point( _
                        120, miscGetI2CDataRateButton.Top)
                    miscSetI2CDataRateButton.Size = miscGetI2CDataRateButton.Size
                    QCOM_SetUnitObjectInterfaceProperties(miscSetI2CDataRateButton, unitNumber)
                    AddHandler miscSetI2CDataRateButton.Click, AddressOf QCOM_MiscUnitSetI2CDataRateButtonClicked
                    miscGetSetI2CDataRateGroupBox.Controls.Add(miscSetI2CDataRateButton)
                    '---------------------------------------------------------
                    ' Tool tip
                    '---------------------------------------------------------
                    Dim miscGetSetI2CDataRateToolTip As New ToolTip()
                    miscGetSetI2CDataRateToolTip.ShowAlways = True
                    miscGetSetI2CDataRateToolTip.AutoPopDelay = 10000
                    miscGetSetI2CDataRateToolTip.ToolTipTitle = "I²C Data Rate"
                    Dim miscGetSetI2CDataRateToolTipText As String = String.Concat( _
                        "Displays or sets the rate at which data is", Environment.NewLine, _
                        "transferred between the QCOM module", Environment.NewLine, _
                        "and the transducer on the I²C bus", Environment.NewLine, _
                        "Minimum rate = 33.3 kHz", Environment.NewLine, _
                        "Maximum rate = 100.0 kHz")
                    miscGetSetI2CDataRateToolTip.SetToolTip(miscGetSetI2CDataRateGroupBox, miscGetSetI2CDataRateToolTipText)
                    miscGetSetI2CDataRateToolTip.SetToolTip(miscGetI2CDataRateButton, miscGetSetI2CDataRateToolTipText)
                    miscGetSetI2CDataRateToolTip.SetToolTip(miscGetSetI2CDataRateBox, miscGetSetI2CDataRateToolTipText)
                    miscGetSetI2CDataRateToolTip.SetToolTip(miscGetSetI2CpInfoDataRateLabel, miscGetSetI2CDataRateToolTipText)
                    miscGetSetI2CDataRateToolTip.SetToolTip(miscSetI2CDataRateButton, miscGetSetI2CDataRateToolTipText)
                End If                  ' If I2CDataRateGroupBoxDefined = True Then
                '-------------------------------------------------------------
                ' Memory type group box
                '-------------------------------------------------------------
                Dim miscGetMemoryTypeGroupBox As New GroupBox()
                Dim memoryTypeGroupBoxDefined As Boolean = True
                '-------------------------------------------------------------
                If memoryTypeGroupBoxDefined = True Then
                    miscGetMemoryTypeGroupBox.Text = "Memory Type"
                    miscGetMemoryTypeGroupBox.Location = New Point( _
                        miscGetSetI2CDataRateGroupBox.Right + 10, _
                        miscGetSetI2CDataRateGroupBox.Top)
                    miscGetMemoryTypeGroupBox.Size = New Drawing.Size( _
                        90, miscGetSetI2CDataRateGroupBox.Height)
                    miscUnitGroupBox.Controls.Add(miscGetMemoryTypeGroupBox)
                    '---------------------------------------------------------
                    ' Get button
                    '---------------------------------------------------------
                    Dim miscGetMemoryTypeButton As New Button()
                    miscGetMemoryTypeButton.Text = "Get"
                    miscGetMemoryTypeButton.Location = New Point(6, 16)
                    miscGetMemoryTypeButton.Size = New Drawing.Size( _
                        GUI_GET_SET_BUTTON_WIDTH, _
                        GUI_REGULAR_BUTTON_HEIGHT)
                    QCOM_SetUnitObjectInterfaceProperties(miscGetMemoryTypeButton, unitNumber)
                    AddHandler miscGetMemoryTypeButton.Click, AddressOf QCOM_MiscUnitGetMemoryTypeButtonClicked
                    miscGetMemoryTypeGroupBox.Controls.Add(miscGetMemoryTypeButton)
                    '---------------------------------------------------------
                    ' Text box
                    '---------------------------------------------------------
                    Dim miscGetMemoryTypeBox As New TextBox()
                    miscGetMemoryTypeBox.Location = New Point( _
                        miscGetMemoryTypeButton.Right + 5, _
                        miscGetMemoryTypeButton.Top + 2)
                    miscGetMemoryTypeBox.Size = New Drawing.Size(40, GUI_REGULAR_TEXT_BOX_HEIGHT)
                    miscGetMemoryTypeBox.Multiline = False
                    miscGetMemoryTypeBox.AcceptsReturn = False
                    miscGetMemoryTypeBox.AcceptsTab = False
                    miscGetMemoryTypeBox.WordWrap = False
                    miscGetMemoryTypeBox.ReadOnly = True
                    miscGetMemoryTypeBox.TextAlign = HorizontalAlignment.Center
                    miscGetMemoryTypeBox.Cursor = Cursors.Default
                    miscGetMemoryTypeBox.BackColor = Color.Lavender
                    miscGetMemoryTypeBoxArray(unitNumber) = miscGetMemoryTypeBox
                    miscGetMemoryTypeGroupBox.Controls.Add(miscGetMemoryTypeBox)
                    '---------------------------------------------------------
                    ' Tool tip
                    '---------------------------------------------------------
                    Dim miscGetMemoryTypeToolTip As New ToolTip()
                    miscGetMemoryTypeToolTip.ShowAlways = True
                    miscGetMemoryTypeToolTip.AutoPopDelay = 10000
                    miscGetMemoryTypeToolTip.ToolTipTitle = "Memory Type"
                    Dim miscGetMemoryTypeToolTipText As String = String.Concat( _
                        "Displays the transducer memory type if a digital", Environment.NewLine, _
                        "transducer is attached to this module, or the QCOM", Environment.NewLine, _
                        "module memory type otherwise, as follows:", Environment.NewLine, _
                        "0x00 = No memory present", Environment.NewLine, _
                        "0x01 = FRAM type", Environment.NewLine, _
                        "0x02 = EEPROM type")
                    miscGetMemoryTypeToolTip.SetToolTip(miscGetMemoryTypeGroupBox, miscGetMemoryTypeToolTipText)
                    miscGetMemoryTypeToolTip.SetToolTip(miscGetMemoryTypeButton, miscGetMemoryTypeToolTipText)
                    miscGetMemoryTypeToolTip.SetToolTip(miscGetMemoryTypeBox, miscGetMemoryTypeToolTipText)
                End If                  ' If memoryTypeGroupBoxDefined = True Then
                '-------------------------------------------------------------
                ' Status register group box
                '-------------------------------------------------------------
                Dim miscGetStatusRegisterGroupBox As New GroupBox()
                Dim statusRegisterGroupBoxDefined As Boolean = True
                '-------------------------------------------------------------
                If statusRegisterGroupBoxDefined = True Then
                    miscGetStatusRegisterGroupBox.Text = "Status Register"
                    miscGetStatusRegisterGroupBox.Location = New Point( _
                        miscGetMemoryTypeGroupBox.Right + 10,
                        miscGetMemoryTypeGroupBox.Top)
                    miscGetStatusRegisterGroupBox.Size = New Drawing.Size( _
                        120, miscGetMemoryTypeGroupBox.Height)
                    miscGetStatusRegisterGroupBox.BackColor = Color.Transparent
                    miscUnitGroupBox.Controls.Add(miscGetStatusRegisterGroupBox)
                    '---------------------------------------------------------
                    ' Get button
                    '---------------------------------------------------------
                    Dim miscGetStatusRegisterButton As New Button()
                    miscGetStatusRegisterButton.Text = "Get"
                    miscGetStatusRegisterButton.Location = New Point(6, 16)
                    miscGetStatusRegisterButton.Size = New Drawing.Size( _
                        GUI_GET_SET_BUTTON_WIDTH, _
                        GUI_REGULAR_BUTTON_HEIGHT)
                    QCOM_SetUnitObjectInterfaceProperties(miscGetStatusRegisterButton, unitNumber)
                    AddHandler miscGetStatusRegisterButton.Click, AddressOf QCOM_MiscUnitGetStatusRegisterButtonClicked
                    miscGetStatusRegisterGroupBox.Controls.Add(miscGetStatusRegisterButton)
                    '---------------------------------------------------------
                    ' Text box
                    '---------------------------------------------------------
                    Dim miscGetStatusRegisterBox As New TextBox()
                    miscGetStatusRegisterBox.Location = New Point( _
                        miscGetStatusRegisterButton.Right + 5, _
                        miscGetStatusRegisterButton.Top + 2)
                    miscGetStatusRegisterBox.Size = New Drawing.Size(30, GUI_REGULAR_TEXT_BOX_HEIGHT)
                    miscGetStatusRegisterBox.Multiline = False
                    miscGetStatusRegisterBox.AcceptsReturn = False
                    miscGetStatusRegisterBox.AcceptsTab = False
                    miscGetStatusRegisterBox.WordWrap = False
                    miscGetStatusRegisterBox.ReadOnly = True
                    miscGetStatusRegisterBox.TextAlign = HorizontalAlignment.Center
                    miscGetStatusRegisterBox.Cursor = Cursors.Default
                    miscGetStatusRegisterBox.BackColor = Color.Lavender
                    miscGetStatusRegisterBoxArray(unitNumber) = miscGetStatusRegisterBox
                    miscGetStatusRegisterGroupBox.Controls.Add(miscGetStatusRegisterBox)
                    '---------------------------------------------------------
                    ' Tool tip
                    '---------------------------------------------------------
                    Dim miscStatusRegisterToolTip As New ToolTip()
                    miscStatusRegisterToolTip.ShowAlways = True
                    miscStatusRegisterToolTip.AutoPopDelay = 10000
                    miscStatusRegisterToolTip.ToolTipTitle = "Module Status Register Value Bitmap"
                    Dim miscStatusRegisterToolTipText As String = String.Concat( _
                        "Bit 7 : Module error detected", Environment.NewLine, _
                        "Bit 6 : High-side switch over-current or thermal shutdown detected", Environment.NewLine, _
                        "Bit 5 : Transducer power is enabled", Environment.NewLine, _
                        "Bit 4 : Transducer is in an over-current condition", Environment.NewLine, _
                        "Bit 3 : Transducer is present", Environment.NewLine, _
                        "Bits 2 - 0 : Transducer type, as follows:", Environment.NewLine, _
                        "        000 = Transducer is absent or has an unknown type", Environment.NewLine, _
                        "        001 = Frequency (analog) transducer", Environment.NewLine, _
                        "        010 = Pre-FireFly digital transducer, with memory", Environment.NewLine, _
                        "        011 = FireFly digital transducer, with Vref", Environment.NewLine, _
                        "        100 = FireFly digital transducer, without Vref", Environment.NewLine, _
                        "        101 = Digital transducer, without memory")
                    miscStatusRegisterToolTip.SetToolTip(miscGetStatusRegisterGroupBox, miscStatusRegisterToolTipText)
                    miscStatusRegisterToolTip.SetToolTip(miscGetStatusRegisterButton, miscStatusRegisterToolTipText)
                    miscStatusRegisterToolTip.SetToolTip(miscGetStatusRegisterBox, miscStatusRegisterToolTipText)
                End If                  ' statusRegisterGroupBoxDefined
                '-------------------------------------------------------------
                ' Cadence timer group box
                '-------------------------------------------------------------
                Dim miscGetSetCadenceTimerGroupBox As New GroupBox()
                Dim cadenceTimerGroupBoxDefined As Boolean = True
                '-------------------------------------------------------------
                If cadenceTimerGroupBoxDefined = True Then
                    miscGetSetCadenceTimerGroupBox.Text = "Cadence Timer"
                    miscGetSetCadenceTimerGroupBox.Location = New Point( _
                        miscGetStatusRegisterGroupBox.Right + 10, _
                        miscGetStatusRegisterGroupBox.Top)
                    miscGetSetCadenceTimerGroupBox.Size = New Drawing.Size( _
                        150, miscGetStatusRegisterGroupBox.Height)
                    miscGetSetCadenceTimerGroupBox.BackColor = Color.Transparent
                    miscUnitGroupBox.Controls.Add(miscGetSetCadenceTimerGroupBox)
                    '---------------------------------------------------------
                    ' Get button
                    '---------------------------------------------------------
                    Dim miscGetCadenceTimerButton As New Button()
                    miscGetCadenceTimerButton.Text = "Read"
                    miscGetCadenceTimerButton.Location = New Point(6, 16)
                    miscGetCadenceTimerButton.Size = New Drawing.Size( _
                        42, GUI_REGULAR_BUTTON_HEIGHT)
                    QCOM_SetUnitObjectInterfaceProperties(miscGetCadenceTimerButton, unitNumber)
                    AddHandler miscGetCadenceTimerButton.Click, AddressOf GUI_MiscUnitReadCadenceTimerButtonClicked
                    miscGetSetCadenceTimerGroupBox.Controls.Add(miscGetCadenceTimerButton)
                    '---------------------------------------------------------
                    ' Text box
                    '---------------------------------------------------------
                    Dim miscGetSetCadenceTimerBox As New TextBox()
                    miscGetSetCadenceTimerBox.Location = New Point( _
                        miscGetCadenceTimerButton.Right + 5, _
                        miscGetCadenceTimerButton.Top + 2)
                    miscGetSetCadenceTimerBox.Size = New Drawing.Size( _
                        32, GUI_REGULAR_TEXT_BOX_HEIGHT)
                    miscGetSetCadenceTimerBox.Multiline = False
                    miscGetSetCadenceTimerBox.AcceptsReturn = False
                    miscGetSetCadenceTimerBox.AcceptsTab = False
                    miscGetSetCadenceTimerBox.WordWrap = False
                    miscGetSetCadenceTimerBox.ReadOnly = False
                    miscGetSetCadenceTimerBox.TextAlign = HorizontalAlignment.Center
                    miscGetSetCadenceTimerBox.Cursor = Cursors.Hand
                    miscGetSetCadenceTimerBox.BackColor = Color.Lavender
                    miscGetSetCadenceTimerBoxArray(unitNumber) = miscGetSetCadenceTimerBox
                    miscGetSetCadenceTimerGroupBox.Controls.Add(miscGetSetCadenceTimerBox)
                    '---------------------------------------------------------
                    ' Units label
                    '---------------------------------------------------------
                    Dim miscGetSetCadenceTimerLabel As New Label()
                    miscGetSetCadenceTimerLabel.Text = "ms"
                    miscGetSetCadenceTimerLabel.Location = New Point( _
                        miscGetSetCadenceTimerBox.Right + 2, _
                        miscGetSetCadenceTimerBox.Top + 3)
                    miscGetSetCadenceTimerLabel.Size = New Drawing.Size(20, GUI_REGULAR_LABEL_HEIGHT)
                    miscGetSetCadenceTimerLabel.BackColor = Color.Transparent
                    miscGetSetCadenceTimerGroupBox.Controls.Add(miscGetSetCadenceTimerLabel)
                    '---------------------------------------------------------
                    ' Set button
                    '---------------------------------------------------------
                    Dim miscSetCadenceTimerButton As New Button()
                    miscSetCadenceTimerButton.Text = "Set"
                    miscSetCadenceTimerButton.Location = New Point( _
                        110, miscGetCadenceTimerButton.Top)
                    miscSetCadenceTimerButton.Size = New Drawing.Size( _
                        GUI_GET_SET_BUTTON_WIDTH, _
                        GUI_REGULAR_BUTTON_HEIGHT)
                    QCOM_SetUnitObjectInterfaceProperties(miscSetCadenceTimerButton, unitNumber)
                    AddHandler miscSetCadenceTimerButton.Click, AddressOf GUI_MiscUnitSetCadenceTimerButtonClicked
                    miscGetSetCadenceTimerGroupBox.Controls.Add(miscSetCadenceTimerButton)
                    '---------------------------------------------------------
                    ' Tool tip
                    '---------------------------------------------------------
                    Dim miscCadenceTimerToolTip As New ToolTip()
                    miscCadenceTimerToolTip.ShowAlways = True
                    miscCadenceTimerToolTip.AutoPopDelay = 10000
                    miscCadenceTimerToolTip.ToolTipTitle = "Cadence Timer"
                    Dim miscCadenceTimerToolTipText As String = String.Concat( _
                        "The cadence timer controls how often the", Environment.NewLine, _
                        "QCOM module queries the transducer to", Environment.NewLine, _
                        "keep the transducer's counts refreshed", Environment.NewLine, _
                        "Minimum = ", QD_MINIMUM_CADENCE_TIMER_MS, " ms", Environment.NewLine, _
                        "Maximum = ", QD_MAXIMUM_CADENCE_TIMER_MS, " ms", Environment.NewLine, _
                        "Default = ", QD_MAXIMUM_CADENCE_TIMER_MS, " ms")
                    miscCadenceTimerToolTip.SetToolTip(miscGetSetCadenceTimerGroupBox, miscCadenceTimerToolTipText)
                    miscCadenceTimerToolTip.SetToolTip(miscGetCadenceTimerButton, miscCadenceTimerToolTipText)
                    miscCadenceTimerToolTip.SetToolTip(miscGetSetCadenceTimerBox, miscCadenceTimerToolTipText)
                    miscCadenceTimerToolTip.SetToolTip(miscGetSetCadenceTimerLabel, miscCadenceTimerToolTipText)
                    miscCadenceTimerToolTip.SetToolTip(miscSetCadenceTimerButton, miscCadenceTimerToolTipText)
                End If                  ' If cadenceTimerGroupBoxDefined = True Then
                '-------------------------------------------------------------
                ' Send I²C command group box
                '-------------------------------------------------------------
                Dim miscSendI2CCommandGroupBox As New GroupBox()
                Dim sendI2CCommandGroupBoxDefined As Boolean = True
                '-------------------------------------------------------------
                If sendI2CCommandGroupBoxDefined = True Then
                    miscSendI2CCommandGroupBox.Text = "Send I² C Command"
                    miscSendI2CCommandGroupBox.Size = New Drawing.Size(410, 140)
                    QCOM_PositionBelow(miscSendI2CCommandGroupBox, miscGetModuleSerialNumberGroupBox, 10)
                    miscUnitGroupBox.Controls.Add(miscSendI2CCommandGroupBox)
                    '---------------------------------------------------------
                    ' Command label
                    '---------------------------------------------------------
                    Dim miscSendI2CCommandLabel As New Label()
                    miscSendI2CCommandLabel.Text = "Command:"
                    miscSendI2CCommandLabel.Location = New Point(10, 22)
                    miscSendI2CCommandLabel.Size = New Drawing.Size(60, GUI_REGULAR_LABEL_HEIGHT)
                    miscSendI2CCommandLabel.BackColor = Color.Transparent
                    miscSendI2CCommandGroupBox.Controls.Add(miscSendI2CCommandLabel)
                    '---------------------------------------------------------
                    ' Combo box
                    '---------------------------------------------------------
                    Dim miscSendI2CCommandCombo As New ComboBox()
                    miscSendI2CCommandCombo.Location = New Point( _
                        miscSendI2CCommandLabel.Right + 2, _
                        miscSendI2CCommandLabel.Top - 2)
                    miscSendI2CCommandCombo.Size = New Drawing.Size( _
                        miscSendI2CCommandGroupBox.Width - 130, _
                        GUI_REGULAR_COMBO_BOX_HEIGHT)
                    miscSendI2CCommandCombo.MaxDropDownItems = 8
                    miscSendI2CCommandCombo.FormattingEnabled = True
                    miscSendI2CCommandCombo.Cursor = Cursors.Hand
                    QCOM_SetUnitBoxInterfaceProperties(miscSendI2CCommandCombo, unitNumber)
                    AddHandler miscSendI2CCommandCombo.Click, AddressOf QCOM_MiscUnitClearI2CCommandBox
                    AddHandler miscSendI2CCommandCombo.PreviewKeyDown, AddressOf QCOM_ComboBoxAcceptEnterKey
                    AddHandler miscSendI2CCommandCombo.KeyDown, AddressOf QCOM_MiscUnitProcessI2CCommandBoxEnterKey
                    Dim transducerIsAnalog As Boolean = False
                    Dim transducerWithNoMemory As Boolean = False
                    Dim defaultI2CList() As String
                    Dim defaultI2CListWithChecksum() As String
                    '---------------------------------------------------------
                    ' Only populate the combo drop-down if the unit is valid
                    '---------------------------------------------------------
                    If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
                        If (unit.transducerType = QD_TRANSDUCER_TYPE_FREQUENCY) Then
                            transducerIsAnalog = True
                            transducerWithNoMemory = True
                        End If
                        If (unit.transducerType = QD_TRANSDUCER_TYPE_DIGITAL_NOMEM) Then
                            transducerWithNoMemory = True
                        End If
                        If (unit.transducerChipID(2) >= 4) AndAlso (unit.transducerChipID(3) >= 2) Then
                            '-------------------------------------------------
                            ' The ASIC is version 4.02 or later, so request
                            ' checksums
                            '-------------------------------------------------
                            defaultI2CListWithChecksum = _
                                { _
                                    "S9DFFFFFFFFFFP [ Read Pres w/chk ]", _
                                    "S9FFFFFFFFFFFP [ Read Temp w/chk ]", _
                                    "S9CR9DFFFFFFFFFFP [ Get chip ID w/chk ]", _
                                    "S9CR9FFFFFFFFFP [ Read XD status ]", _
                                    "S9DFFP [ Poll for ACK ]", _
                                    "SADFFFFFFP [ Read ASIC curr addr ]", _
                                    "SAC0000RADFFP [ Read XD mem ]", _
                                    "S9C1FP [ Unlock XD mem ]", _
                                    "S9C3FP [ Lock XD mem ]", _
                                    "SAC0000XXP [ Write XX to XD mem ]"
                                }
                            If transducerIsAnalog = True Then
                                defaultI2CListWithChecksum(0) = "S99FFFFFFFFFFP [ Read Pres w/chk ]"
                                defaultI2CListWithChecksum(1) = "S9BFFFFFFFFFFP [ Read Temp w/chk ]"
                                defaultI2CListWithChecksum(2) = "S98R99FFFFFFFFFFP [ Get chip ID w/chk ]"
                                defaultI2CListWithChecksum(3) = "S98R9BFFFFFFFFP [ Read XD status ]"
                                defaultI2CListWithChecksum(4) = "S99FFP [ Poll for ACK ]"
                            End If
                            If transducerWithNoMemory = True Then
                                defaultI2CListWithChecksum(5) = "SA3FFFFFFP [ Read QCOM curr addr ]"
                                defaultI2CListWithChecksum(6) = "SA20000RA3FFP [ Read QCOM mem ]"
                                defaultI2CListWithChecksum(7) = "S981FP [ Unlock QCOM mem ]"
                                defaultI2CListWithChecksum(8) = "S983FP [ Lock QCOM mem ]"
                                defaultI2CListWithChecksum(9) = "SA20000XXP [ Write XX to QCOM mem ]"
                            End If
                            miscSendI2CCommandCombo.Items.AddRange(defaultI2CListWithChecksum)
                            miscSendI2CCommandCombo.Text = defaultI2CListWithChecksum(0)
                        Else
                            '-------------------------------------------------
                            ' Do not request checksums
                            '-------------------------------------------------
                            defaultI2CList = _
                                { _
                                    "S9DFFFFFFFFP [ Read Pres w/o chk ]", _
                                    "S9FFFFFFFFFP [ Read Temp w/o chk ]", _
                                    "S9CR9DFFFFFFFFP [ Get chip ID w/o chk ]", _
                                    "S9CR9FFFFFP [ Read XD status ]", _
                                    "S9DFFP [ Poll for ACK ]", _
                                    "SADFFFFFFP [ Read ASIC curr addr ]", _
                                    "SAC0000RADFFP [ Read XD mem ]", _
                                    "S9C1FP [ Unlock XD mem ]", _
                                    "S9C3FP [ Lock XD mem ]", _
                                    "SAC0000XXP [ Write XX to XD mem ]"
                                }
                            If transducerIsAnalog = True Then
                                defaultI2CList(0) = "S99FFFFFFFFP [ Read Pres w/o chk ]"
                                defaultI2CList(1) = "S9BFFFFFFFFP [ Read Temp w/o chk ]"
                                defaultI2CList(2) = "S98R99FFFFFFFFP [ Get chip ID w/o chk ]"
                                defaultI2CList(3) = "S98R9BFFFFP [ Read XD status ]"
                                defaultI2CList(4) = "S99FFP [ Poll for ACK ]"
                            End If
                            If transducerWithNoMemory = True Then
                                defaultI2CList(5) = "SA3FFFFFFP [ Read QCOM curr addr ]"
                                defaultI2CList(6) = "SA20000RA3FFP [ Read QCOM mem ]"
                                defaultI2CList(7) = "S981FP [ Unlock QCOM mem ]"
                                defaultI2CList(8) = "S983FP [ Lock QCOM mem ]"
                                defaultI2CList(9) = "SA20000XXP [ Write XX to QCOM mem ]"
                            End If
                            miscSendI2CCommandCombo.Items.AddRange(defaultI2CList)
                            miscSendI2CCommandCombo.Text = defaultI2CList(0)
                        End If          ' If (unit.transducerChipID(2) >= 4) AndAlso ...
                    End If              ' If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
                    miscSendI2CCommandComboArray(unitNumber) = miscSendI2CCommandCombo
                    miscSendI2CCommandGroupBox.Controls.Add(miscSendI2CCommandCombo)
                    '---------------------------------------------------------
                    ' Send button
                    '---------------------------------------------------------
                    Dim miscSendI2CCommandButton As New Button()
                    miscSendI2CCommandButton.Text = "Send"
                    miscSendI2CCommandButton.Location = New Point(
                        miscSendI2CCommandGroupBox.Width - 50, _
                        miscSendI2CCommandLabel.Top - 5)
                    miscSendI2CCommandButton.Size = New Drawing.Size(40, GUI_REGULAR_BUTTON_HEIGHT)
                    QCOM_SetUnitObjectInterfaceProperties(miscSendI2CCommandButton, unitNumber)
                    AddHandler miscSendI2CCommandButton.Click, _
                        AddressOf QCOM_MiscUnitSendI2CCommandButtonClicked
                    miscSendI2CCommandGroupBox.Controls.Add(miscSendI2CCommandButton)
                    '---------------------------------------------------------
                    ' Response label
                    '---------------------------------------------------------
                    Dim miscSendI2CResponseLabel As New Label()
                    miscSendI2CResponseLabel.Text = "Response:"
                    miscSendI2CResponseLabel.Location = New Point( _
                        miscSendI2CCommandLabel.Left, _
                        miscSendI2CCommandCombo.Bottom + 4)
                    miscSendI2CResponseLabel.Size = New Drawing.Size(60, GUI_REGULAR_LABEL_HEIGHT)
                    miscSendI2CResponseLabel.BackColor = Color.Transparent
                    miscSendI2CCommandGroupBox.Controls.Add(miscSendI2CResponseLabel)
                    '---------------------------------------------------------
                    ' Response text box
                    '---------------------------------------------------------
                    Dim miscI2CResponseBox As New TextBox()
                    miscI2CResponseBox.Location = New Point( _
                        miscSendI2CCommandLabel.Left, _
                        miscSendI2CResponseLabel.Bottom + 2)
                    miscI2CResponseBox.Size = New Drawing.Size( _
                        miscSendI2CCommandGroupBox.Width - 20, _
                        miscSendI2CCommandGroupBox.Height - 70)
                    miscI2CResponseBox.MaxLength = 600
                    miscI2CResponseBox.Multiline = True
                    miscI2CResponseBox.ReadOnly = True
                    miscI2CResponseBox.ScrollBars = ScrollBars.Vertical
                    miscI2CResponseBox.AcceptsReturn = True
                    miscI2CResponseBox.AcceptsTab = True
                    miscI2CResponseBox.WordWrap = True
                    miscI2CResponseBox.BackColor = Color.Lavender
                    miscI2CResponseBox.Font = New Drawing.Font( _
                        FontFamily.GenericMonospace, _
                        8.0F, _
                        FontStyle.Regular)
                    miscI2CResponseBox.Cursor = Cursors.Default
                    miscI2CResponseBoxArray(unitNumber) = miscI2CResponseBox
                    miscSendI2CCommandGroupBox.Controls.Add(miscI2CResponseBox)
                    '---------------------------------------------------------
                    ' Tool Tip
                    '---------------------------------------------------------
                    Dim miscSendI2CCommandToolTip As New ToolTip()
                    miscSendI2CCommandToolTip.ShowAlways = True
                    miscSendI2CCommandToolTip.AutoPopDelay = 10000
                    miscSendI2CCommandToolTip.ToolTipTitle = "Send I²C Command"
                    Dim miscSendI2CCommandToolTipText As String = String.Concat( _
                        "Allows you to send an I²C command to the QCOM", Environment.NewLine, _
                        "module or the transducer, then displays the response.", Environment.NewLine, _
                        "The ER= response indicates the error number returned", Environment.NewLine, _
                        "from the I²C processor in the case of an error.", Environment.NewLine, _
                        "Responses ending in NP indicate an unknown error.")
                    miscSendI2CCommandToolTip.SetToolTip(miscSendI2CCommandGroupBox, miscSendI2CCommandToolTipText)
                    miscSendI2CCommandToolTip.SetToolTip(miscSendI2CCommandLabel, miscSendI2CCommandToolTipText)
                    miscSendI2CCommandToolTip.SetToolTip(miscSendI2CCommandCombo, miscSendI2CCommandToolTipText)
                    miscSendI2CCommandToolTip.SetToolTip(miscSendI2CCommandButton, miscSendI2CCommandToolTipText)
                    miscSendI2CCommandToolTip.SetToolTip(miscSendI2CResponseLabel, miscSendI2CCommandToolTipText)
                    miscSendI2CCommandToolTip.SetToolTip(miscI2CResponseBox, miscSendI2CCommandToolTipText)
                End If                  ' If sendI2CCommandGroupBoxDefined = True Then
                '-------------------------------------------------------------
                ' Control register group box
                '-------------------------------------------------------------
                Dim miscGetSetControlRegisterGroupBox As New GroupBox()
                Dim controlRegisterGroupBoxDefined As Boolean = True
                '-------------------------------------------------------------
                If controlRegisterGroupBoxDefined = True Then
                    miscGetSetControlRegisterGroupBox.Text = "Control Register"
                    QCOM_PositionAndSizeBelow( _
                        miscGetSetControlRegisterGroupBox, _
                        miscGetStatusRegisterGroupBox, 10)
                    miscUnitGroupBox.Controls.Add(miscGetSetControlRegisterGroupBox)
                    '---------------------------------------------------------
                    ' Get button
                    '---------------------------------------------------------
                    Dim miscGetControlRegisterButton As New Button()
                    miscGetControlRegisterButton.Text = "Get"
                    miscGetControlRegisterButton.Location = New Point(6, 16)
                    miscGetControlRegisterButton.Size = New Drawing.Size( _
                        GUI_GET_SET_BUTTON_WIDTH, _
                        GUI_REGULAR_BUTTON_HEIGHT)
                    QCOM_SetUnitObjectInterfaceProperties(miscGetControlRegisterButton, unitNumber)
                    AddHandler miscGetControlRegisterButton.Click, _
                        AddressOf GUI_MiscUnitGetControlRegisterButtonClicked
                    miscGetSetControlRegisterGroupBox.Controls.Add(miscGetControlRegisterButton)
                    '---------------------------------------------------------
                    ' Text box
                    '---------------------------------------------------------
                    Dim miscGetSetControlRegisterBox As New TextBox()
                    miscGetSetControlRegisterBox.Location = New Point( _
                        miscGetControlRegisterButton.Right + 5, _
                        miscGetControlRegisterButton.Top + 2)
                    miscGetSetControlRegisterBox.Size = New Drawing.Size( _
                        34, GUI_REGULAR_TEXT_BOX_HEIGHT)
                    miscGetSetControlRegisterBox.Multiline = False
                    miscGetSetControlRegisterBox.AcceptsReturn = False
                    miscGetSetControlRegisterBox.AcceptsTab = False
                    miscGetSetControlRegisterBox.WordWrap = False
                    miscGetSetControlRegisterBox.TextAlign = HorizontalAlignment.Center
                    miscGetSetControlRegisterBox.ReadOnly = False
                    miscGetSetControlRegisterBox.Cursor = Cursors.Hand
                    miscGetSetControlRegisterBox.BackColor = Color.Lavender
                    miscGetSetControlRegisterBoxArray(unitNumber) = miscGetSetControlRegisterBox
                    miscGetSetControlRegisterGroupBox.Controls.Add(miscGetSetControlRegisterBox)
                    '---------------------------------------------------------
                    ' Set button
                    '---------------------------------------------------------
                    Dim miscSetControlRegisterButton As New Button()
                    miscSetControlRegisterButton.Text = "Set"
                    miscSetControlRegisterButton.Location = New Point( _
                        miscGetSetControlRegisterBox.Right + 7,
                        miscGetControlRegisterButton.Top)
                    miscSetControlRegisterButton.Size = miscGetControlRegisterButton.Size
                    QCOM_SetUnitObjectInterfaceProperties(miscSetControlRegisterButton, unitNumber)
                    AddHandler miscSetControlRegisterButton.Click, _
                        AddressOf GUI_MiscUnitSetControlRegisterButtonClicked
                    miscGetSetControlRegisterGroupBox.Controls.Add(miscSetControlRegisterButton)
                    '---------------------------------------------------------
                    ' Tool Tip
                    '---------------------------------------------------------
                    Dim miscControlRegisterToolTip As New ToolTip()
                    miscControlRegisterToolTip.ShowAlways = True
                    miscControlRegisterToolTip.AutoPopDelay = 10000
                    miscControlRegisterToolTip.ToolTipTitle = _
                        "Module Control Register Value Bitmap"
                    Dim miscControlRegisterToolTipText = String.Concat( _
                        "Bits 7 - 2 : Reserved", Environment.NewLine, _
                        "Bit 1 : Automatically copy the coefficient data of the newly", Environment.NewLine, _
                        "           attached digital transducer into QCOM module memory", Environment.NewLine, _
                        "Bit 0 : Reserved")
                    miscControlRegisterToolTip.SetToolTip(miscGetSetControlRegisterGroupBox, miscControlRegisterToolTipText)
                    miscControlRegisterToolTip.SetToolTip(miscGetControlRegisterButton, miscControlRegisterToolTipText)
                    miscControlRegisterToolTip.SetToolTip(miscGetSetControlRegisterBox, miscControlRegisterToolTipText)
                    miscControlRegisterToolTip.SetToolTip(miscSetControlRegisterButton, miscControlRegisterToolTipText)
                End If                  ' If controlRegisterGroupBoxDefined = True Then
                '-------------------------------------------------------------
                ' Module firmware ID group box
                '-------------------------------------------------------------
                Dim miscGetModuleFirmwareIDGroupBox As New GroupBox()
                Dim moduleFirmwareIDGroupBoxDefined As Boolean = True
                '-------------------------------------------------------------
                If moduleFirmwareIDGroupBoxDefined = True Then
                    miscGetModuleFirmwareIDGroupBox.Text = "Module Firmware ID"
                    QCOM_PositionAndSizeBelow( _
                        miscGetModuleFirmwareIDGroupBox, _
                        miscGetSetCadenceTimerGroupBox, 10)
                    miscUnitGroupBox.Controls.Add(miscGetModuleFirmwareIDGroupBox)
                    '---------------------------------------------------------
                    ' Get button
                    '---------------------------------------------------------
                    Dim miscGetModuleFirmwareIDButton As New Button()
                    miscGetModuleFirmwareIDButton.Text = "Get"
                    miscGetModuleFirmwareIDButton.Location = New Point(6, 16)
                    miscGetModuleFirmwareIDButton.Size = New Drawing.Size( _
                        GUI_GET_SET_BUTTON_WIDTH, _
                        GUI_REGULAR_BUTTON_HEIGHT)
                    QCOM_SetUnitObjectInterfaceProperties(miscGetModuleFirmwareIDButton, unitNumber)
                    AddHandler miscGetModuleFirmwareIDButton.Click, _
                        AddressOf GUI_MiscUnitGetModuleFirmwareIDButtonClicked
                    miscGetModuleFirmwareIDGroupBox.Controls.Add(miscGetModuleFirmwareIDButton)
                    '---------------------------------------------------------
                    ' Text box
                    '---------------------------------------------------------
                    Dim miscGetModuleFirmwareIDBox As New TextBox()
                    miscGetModuleFirmwareIDBox.Location = New Point( _
                        miscGetModuleFirmwareIDButton.Right + 5, _
                        miscGetModuleFirmwareIDButton.Top + 2)
                    miscGetModuleFirmwareIDBox.Size = New Drawing.Size( _
                        80, GUI_REGULAR_TEXT_BOX_HEIGHT)
                    miscGetModuleFirmwareIDBox.Multiline = False
                    miscGetModuleFirmwareIDBox.AcceptsReturn = False
                    miscGetModuleFirmwareIDBox.AcceptsTab = False
                    miscGetModuleFirmwareIDBox.WordWrap = False
                    miscGetModuleFirmwareIDBox.ReadOnly = True
                    miscGetModuleFirmwareIDBox.TextAlign = HorizontalAlignment.Center
                    miscGetModuleFirmwareIDBox.BackColor = Color.Lavender
                    miscGetModuleFirmwareIDBox.Cursor = Cursors.Default
                    miscGetModuleFirmwareIDBoxArray(unitNumber) = miscGetModuleFirmwareIDBox
                    miscGetModuleFirmwareIDGroupBox.Controls.Add(miscGetModuleFirmwareIDBox)
                    '---------------------------------------------------------
                    ' Tool Tip
                    '---------------------------------------------------------
                    Dim miscGetModuleFirmwareIDToolTip As New ToolTip()
                    miscGetModuleFirmwareIDToolTip.ShowAlways = True
                    miscGetModuleFirmwareIDToolTip.AutoPopDelay = 10000
                    miscGetModuleFirmwareIDToolTip.ToolTipTitle = "Module Firmware ID Bytes"
                    Dim miscGetModuleFirmwareIDToolTipText As String = String.Concat( _
                        "0D = Produced by Quartzdyne", Environment.NewLine, _
                        "16 = QCOM firmware", Environment.NewLine, _
                        "01 = Major version number", Environment.NewLine, _
                        "02 = Minor version number")
                    miscGetModuleFirmwareIDToolTip.SetToolTip(miscGetModuleFirmwareIDGroupBox, miscGetModuleFirmwareIDToolTipText)
                    miscGetModuleFirmwareIDToolTip.SetToolTip(miscGetModuleFirmwareIDButton, miscGetModuleFirmwareIDToolTipText)
                    miscGetModuleFirmwareIDToolTip.SetToolTip(miscGetModuleFirmwareIDBox, miscGetModuleFirmwareIDToolTipText)
                End If                  ' If moduleFirmwareIDGroupBoxDefined = True Then
                '-------------------------------------------------------------
                ' Transducer type group box
                '-------------------------------------------------------------
                Dim miscGetTransducerTypeGroupBox As New GroupBox()
                Dim transducerTypeGroupBoxDefined As Boolean = True
                '-------------------------------------------------------------
                If transducerTypeGroupBoxDefined = True Then
                    miscGetTransducerTypeGroupBox.Text = "Transducer Type"
                    QCOM_PositionAndSizeBelow( _
                        miscGetTransducerTypeGroupBox, _
                        miscGetSetControlRegisterGroupBox, 10)
                    miscUnitGroupBox.Controls.Add(miscGetTransducerTypeGroupBox)
                    '---------------------------------------------------------
                    ' Get button
                    '---------------------------------------------------------
                    Dim miscGetTransdcuerTypeButton As New Button()
                    miscGetTransdcuerTypeButton.Text = "Get"
                    miscGetTransdcuerTypeButton.Location = New Point(6, 16)
                    miscGetTransdcuerTypeButton.Size = New Drawing.Size( _
                        GUI_GET_SET_BUTTON_WIDTH, _
                        GUI_REGULAR_BUTTON_HEIGHT)
                    QCOM_SetUnitObjectInterfaceProperties(miscGetTransdcuerTypeButton, unitNumber)
                    AddHandler miscGetTransdcuerTypeButton.Click, AddressOf GUI_MiscUnitGetTransducerTypeButtonClicked
                    miscGetTransducerTypeGroupBox.Controls.Add(miscGetTransdcuerTypeButton)
                    '---------------------------------------------------------
                    ' Text box
                    '---------------------------------------------------------
                    Dim miscGetTransducerTypeBox As New TextBox()
                    miscGetTransducerTypeBox.Location = New Point( _
                        miscGetTransdcuerTypeButton.Right + 5, _
                        miscGetTransdcuerTypeButton.Top + 2)
                    miscGetTransducerTypeBox.Size = New Drawing.Size( _
                        40, GUI_REGULAR_TEXT_BOX_HEIGHT)
                    miscGetTransducerTypeBox.Multiline = False
                    miscGetTransducerTypeBox.AcceptsReturn = False
                    miscGetTransducerTypeBox.AcceptsTab = False
                    miscGetTransducerTypeBox.WordWrap = False
                    miscGetTransducerTypeBox.ReadOnly = True
                    miscGetTransducerTypeBox.TextAlign = HorizontalAlignment.Center
                    miscGetTransducerTypeBox.Cursor = Cursors.Default
                    miscGetTransducerTypeBox.BackColor = Color.Lavender
                    miscGetTransducerTypeBoxArray(unitNumber) = miscGetTransducerTypeBox
                    miscGetTransducerTypeGroupBox.Controls.Add(miscGetTransducerTypeBox)
                    '---------------------------------------------------------
                    ' Tool Tip
                    '---------------------------------------------------------
                    Dim miscGetTransducerTypeToolTip As New ToolTip()
                    miscGetTransducerTypeToolTip.ShowAlways = True
                    miscGetTransducerTypeToolTip.AutoPopDelay = 10000
                    miscGetTransducerTypeToolTip.ToolTipTitle = "Attached Transducer Type"
                    Dim miscGetTransducerTypeToolTipText As String = String.Concat( _
                        "0x00 = Transducer is absent", Environment.NewLine, _
                        "0x01 = Frequency (analog) transducer", Environment.NewLine, _
                        "0x02 = Pre-FireFly digital transducer, with memory", Environment.NewLine, _
                        "0x03 = FireFly digital transducer, with Vref", Environment.NewLine, _
                        "0x04 = FireFly digital transducer, without Vref", Environment.NewLine, _
                        "0x05 = Digital transducer, with no memory")
                    miscGetTransducerTypeToolTip.SetToolTip(miscGetTransducerTypeGroupBox, miscGetTransducerTypeToolTipText)
                    miscGetTransducerTypeToolTip.SetToolTip(miscGetTransdcuerTypeButton, miscGetTransducerTypeToolTipText)
                    miscGetTransducerTypeToolTip.SetToolTip(miscGetTransducerTypeBox, miscGetTransducerTypeToolTipText)
                End If                  ' If transducerTypeGroupBoxDefined = True Then
                '-------------------------------------------------------------
                ' Transducer chip ID group box
                '-------------------------------------------------------------
                Dim miscGetTransducerChipIDGroupBox As New GroupBox()
                Dim transducerChipIDGroupBoxDefined As Boolean = True
                '-------------------------------------------------------------
                If transducerChipIDGroupBoxDefined = True Then
                    miscGetTransducerChipIDGroupBox.Text = "Transducer Chip ID"
                    QCOM_PositionAndSizeBelow( _
                        miscGetTransducerChipIDGroupBox, _
                        miscGetModuleFirmwareIDGroupBox, 10)
                    miscUnitGroupBox.Controls.Add(miscGetTransducerChipIDGroupBox)
                    '---------------------------------------------------------
                    ' Get button
                    '---------------------------------------------------------
                    Dim miscGetTransdcuerChipIDButton As New Button()
                    miscGetTransdcuerChipIDButton.Text = "Get"
                    miscGetTransdcuerChipIDButton.Location = New Point(6, 16)
                    miscGetTransdcuerChipIDButton.Size = New Drawing.Size( _
                        GUI_GET_SET_BUTTON_WIDTH, _
                        GUI_REGULAR_BUTTON_HEIGHT)
                    QCOM_SetUnitObjectInterfaceProperties(miscGetTransdcuerChipIDButton, unitNumber)
                    AddHandler miscGetTransdcuerChipIDButton.Click, _
                        AddressOf GUI_MiscUnitGetTransducerChipIDButtonClicked
                    miscGetTransducerChipIDGroupBox.Controls.Add(miscGetTransdcuerChipIDButton)
                    '---------------------------------------------------------
                    ' Text box
                    '---------------------------------------------------------
                    Dim miscGetTransducerChipIDBox As New TextBox()
                    miscGetTransducerChipIDBox.Location = New Point( _
                        miscGetTransdcuerChipIDButton.Right + 5, _
                        miscGetTransdcuerChipIDButton.Top + 2)
                    miscGetTransducerChipIDBox.Size = New Drawing.Size( _
                        80, GUI_REGULAR_TEXT_BOX_HEIGHT)
                    miscGetTransducerChipIDBox.Multiline = False
                    miscGetTransducerChipIDBox.AcceptsReturn = False
                    miscGetTransducerChipIDBox.AcceptsTab = False
                    miscGetTransducerChipIDBox.WordWrap = False
                    miscGetTransducerChipIDBox.ReadOnly = True
                    miscGetTransducerChipIDBox.TextAlign = HorizontalAlignment.Center
                    miscGetTransducerChipIDBox.BackColor = Color.Lavender
                    miscGetTransducerChipIDBox.Cursor = Cursors.Default
                    miscGetTransducerChipIDBoxArray(unitNumber) = miscGetTransducerChipIDBox
                    miscGetTransducerChipIDGroupBox.Controls.Add(miscGetTransducerChipIDBox)
                    '---------------------------------------------------------
                    ' Tool Tip
                    '---------------------------------------------------------
                    Dim miscGetTransducerChipIDToolTip As New ToolTip()
                    miscGetTransducerChipIDToolTip.ShowAlways = True
                    miscGetTransducerChipIDToolTip.AutoPopDelay = 10000
                    miscGetTransducerChipIDToolTip.ToolTipTitle = "Transducer Chip ID Bytes"
                    Dim miscGetTransducerChipIDToolTipText As String = String.Concat( _
                        "0D = Produced by Quartzdyne", Environment.NewLine, _
                        "XX = Transducer chip type", Environment.NewLine, _
                        "        02 = SMT FPGA", Environment.NewLine, _
                        "        05 = Hybrid FPGA", Environment.NewLine, _
                        "        09 = ASIC", Environment.NewLine, _
                        "04 = Major version number", Environment.NewLine, _
                        "02 = Minor version number")
                    miscGetTransducerChipIDToolTip.SetToolTip( _
                        miscGetTransducerChipIDGroupBox, _
                        miscGetTransducerChipIDToolTipText)
                    miscGetTransducerChipIDToolTip.SetToolTip( _
                        miscGetTransdcuerChipIDButton, _
                        miscGetTransducerChipIDToolTipText)
                    miscGetTransducerChipIDToolTip.SetToolTip( _
                        miscGetTransducerChipIDBox, _
                        miscGetTransducerChipIDToolTipText)
                End If                  ' If transducerChipIDGroupBoxDefined = True Then
                '-------------------------------------------------------------
                ' Import coefficient data button
                '-------------------------------------------------------------
                Dim miscImportCoefficientDataButton As New Button()
                Dim importCoefficientDataButtonDefined As Boolean = True
                '-------------------------------------------------------------
                If importCoefficientDataButtonDefined = True Then
                    miscImportCoefficientDataButton.Text = "Import Coefficient Data File"
                    miscImportCoefficientDataButton.Size = New Drawing.Size( _
                        160, GUI_REGULAR_BUTTON_HEIGHT)
                    QCOM_PositionBelow(miscImportCoefficientDataButton, miscSendI2CCommandGroupBox, 10)
                    QCOM_SetUnitObjectInterfaceProperties(miscImportCoefficientDataButton, unitNumber)
                    AddHandler miscImportCoefficientDataButton.Click, _
                        AddressOf QCOM_MiscUnitImportCoefficientDataButtonClicked
                    miscUnitGroupBox.Controls.Add(miscImportCoefficientDataButton)
                    '---------------------------------------------------------
                    ' Tool Tip
                    '---------------------------------------------------------
                    Dim miscImportCoefficientDataToolTip As New ToolTip()
                    miscImportCoefficientDataToolTip.ShowAlways = True
                    miscImportCoefficientDataToolTip.AutoPopDelay = 10000
                    miscImportCoefficientDataToolTip.ToolTipTitle = "Import Coefficient Data From a File"
                    Dim miscImportCoefficientDataToolTipText As String = String.Concat( _
                        "Allows you to load the coefficient data for a", Environment.NewLine, _
                        "transducer with serial number SNSNSN from the", Environment.NewLine, _
                        "file named SNSNSN.hex into the memory of your", Environment.NewLine, _
                        "QCOM module. If your transducer is a digital", Environment.NewLine, _
                        "type, you will be prompted to store the data", Environment.NewLine, _
                        "in its memory as well.", Environment.NewLine, _
                        Environment.NewLine, _
                        "Navigate to the appropriate folder, select the .hex", Environment.NewLine, _
                        "file to load, and click Open.")
                    miscImportCoefficientDataToolTip.SetToolTip( _
                        miscImportCoefficientDataButton, _
                        miscImportCoefficientDataToolTipText)
                End If                  ' If importCoefficientDataButtonDefined = True Then
                '-------------------------------------------------------------
                ' Boot Loader Mode button
                '-------------------------------------------------------------
                Dim miscBootLoaderModeButton As New Button()
                Dim bootLoaderModeButtonDefined As Boolean = True
                '-------------------------------------------------------------
                If bootLoaderModeButtonDefined = True Then
                    If (unit.flags And QCOM_UNIT_BOOT_LOADER_MODE) <> 0 Then
                        miscBootLoaderModeButton.Text = QCOM_BLM_RESET_STRING
                    Else
                        miscBootLoaderModeButton.Text = QCOM_BLM_ENTER_STRING
                    End If
                    miscBootLoaderModeButton.Location = New Point( _
                        miscImportCoefficientDataButton.Right + 10, _
                        miscImportCoefficientDataButton.Top)
                    miscBootLoaderModeButton.Size = miscImportCoefficientDataButton.Size
                    QCOM_SetUnitObjectInterfaceProperties(miscBootLoaderModeButton, unitNumber)
                    AddHandler miscBootLoaderModeButton.Click, _
                        AddressOf QCOM_MiscUnitBootLoaderModeButtonClicked
                    miscBootLoaderModeButtonArray(unitNumber) = miscBootLoaderModeButton
                    miscUnitGroupBox.Controls.Add(miscBootLoaderModeButton)
                    '---------------------------------------------------------
                    ' Tool Tip
                    '---------------------------------------------------------
                    Dim miscBootLoaderModeToolTip As New ToolTip()
                    miscBootLoaderModeToolTip.ShowAlways = True
                    miscBootLoaderModeToolTip.AutoPopDelay = 10000
                    miscBootLoaderModeToolTip.ToolTipTitle = "Boot Loader Mode"
                    Dim miscBootLoaderModeToolTipText As String = String.Concat( _
                        "Boot Loader Mode is a QCOM module state that allows", Environment.NewLine, _
                        "an update of its firmware. This button is provided", Environment.NewLine, _
                        "to test entry and exit from Boot Loader Mode or to", Environment.NewLine, _
                        "place your module in the desired state if it had been", Environment.NewLine, _
                        "left in Boot Loader Mode previously.")
                    miscBootLoaderModeToolTip.SetToolTip(miscBootLoaderModeButton, miscBootLoaderModeToolTipText)
                End If                  ' If bootLoaderModeButtonDefined = True Then
                '-------------------------------------------------------------
                ' Transducer power button
                '-------------------------------------------------------------
                Dim miscTransducerPowerButton As New Button()
                Dim transducerPowerButtonDefined As Boolean = True
                '-------------------------------------------------------------
                If transducerPowerButtonDefined = True Then
                    If (unit.flags And QCOM_UNIT_TRANSDUCER_POWER_ENABLED) <> 0 Then
                        miscTransducerPowerButton.Text = QCOM_XD_POWER_DISABLE_STRING
                    Else
                        miscTransducerPowerButton.Text = QCOM_XD_POWER_ENABLE_STRING
                    End If
                    miscTransducerPowerButton.Location = New Point( _
                        miscBootLoaderModeButton.Right + 10, _
                        miscBootLoaderModeButton.Top)
                    miscTransducerPowerButton.Size = miscBootLoaderModeButton.Size
                    QCOM_SetUnitObjectInterfaceProperties(miscTransducerPowerButton, unitNumber)
                    AddHandler miscTransducerPowerButton.Click, _
                        AddressOf QCOM_MiscUnitTransducerPowerButtonClicked
                    miscTransducerPowerButtonArray(unitNumber) = miscTransducerPowerButton
                    miscUnitGroupBox.Controls.Add(miscTransducerPowerButton)
                    '---------------------------------------------------------
                    ' Tool Tip
                    '---------------------------------------------------------
                    Dim miscTransducerPowerToolTip As New ToolTip()
                    miscTransducerPowerToolTip.ShowAlways = True
                    miscTransducerPowerToolTip.AutoPopDelay = 10000
                    miscTransducerPowerToolTip.ToolTipTitle = "Transducer Power"
                    Dim miscTransducerPowerToolTipText As String = String.Concat( _
                        "Enables or disables the power supplied to the", Environment.NewLine, _
                        "transducer by the QCOM module. Disabling", Environment.NewLine, _
                        "power to the transducer will prevent any I²C", Environment.NewLine, _
                        "communications.")
                    miscTransducerPowerToolTip.SetToolTip(miscTransducerPowerButton, miscTransducerPowerToolTipText)
                End If                  ' If transducerPowerButtonDefined = True Then
            End If                      ' If unitGroupBoxDefined = True Then
        End If                          ' If unitTabPageDefined = True Then
        Return Nothing
    End Function                        ' QCOM_SetUpUnitMiscControls
    '-------------------------------------------------------------------------
    ' QCOM_ShutDownSoftware
    '
    ' Closes all open device handles, stops running processes and timers,
    ' releases allocated resources, and terminates the software
    '
    ' Called by:    QCOM_DefaultExceptionHandler
    '               QCOM_HomeClosingWindow
    '-------------------------------------------------------------------------
    Public Function QCOM_ShutDownSoftware()
        If (QCOM_GeneralInfo.flags And QCOM_GENERAL_CURRENTLY_SAMPLING) <> 0 Then
            '-----------------------------------------------------------------
            ' Stop all sampling
            '-----------------------------------------------------------------
            QCOM_GeneralInfo.flags = (QCOM_GeneralInfo.flags And Not QCOM_GENERAL_CURRENTLY_SAMPLING)
            '-----------------------------------------------------------------
            ' Shut down the sampling timer
            '-----------------------------------------------------------------
            If samplingTimer.Enabled = True Then
                samplingTimer.Stop()
            End If
        End If
        '---------------------------------------------------------------------
        ' Shut down the one-second timer
        '---------------------------------------------------------------------
        If oneSecondTimer.Enabled = True Then
            oneSecondTimer.Stop()
        End If
        '---------------------------------------------------------------------
        ' Save the config file and release resources
        '---------------------------------------------------------------------
        QCOM_Finalize()
        Return Nothing
    End Function                        ' QCOM_ShutDownSoftware
    '-------------------------------------------------------------------------
    ' QCOM_StringWidth
    '
    ' Returns the width of the specified string in pixels
    '-------------------------------------------------------------------------
    Public Function QCOM_StringWidth(ByVal textString As String) As UInt32
        Dim widthInPixels As UInt32 = 0
        '---------------------------------------------------------------------
        If textString.Length > 0 Then
            Dim numberOfPixels As Drawing.Size = New Drawing.Size( _
                TextRenderer.MeasureText( _
                    textString, _
                    New Drawing.Font(Me.Font, FontStyle.Regular)))
            widthInPixels = numberOfPixels.Width
        End If                          ' If textString.Length > 0 Then
        Return widthInPixels
    End Function                        ' QCOM_StringWidth
    '-------------------------------------------------------------------------
    ' QCOM_SwitchFromBootLoaderMode
    '
    ' Switches the specified unit to Application Mode
    '
    ' Called by:    QCOM_CheckModuleRegistryEntry
    '               QCOM_MiscUnitBootLoaderModeButtonClicked
    '               QCOM_ScanForDevices
    '-------------------------------------------------------------------------
    Public Function QCOM_SwitchFromBootLoaderMode(ByRef unit As UnitInfo)
        Dim waited As UInt32 = 0
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
            status = QD_SetFirmwareMode(unit.unitHandle, QD_SET_APPLICATION_MODE)
            If status = QCOM_SUCCESS Then
                Thread.Sleep(QD_MINIMUM_BOOT_LOADER_MODE_WAIT_TIME)
                QCOM_CloseUnit(unit)
                status = QCOM_OpenUnit(unit)
                If status = QCOM_SUCCESS Then
                    If unit.unitHandle <> INVALID_HANDLE_VALUE Then
                        QCOM_GetModuleInfo(unit)
                        While (unit.flags And QCOM_UNIT_BOOT_LOADER_MODE) <> 0 AndAlso _
                            (waited < QD_MINIMUM_BOOT_LOADER_MODE_WAIT_TIME)
                            If (unit.flags And QCOM_UNIT_BOOT_LOADER_MODE) <> 0 Then
                                waited += 1
                                Thread.Sleep(10)
                                QCOM_GetModuleInfo(unit)
                            End If
                        End While
                        If waited >= QD_MINIMUM_BOOT_LOADER_MODE_WAIT_TIME Then
                            EMsg("The transition from Boot Loader Mode timed out")
                        End If
                    End If              ' If unit.unitHandle <> INVALID_HANDLE_VALUE Then
                End If                  ' If status = QCOM_SUCCESS Then
            End If                      ' If status = QCOM_SUCCESS Then
        End If                          ' If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
        Return Nothing
    End Function                        ' QCOM_SwitchFromBootLoaderMode
    '-------------------------------------------------------------------------
    ' QCOM_SwitchToBootLoaderMode
    '
    ' Switches the specified unit to Boot Loader Mode
    '
    ' Called by:    QCOM_CheckModuleRegistryEntry
    '               QCOM_MiscUnitBootLoaderModeButtonClicked
    '-------------------------------------------------------------------------
    Public Function QCOM_SwitchToBootLoaderMode(ByRef unit As UnitInfo)
        Dim waited As UInt32 = 0
        Dim status As UInt32 = QCOM_SUCCESS
        '---------------------------------------------------------------------
        If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
            unit.flags = (unit.flags Or QCOM_UNIT_BOOT_LOADER_MODE)
            status = QD_SetFirmwareMode(unit.unitHandle, QD_SET_BOOT_LOADER_MODE)
            If status = QCOM_SUCCESS Then
                Thread.Sleep(QD_MINIMUM_BOOT_LOADER_MODE_WAIT_TIME)
                QCOM_CloseUnit(unit)
                status = QCOM_OpenUnit(unit)
                If status = QCOM_SUCCESS Then
                    If unit.unitHandle <> INVALID_HANDLE_VALUE Then
                        QCOM_GetModuleInfo(unit)
                        While (unit.flags And QCOM_UNIT_BOOT_LOADER_MODE) = 0 AndAlso _
                            (waited < QD_MINIMUM_BOOT_LOADER_MODE_WAIT_TIME)
                            If (unit.flags And QCOM_UNIT_BOOT_LOADER_MODE) = 0 Then
                                waited += 1
                                Thread.Sleep(10)
                                QCOM_GetModuleInfo(unit)
                            End If
                        End While
                        If waited >= QD_MINIMUM_BOOT_LOADER_MODE_WAIT_TIME Then
                            EMsg("The transition to Boot Loader Mode timed out")
                        End If
                    End If              ' If unit.unitHandle <> INVALID_HANDLE_VALUE Then
                End If                  ' If status = QCOM_SUCCESS Then
            End If                      ' If status = QCOM_SUCCESS Then
        End If                          ' If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
        Return Nothing
    End Function                        ' QCOM_SwitchToBootLoaderMode
    '-------------------------------------------------------------------------
    ' QCOM_TimeElapsed
    '
    ' Determines the amount of time that has elapsed in days, hours, minutes,
    ' seconds, and milliseconds from the total time in milliseconds elapsed
    '
    ' Called by:    QCOM_DisplayProgramInformation
    '               QCOM_UpdateElapsedTime
    '-------------------------------------------------------------------------
    Public Function QCOM_TimeElapsed( _
        ByVal lapsedTimeMilliseconds As Integer, _
        ByRef lapsedDays As Integer, _
        ByRef lapsedHours As Integer, _
        ByRef lapsedMinutes As Integer, _
        ByRef lapsedSeconds As Integer, _
        ByRef lapsedMilliseconds As Integer)
        '---------------------------------------------------------------------
        lapsedDays = 0
        lapsedHours = 0
        lapsedMinutes = 0
        lapsedSeconds = 0
        While lapsedTimeMilliseconds >= 86400000
            lapsedDays += 1
            lapsedTimeMilliseconds -= 86400000
        End While
        While lapsedTimeMilliseconds >= 3600000
            lapsedHours += 1
            lapsedTimeMilliseconds -= 3600000
        End While
        While lapsedTimeMilliseconds >= 60000
            lapsedMinutes += 1
            lapsedTimeMilliseconds -= 60000
        End While
        While lapsedTimeMilliseconds >= 1000
            lapsedSeconds += 1
            lapsedTimeMilliseconds -= 1000
        End While
        lapsedMilliseconds = lapsedTimeMilliseconds
        Return Nothing
    End Function                        ' QCOM_TimeElapsed
    '-------------------------------------------------------------------------
    ' QCOM_UpdateElapsedTime
    '
    ' Updates the Elapsed Time display on the home window with the elapsed
    ' time calculated since the start of the most recent sampling
    '
    ' Called by:    QCOM_InstallHomeWindowObjects
    '               QCOM_OneSecondTimerElapsed
    '-------------------------------------------------------------------------
    Public Function QCOM_UpdateElapsedTime()
        Dim lapsedDays As Integer = 0
        Dim lapsedHours As Integer = 0
        Dim lapsedMinutes As Integer = 0
        Dim lapsedSeconds As Integer = 0
        Dim lapsedMilliseconds As Integer = 0
        Dim dateTimes As DateTime = DateTime.Now
        '---------------------------------------------------------------------
        QCOM_TimeElapsed( _
            GetTickCount() - QCOM_GeneralInfo.samplingStartTime, _
            lapsedDays, lapsedHours, _
            lapsedMinutes, lapsedSeconds, lapsedMilliseconds)
        elapsedTimeLabel.Text = String.Format( _
            "Elapsed Sampling Time: {0:D} d {1:D} h {2:D} m {3:D} s", _
            lapsedDays, _
            lapsedHours, _
            lapsedMinutes, _
            lapsedSeconds)
        Return Nothing
    End Function                        ' QCOM_UpdateElapsedTime
    '-------------------------------------------------------------------------
    ' QCOM_UpdateHomeProgressBar
    '
    ' Updates the home progress bar with the specified whole number
    ' percentage, calculated from the numbers of items completed out of the
    ' items to complete
    '-------------------------------------------------------------------------
    Public Function QCOM_UpdateHomeProgressBar( _
        ByVal numberOfItemsCompleted As UInt32, _
        ByVal numberOfItemsToComplete As UInt32)
        Dim percentComplete As Integer = 0
        '---------------------------------------------------------------------
        If (QCOM_GeneralInfo.flags And QCOM_GENERAL_STATUS_LINE_AVAILABLE) <> 0 Then
            If numberOfItemsToComplete <> 0 Then
                percentComplete = QCOM_CalculatePercentage( _
                    numberOfItemsCompleted, _
                    numberOfItemsToComplete)
            End If                      ' If numberOfItemsToComplete <> 0 Then
            If (percentComplete <= 100) AndAlso (numberOfItemsCompleted <= numberOfItemsToComplete) Then
                homeTSProgressBar.Value = percentComplete
                homeStatusStrip.Update()
                Thread.Sleep(10)
            Else
                EMsg("Home progress bar percentage out of range")
            End If                      ' If (percentComplete <= 100) AndAlso ...
        End If                          ' If (QCOM_GeneralInfo.flags And QCOM_GENERAL_STATUS_LINE_AVAILABLE) <> 0 Then
        Return Nothing
    End Function                        ' QCOM_UpdateHomeProgressBar
    '-------------------------------------------------------------------------
    ' QCOM_UpdateStatusLine
    '
    ' Updates the status strip message box with the specified managed string
    '-------------------------------------------------------------------------
    Public Function QCOM_UpdateStatusLine(ByVal statusString As String)
        If (QCOM_GeneralInfo.flags And QCOM_GENERAL_STATUS_LINE_AVAILABLE) <> 0 Then
            If statusString.Length <> 0 Then
                homeStatusLineTSSLabel.Text = statusString
                homeStatusStrip.Update()
            Else
                homeStatusLineTSSLabel.Text = Nothing
            End If
        End If                          ' If (QCOM_GeneralInfo.flags And QCOM_GENERAL_STATUS_LINE_AVAILABLE) <> 0 Then
        Return Nothing
    End Function                        ' QCOM_UpdateStatusLine
    '-------------------------------------------------------------------------
    ' QCOM_UpdateReadout
    '
    ' Updates the pressure and temperature text boxes with the current values
    ' stored in the unit structure
    '
    ' Called by:    QCOM_ConstructUnitReadingsDisplay
    '               QCOM_SamplingTimerElapsed
    '-------------------------------------------------------------------------
    Public Function QCOM_UpdateReadout(ByRef unit As UnitInfo)
        Dim unitNumber As UInt32 = unit.unitNumber
        '---------------------------------------------------------------------
        If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
            pressureBoxArray(unitNumber).Text = _
                String.Format("{0:F3}", unit.pressureValuePSI)
            temperatureBoxArray(unitNumber).Text = _
                String.Format("{0:F3}", unit.temperatureValueCelsius)
            pressureCountBoxArray(unitNumber).Text = String.Concat(unit.pressureCount)
            temperatureCountBoxArray(unitNumber).Text = String.Concat(unit.temperatureCount)
            pressureFreqBoxArray(unitNumber).Text = _
                String.Format("{0:F3}", unit.pressureCount * QCOM_FREQUENCY_MULTIPLIER)
            temperatureFreqBoxArray(unitNumber).Text = _
                String.Format("{0:F3}", unit.temperatureCount * QCOM_FREQUENCY_MULTIPLIER)
            voltageBoxArray(unitNumber).Text = _
                String.Format("{0:F2}", unit.transducerVoltage)
            currentBoxArray(unitNumber).Text = _
                String.Format("{0:F2}", unit.transducerCurrent)
        End If                          ' If (unit.flags And QCOM_UNIT_VALID) <> 0 Then
        Return Nothing
    End Function                        ' QCOM_UpdateReadout
End Class                               ' QCOM_Form
'=============================================================================
' End of Utility.cpp
'=============================================================================
'